#ifndef BATIMENT_CPP
#define BATIMENT_CPP

#include "batiment.hpp"

#include <sstream>
#include <cmath>
#include <cstring>

#include "gestionnaire.hpp"
#include "monde.hpp"
#include "fonctions.hpp"
#include "personnage.hpp"

#define method(class, name) {#name, &class::name}

namespace GS
{
	const char Batiment::className[] = "Batiment";
	Lunar<Batiment>::RegType Batiment::methods[] =
		{
			{0, 0}
		};
	
	Batiment::Batiment(GS::Etre* parent, TiXmlElement* element, const char* scriptFile) : Etre(parent, element)
	{
		mType = Types::Batiment;
		
		mConstruit = false;
		mX = 0;
		mY = 0;
		mVie = 0;
		mTypeBatiment = NULL;
		mListeElementsAFabriquer = NULL;
		
		mQuantitesRessourcesConstruction.resize(0);
		mAvancementsFabricationObjet.resize(0);
		mNombresObjetsFabriques.resize(0);
		
		ChargeLua(&mIA);
		if (strcmp(scriptFile,"") && luaL_dofile(mIA, scriptFile) != 0)
			CastErreur(lua_tostring(mIA, -1));
	}
	
	Batiment::Batiment(lua_State* lua) : Etre(lua)
	{
	}
	
	Batiment::~Batiment()
	{
		ListeElements* pointeurSuivant;
		ListeElements* pointeurActuel;
		
		if (mListeElementsAFabriquer)
		{
			pointeurActuel = mListeElementsAFabriquer;
			
			while (pointeurActuel->elementSuivant)
			{
				pointeurSuivant = pointeurActuel->elementSuivant;
				delete pointeurActuel;
				pointeurActuel = pointeurSuivant;
			}
		}
	}
	
	void
	Batiment::Charge()
	{
		if (mXml)
		{
			ChargeXmlValeur(mXml, "Abscisse", mX);
			ChargeXmlValeur(mXml, "Ordonnee", mY);
			
			unsigned int type = 0;
			ChargeXmlValeur(mXml, "Type", type);
			this->SetType(GestionnaireDonnees->GetTypeBatiment(type));
			
			ChargeXmlValeur(mXml, "Construit", mConstruit);
			ChargeXmlValeur(mXml, "Vie", mVie);
			
			TiXmlElement* quantitesRessources = mXml->FirstChildElement("Construction");
			if (quantitesRessources)
			{
				unsigned int ressource;
				
				quantitesRessources = quantitesRessources->FirstChildElement("Ressource");
				while (quantitesRessources)
				{
					if (ChargeXmlParametre(quantitesRessources, "valeur", ressource) && ressource <= mTypeBatiment->GetNombreTypesRessourcesNecessaires())
					{
						quantitesRessources = quantitesRessources->NextSiblingElement("Quantite");
						
						quantitesRessources->QueryDoubleAttribute("valeur", &mQuantitesRessourcesConstruction[ressource]);
					}
					
					quantitesRessources = quantitesRessources->NextSiblingElement("Ressource");
				}
			}
			
			std::string stringAvancements;
			ChargeXmlValeur(mXml, "AvancementsFabricationObjet", stringAvancements);
			const char* listeAvancements = stringAvancements.c_str();
			unsigned int longueurAvancements = strlen(listeAvancements);
			char caractereAvancements = 0;
			unsigned int rangAvancements = 0;
			
			std::string stringNombres;
			ChargeXmlValeur(mXml, "NombresObjetsFabriques", stringNombres);
			const char* listeNombres = stringNombres.c_str();
			unsigned int longueurNombres = strlen(listeNombres);
			char caractereNombres = 0;
			unsigned int rangNombres = 0;
			
			std::ostringstream identifiant;
			unsigned int l = mTypeBatiment->GetNombreObjetsFabricables();
			unsigned int i = 0;
			while (i < l)
			{
				identifiant.str("");
				while (rangAvancements < longueurAvancements && caractereAvancements != '/')
				{
					caractereAvancements = listeAvancements[rangAvancements];
					if (caractereAvancements != '/')
						identifiant << caractereAvancements;
					rangAvancements++;
				}
				caractereAvancements = 0;
				if (strcmp(identifiant.str().c_str(), ""))
				{
					double quantite = atof(identifiant.str().c_str());
					
					if (quantite > 0)
						mAvancementsFabricationObjet[i] = quantite;
				}
				
				identifiant.str("");
				while (rangNombres < longueurNombres && caractereNombres != '/')
				{
					caractereNombres = listeNombres[rangNombres];
					if (caractereNombres != '/')
						identifiant << caractereNombres;
					rangNombres++;
				}
				caractereNombres = 0;
				if (strcmp(identifiant.str().c_str(), ""))
				{
					double nombre = atoi(identifiant.str().c_str());
					
					if (nombre > 0)
						mNombresObjetsFabriques[i] = nombre;
				}
				
				i++;
			}
			
			TiXmlElement* listeElementXml = mXml->FirstChildElement("ListeElementsAFabriquer");
			TiXmlElement* elementXml = listeElementXml->FirstChildElement();
			ListeElements* elementCourant = NULL;
			mListeElementsAFabriquer = NULL;
			while (elementXml)
			{
				ListeElements* element = new ListeElements();
				
				if (strcmp(elementXml->Value(), "Objet"))
				{
					elementXml->QueryIntAttribute("id", &element->numRessource);
					elementXml->QueryDoubleAttribute("quantite", &element->quantite);
					element->numObjet = -1;
				}
				else
				{
					elementXml->QueryIntAttribute("id", &element->numObjet);
					element->numRessource = -1;
				}
				
				if (!mListeElementsAFabriquer)
					mListeElementsAFabriquer = element;
				else
					elementCourant->elementSuivant = element;
				
				elementCourant = element;
				elementXml = elementXml->NextSiblingElement();
			}
		}
	}
	
	TiXmlElement*
	Batiment::Sauvegarde()
	{
		TiXmlElement* retour = new TiXmlElement("Batiment");
		TiXmlElement* selement;
		TiXmlElement* sselement;
		
		retour->SetAttribute("id", mIdentifiant);
		
		selement = new TiXmlElement("Abscisse");
		retour->LinkEndChild(selement);
		selement->SetDoubleAttribute("valeur", mX);
		
		selement = new TiXmlElement("Ordonnee");
		retour->LinkEndChild(selement);
		selement->SetDoubleAttribute("valeur", mY);
		
		selement = new TiXmlElement("Type");
		retour->LinkEndChild(selement);
		selement->SetAttribute("valeur", mTypeBatiment->GetId());
		
		selement = new TiXmlElement("Construit");
		retour->LinkEndChild(selement);
		selement->SetAttribute("valeur", mConstruit);
		
		selement = new TiXmlElement("Vie");
		retour->LinkEndChild(selement);
		selement->SetDoubleAttribute("valeur", mVie);
		
		selement = new TiXmlElement("Construction");
		retour->LinkEndChild(selement);
		unsigned int i = 0;
		unsigned int nombre = mTypeBatiment->GetNombreTypesRessourcesNecessaires();
		while (i < nombre)
		{
			sselement = new TiXmlElement("Ressource");
			selement->LinkEndChild(sselement);
			sselement->SetAttribute("valeur", i);
			
			sselement = new TiXmlElement("Quantite");
			selement->LinkEndChild(sselement);
			sselement->SetDoubleAttribute("valeur", mQuantitesRessourcesConstruction[i]);
			
			i++;
		}
		
		std::ostringstream chaineAvancements;
		std::ostringstream chaineObjets;
		chaineAvancements.str("");
		chaineObjets.str("");
		unsigned int longueur = mTypeBatiment->GetNombreObjetsFabricables();
		i = 0;
		while (i < longueur)
		{
			if (mAvancementsFabricationObjet[i])
				chaineAvancements << mAvancementsFabricationObjet[i] << "/";
			else
				chaineAvancements << "/";
			
			if (mNombresObjetsFabriques[i])
				chaineObjets << mNombresObjetsFabriques[i] << "/";
			else
				chaineObjets << "/";
			i++;
		}
		selement = new TiXmlElement("AvancementsFabricationObjet");
		retour->LinkEndChild(selement);
		selement->SetAttribute("valeur", chaineAvancements.str().c_str());
		
		selement = new TiXmlElement("NombresObjetsFabriques");
		retour->LinkEndChild(selement);
		selement->SetAttribute("valeur", chaineObjets.str().c_str());
		
		selement = new TiXmlElement("ListeElementsAFabriquer");
		retour->LinkEndChild(selement);
		ListeElements* element = mListeElementsAFabriquer;
		while (element)
		{
			if (element->numObjet >= 0)
			{
				sselement = new TiXmlElement("Objet");
				selement->LinkEndChild(sselement);
				sselement->SetAttribute("id", element->numObjet);
			}
			
			if (element->numRessource >= 0)
			{
				sselement = new TiXmlElement("Ressource");
				selement->LinkEndChild(sselement);
				sselement->SetAttribute("id", element->numRessource);
				sselement->SetAttribute("quantite", element->quantite);
			}
			
			element = element->elementSuivant;
		}
		
		return retour;
	}
	
	bool
	Batiment::Construit(Etre* constructeur, double tempsFrame)
	{
		if (!mConstruit)
		{
			Personnage* perso = static_cast<Personnage*>(constructeur);
			unsigned int i = 0;
			unsigned int nombre = mTypeBatiment->GetNombreTypesRessourcesNecessaires();
			double max = 100 * tempsFrame / mTypeBatiment->GetMaxVie() / nombre;
			
			/*
			double maxPerso;
			while (i < nombre && max > 0)
			{
				maxPerso = perso->GetQuantiteRessource(mTypeBatiment->GetTypeRessourceNecessaire(i)) / mTypeBatiment->GetQuantiteRessourceNecessaire(i);
				if (maxPerso < max)
					max = maxPerso;
				
				i++;
			}
			*/
			
			if (max)
			{
				double maxRessource;
				double quantiteNecessaire;
				double effectif;
				
				i = 0;
				while (i < nombre)
				{
					quantiteNecessaire = mTypeBatiment->GetQuantiteRessourceNecessaire(i);
					maxRessource = (quantiteNecessaire - mQuantitesRessourcesConstruction[i]) / quantiteNecessaire;
					max = max < maxRessource ? max : maxRessource;
					
					effectif = perso->EnleveRessource(mTypeBatiment->GetTypeRessourceNecessaire(i), max * quantiteNecessaire);
					
					mQuantitesRessourcesConstruction[i] += effectif;
					mVie += mTypeBatiment->GetMaxVie() * effectif / quantiteNecessaire / nombre;
					
					if (mTypeBatiment->GetMaxVie() - mVie <= 0.01) // correction d'un probleme de prcision
					{
						mConstruit = true;
						mVie = mTypeBatiment->GetMaxVie();
					}
					
					i++;
				}
			}
		}
		
		return mConstruit;
	}
	
	void
	Batiment::Travaille(Etre* fabricant, double tempsFrame)
	{
		if (mListeElementsAFabriquer)
		{
			Personnage* perso = static_cast<Personnage*>(fabricant);
			
			if (mListeElementsAFabriquer->numRessource >= 0)
			{
				TypeRessource* typeRessource = GestionnaireDonnees->GetTypeRessource(mTypeBatiment->GetNumRessourceTransformable(mListeElementsAFabriquer->numRessource));
				
				double maxPerso;
				double max = tempsFrame * typeRessource->GetVitesseTransformation();
				
				if (max > mListeElementsAFabriquer->quantite)
					max = mListeElementsAFabriquer->quantite;
				
				unsigned int i = 0;
				unsigned int nombre = typeRessource->GetNombreTypesRessourcesNecessaires();
				while (i < nombre && max > 0)
				{
					maxPerso = perso->GetQuantiteRessource(typeRessource->GetTypeRessourceNecessaire(i)) / typeRessource->GetQuantiteRessourceNecessaire(i);
					if (max > maxPerso)
						max = maxPerso;
					
					i++;
				}
				
				if (max)
				{
					i = 0;
					while (i < nombre)
					{
						perso->EnleveRessource(typeRessource->GetTypeRessourceNecessaire(i), max * typeRessource->GetQuantiteRessourceNecessaire(i));
						i++;
					}
					
					perso->AjouteRessource(mTypeBatiment->GetNumRessourceTransformable(mListeElementsAFabriquer->numRessource), max);
					mListeElementsAFabriquer->quantite -= max;
					
					if (mListeElementsAFabriquer->quantite <= 0)
						mListeElementsAFabriquer = mListeElementsAFabriquer->elementSuivant;
				}
			}
			
			if (mListeElementsAFabriquer->numObjet >= 0)
			{
				TypeObjet* typeObjet = GestionnaireDonnees->GetTypeObjet(mTypeBatiment->GetNumObjetFabricable(mListeElementsAFabriquer->numObjet));
				
				double maxPerso;
				double max = tempsFrame / typeObjet->GetDureeFabrication();
				if (max > 1 - mAvancementsFabricationObjet[mListeElementsAFabriquer->numObjet])
					max = 1 - mAvancementsFabricationObjet[mListeElementsAFabriquer->numObjet];
				
				unsigned int i = 0;
				unsigned int nombre = typeObjet->GetNombreTypesRessourcesNecessaires();
				while (i < nombre && max > 0)
				{
					maxPerso = perso->GetQuantiteRessource(typeObjet->GetTypeRessourceNecessaire(i)) / typeObjet->GetQuantiteRessourceNecessaire(i);
					if (max > maxPerso)
						max = maxPerso;
					
					i++;
				}
				
				if (max)
				{
					i = 0;
					while (i < nombre)
					{
						perso->EnleveRessource(typeObjet->GetTypeRessourceNecessaire(i), max * typeObjet->GetQuantiteRessourceNecessaire(i));
						
						i++;
					}
					
					mAvancementsFabricationObjet[mListeElementsAFabriquer->numObjet] += max;
					
					if (mAvancementsFabricationObjet[mListeElementsAFabriquer->numObjet] >= 1)
					{
						mAvancementsFabricationObjet[mListeElementsAFabriquer->numObjet] = 0;
						mNombresObjetsFabriques[mListeElementsAFabriquer->numObjet]++;
						mListeElementsAFabriquer = mListeElementsAFabriquer->elementSuivant;
					}
				}
			}
		}
	}
	
	std::string
	Batiment::GetTexte() const
	{
		std::ostringstream chaineRetour;
		chaineRetour << mTypeBatiment->GetNom();
		
		if (!mConstruit)
		{
			chaineRetour << "\nManquent:";
			
			unsigned int longueur = mTypeBatiment->GetNombreTypesRessourcesNecessaires();
			unsigned int i = 0;
			while (i < longueur)
			{
				chaineRetour << "\n" << mTypeBatiment->GetQuantiteRessourceNecessaire(i) - floor(mQuantitesRessourcesConstruction[i]) << " " << GestionnaireDonnees->GetTypeRessource(mTypeBatiment->GetTypeRessourceNecessaire(i))->GetNom();
				
				i++;
			}
		}
		
		return chaineRetour.str();
	}
	
	void
	Batiment::Dessine(sf::RenderTarget* const App, const double x, const double y) const
	{
		if (mTypeBatiment)
			mTypeBatiment->DessineStructure(App, mX + x, mY + y, mVie, mConstruit);
	}
	
	void
	Batiment::AjouteRessourceAFabriquer(const unsigned int num, const unsigned int quantite)
	{
		ListeElements* pointeur;
		
		if (mListeElementsAFabriquer)
		{
			pointeur = mListeElementsAFabriquer;
			
			while (pointeur->elementSuivant)
			{
				pointeur = pointeur->elementSuivant;
			}
			
			pointeur->elementSuivant = new ListeElements();
			pointeur = pointeur->elementSuivant;
		}
		else
		{
			mListeElementsAFabriquer = new ListeElements();
			pointeur = mListeElementsAFabriquer;
		}
		
		pointeur->numRessource = num;
		pointeur->numObjet = -1;
		pointeur->quantite = quantite;
		pointeur->elementSuivant = NULL;
	}
	
	void
	Batiment::AjouteObjetAFabriquer(const unsigned int num)
	{
		ListeElements* pointeur;
		
		if (mListeElementsAFabriquer)
		{
			pointeur = mListeElementsAFabriquer;
		 	
			while (pointeur->elementSuivant)
			{
				pointeur = pointeur->elementSuivant;
			}
			
			pointeur->elementSuivant = new ListeElements();
			pointeur = pointeur->elementSuivant;
		}
		else
		{
			mListeElementsAFabriquer = new ListeElements();
			pointeur = mListeElementsAFabriquer;
		}
		
		pointeur->numRessource = -1;
		pointeur->numObjet = num;
		pointeur->quantite = 0;
		pointeur->elementSuivant = NULL;
	}
	
	void
	Batiment::EnleveElementListeTravail(const unsigned int num)
	{
		if (mListeElementsAFabriquer)
		{
			if (num)
			{
				ListeElements* pointeur = mListeElementsAFabriquer;
				unsigned int i = 0;
				while (i < num - 1 && pointeur->elementSuivant)
				{
					pointeur = pointeur->elementSuivant;
					
					i++;
				}
				
				if (pointeur->elementSuivant)
					pointeur->elementSuivant = pointeur->elementSuivant->elementSuivant;
			}
			else
				mListeElementsAFabriquer = mListeElementsAFabriquer->elementSuivant;
		}
	}
	
	GS::Objet*
	Batiment::PrendObjet(const unsigned int num)
	{
		if (mNombresObjetsFabriques[num] && mPere)
		{
			mNombresObjetsFabriques[num]--;
			GS::Objet* retour = static_cast<GS::Monde*>(mPere)->CreerObjet();
			retour->SetTypeObjet(GestionnaireDonnees->GetTypeObjet(mTypeBatiment->GetNumObjetFabricable(num)));
			
			return retour;
		}
		else
			return NULL;
	}
	
	GS::TypeObjet*
	Batiment::GetTypeObjetFabricable(const unsigned int num) const
	{
		return GestionnaireDonnees->GetTypeObjet(mTypeBatiment->GetNumObjetFabricable(num));
	}
	
	GS::TypeRessource*
	Batiment::GetTypeRessourceATransformer(const unsigned int num) const
	{
		int id;
		
		if (mListeElementsAFabriquer)
		{
			if (num)
			{
				ListeElements* pointeur = mListeElementsAFabriquer;
				unsigned int i = 0;
				while (i < num - 1 && pointeur->elementSuivant)
				{
					pointeur = pointeur->elementSuivant;
					i++;
				}
				
				id = pointeur->numRessource;
			}
			else
			{
				id = mListeElementsAFabriquer->numRessource;
			}
			
			if (id >= 0)
				return GestionnaireDonnees->GetTypeRessource(mTypeBatiment->GetNumRessourceTransformable(id));
			else
				return NULL;
		}
		else
			return NULL;
	}
	
	GS::TypeObjet*
	Batiment::GetTypeObjetAFabriquer(const unsigned int num) const
	{
		int id;
		
		if (mListeElementsAFabriquer)
		{
			if (num)
			{
				ListeElements* pointeur = mListeElementsAFabriquer;
				unsigned int i = 0;
				while (i < num - 1 && pointeur->elementSuivant)
				{
					pointeur = pointeur->elementSuivant;
					i++;
				}
				
				id = pointeur->numObjet;
			}
			else
			{
				id = mListeElementsAFabriquer->numObjet;
			}
			
			if (id >= 0)
				return GestionnaireDonnees->GetTypeObjet(mTypeBatiment->GetNumObjetFabricable(id));
			else
				return NULL;
		}
		else
			return NULL;
	}
	
	void
	Batiment::SetType(TypeBatiment* type)
	{
		mTypeBatiment = type;
		
		mHauteur = mTypeBatiment->GetHauteurBatiment();
		mLargeur = mTypeBatiment->GetLargeurBatiment();
		
		unsigned int longueur = mTypeBatiment->GetNombreTypesRessourcesNecessaires();
		mQuantitesRessourcesConstruction.resize(longueur);
		
		unsigned int i = 0;
		while (i < longueur)
		{
			mQuantitesRessourcesConstruction[i] = 0;
			
			i++;
		}
		
		
		longueur = mTypeBatiment->GetNombreObjetsFabricables();
		
		mAvancementsFabricationObjet.resize(longueur);
		mNombresObjetsFabriques.resize(longueur);
		
		i = 0;
		while (i < longueur)
		{
			mAvancementsFabricationObjet[i] = 0;
			mNombresObjetsFabriques[i] = 0;
			
			i++;
		}
	}
	
	void
	Batiment::SetPosition(const double x, const double y)
	{
		mX = x;
		mY = y;
	}
	
	double
	Batiment::GetQuantitesRessourceConstruction(const unsigned int num) const
	{
		return mQuantitesRessourcesConstruction[num];
	}
	
	TypeBatiment*
	Batiment::GetTypeBatiment() const
	{
		return mTypeBatiment;
	}
	
	bool
	Batiment::GetConstruit() const
	{
		return mConstruit;
	}
	
	unsigned int
	Batiment::GetNombreObjetsConstruits(const unsigned int num) const
	{
		return mNombresObjetsFabriques[num];
	}
	
	double
	Batiment::GetAvancementFabricationObjet(const unsigned int num) const
	{
		return mAvancementsFabricationObjet[num];
	}
	
	ListeElements*
	Batiment::GetListeElements() const
	{
		return mListeElementsAFabriquer;
	}
	
	bool
	Batiment::GetConstructible(const double x, const double y, const unsigned int largeur, const unsigned int hauteur) const
	{
		return mX <= x + largeur && mY <= y + hauteur && x <= mX + mLargeur && y <= mY + mHauteur;
	}
}

#endif
