#ifndef CHAMP_CPP
#define CHAMP_CPP

#include "champ.hpp"

namespace Interface
{
	Champ::Champ(const std::string txt)
	{
		mAfficher = false;
		mActif = false;
		
		mLeft = 0;
		mTop = 0;
		
		mTexteInvite = new sf::String(txt);
		mTexteInvite->SetSize(20);
		
		mTexteUtilisateur = new sf::String("");
		mTexteUtilisateur->SetSize(20);
		
		mInsere = true;
		mCaractereAcuel = 0;
		
		mChariot = new sf::String("|");
		mChariot->SetSize(20);
		
		mTextePrecedent = new sf::String("");
		mTextePrecedent->SetSize(20);
		
		// caracteres speciaux
		mNumChars[0] = 'a';
		mNumChars[1] = '&';
		mNumChars[2] = 'e';
		mNumChars[3] = '"';
		mNumChars[4] = '\'';
		mNumChars[5] = '(';
		mNumChars[6] = '-';
		mNumChars[7] = 'e';
		mNumChars[8] = '_';
		mNumChars[9] = '';
		
		mFonctionRetour = NULL;
		mCible = NULL;
	}
	
	Champ::~Champ()
	{
		delete mTexteInvite;
		delete mTexteUtilisateur;
		delete mChariot;
		delete mTextePrecedent;
	}
	
	void
	Champ::Dessine(sf::RenderTarget* const App)
	{
		if (mAfficher)
		{
			mTexteInvite->SetPosition(mLeft, mTop);
			App->Draw(*mTexteInvite);
			
			mTexteUtilisateur->SetPosition(mLeft + mTexteInvite->GetRect().GetWidth(), mTop);
			App->Draw(*mTexteUtilisateur);
			
			if (mActif)
			{
				mTextePrecedent->SetText(((std::string)(mTexteUtilisateur)->GetText()).substr(0, mCaractereAcuel));
				
				mChariot->SetPosition(mLeft + mTexteInvite->GetRect().GetWidth() + mTextePrecedent->GetRect().GetWidth(),mTop);
				App->Draw(*mChariot);
			}
		}
	}
	
	void
	Champ::ReInitialise()
	{
		mAfficher = false; // par defaut, l'invite n'est pas activee
		mActif = false; // donc la capture n'est pas active non plus
		mTexteUtilisateur->SetText("");
		mCaractereAcuel = 0;
	}
	
	bool
	Champ::Ecrit(const sf::Key::Code code)
	{
		if (mActif)
		{
			bool traite = true;
			int codeTouche = static_cast<int>(code);
			std::string texte = mTexteUtilisateur->GetText();
			
			if (codeTouche == sf::Key::Escape) // echap
				Desactive();
			else if (codeTouche == sf::Key::Return) // envoi des donnees
			{
				if (mCible != 0)
						mCible->RecoitChaine(texte);
				else if (mFonctionRetour != 0)
					(*mFonctionRetour)(texte);
				
				Desactive();
				Cache();
			}
			else if (codeTouche == sf::Key::Back) // effacement arriere
			{
				if (mCaractereAcuel > 0)
				{
					mTexteUtilisateur->SetText(texte.substr(0, mCaractereAcuel - 1) + texte.substr(mCaractereAcuel, texte.size() - mCaractereAcuel));
					mCaractereAcuel--;
				}
			}
			else if (codeTouche == sf::Key::Delete) // effacement avant
			{
				if (mCaractereAcuel < texte.size() - 1)
				{
					mTexteUtilisateur->SetText(texte.substr(0, mCaractereAcuel) + texte.substr(mCaractereAcuel + 1, texte.size() - mCaractereAcuel));
					mCaractereAcuel--;
				}
			}
			else if (codeTouche == sf::Key::Insert) // insertion
			{
				if (mInsere)
				{
					mChariot->SetText("_");
					mInsere = false;
				}
				else
				{
					mChariot->SetText("|");
					mInsere = true;
				}
			}
			else if (codeTouche == sf::Key::Up) // haut -> debut du texte
				mCaractereAcuel = 0;
			else if (codeTouche == sf::Key::Down) // bas -> fin du texte
				mCaractereAcuel = texte.size();
			else if (codeTouche == sf::Key::Left) // deplacement chariot gauche
			{
				if (mCaractereAcuel > 0)
					mCaractereAcuel--;
			}
			else if (codeTouche == sf::Key::Right) // deplacement chariot droit
			{
				if (mCaractereAcuel < texte.size())
					mCaractereAcuel++;
			}
			else if (sf::Key::A <= codeTouche && codeTouche <= sf::Key::Z) // touches alphabetiques
			{
				if (mInput->IsKeyDown(sf::Key::LShift) || mInput->IsKeyDown(sf::Key::RShift))
					codeTouche -= 32;
				
				AjouteChar( static_cast<char>(codeTouche) );
			}
			else if (sf::Key::Num0 <= codeTouche && codeTouche <= sf::Key::Num9) // touches numeriques et speciales
			{
				if (mInput->IsKeyDown(sf::Key::LShift) || mInput->IsKeyDown(sf::Key::RShift))
					AjouteChar( static_cast<char>(codeTouche) );
				else
					AjouteChar(mNumChars[codeTouche - sf::Key::Num0]);
				
			}
			else if (sf::Key::Numpad0 <= codeTouche && codeTouche <= sf::Key::Numpad9) // clavier numerique
			{
				codeTouche = codeTouche - sf::Key::Numpad0 + sf::Key::Num0;
				AjouteChar( static_cast<char>(codeTouche) );
			}
			else if (codeTouche == sf::Key::Space) // espace
				AjouteChar(' ');
			else
				traite = false;
			
			return traite;
		}
		else
			return false;
	}
	
	void
	Champ::AjouteChar(const char carac)
	{
		std::string texte = mTexteUtilisateur->GetText();
		
		if (mCaractereAcuel < texte.size())
			mTexteUtilisateur->SetText(texte.substr(0, mCaractereAcuel) + carac + texte.substr(mCaractereAcuel + static_cast<int>(!mInsere), texte.size() - mCaractereAcuel));
		else
			mTexteUtilisateur->SetText(texte.substr(0, mCaractereAcuel + 1) + carac);
		
		mCaractereAcuel++;
	}
	
	void
	Champ::SetPosition(const double l, const double t)
	{
		mLeft = l;
		mTop = t;
	}
	
	void
	Champ::SetInput(const sf::Input* in)
	{
		mInput = in;
	}
	
	void
	Champ::Active()
	{
		mActif = true;
	}
	
	void
	Champ::Desactive()
	{
		mActif = false;
	}
	
	void
	Champ::Affiche()
	{
		mAfficher = true;
	}
	
	void
	Champ::Cache()
	{
		mAfficher = false;
	}
	
	void
	Champ::SetTexte(const std::string txt)
	{
		mTexteInvite->SetText(txt);
	}
	
	void
	Champ::SetContenu(const std::string txt)
	{
		mTexteUtilisateur->SetText(txt);
	}
	
	void
	Champ::SetFonction(void (*fct)(std::string))
	{
		mFonctionRetour = fct;
	}
	
	void
	Champ::SetObjet(ReceveurChaine* objet)
	{
		mCible = objet;
	}
	
	std::string
	Champ::GetTexte() const
	{
		return mTexteInvite->GetText();
	}
	
	std::string
	Champ::GetContenu() const
	{
		return mTexteUtilisateur->GetText();
	}
}

#endif
