/**
* \file champ.hpp
* \brief Interface::Champ
* \author Yoha
* \version alpha-0.0.07
* \date Aot 2009
*
* Definition des champs de l'interface
*
*/

#ifndef CHAMP_HPP
#define CHAMP_HPP

/// Mode dynamique
#define SFML_DYNAMIC

#include <iostream>
#include <SFML/Graphics.hpp>

#include "gamesystem.hpp"

/// Groupement de tous les elements concernant l'interface
namespace Interface
{
	/// Definition des champs
	class Champ
	{
		public:
			/// \brief Constructeur
			/// \param txt Texte d'invite
			Champ(const std::string txt);
			/// \brief Destructeur
			~Champ();
			
			/// \brief Methode de dessine du champ
			/// \param App Pointeur sur la fenetre de rendu
			void Dessine(sf::RenderTarget* const App);
			/// \brief Remise a zero pour nouvel affichage
			void ReInitialise();
			/// \brief Capture de la pressions des touches
			/// \param code Code de la touche pressee
			bool Ecrit(const sf::Key::Code code);
			/// \brief Permet d'ajouter un caractere a l'emplacement du chariot
			void AjouteChar(const char texte);
			
			/// \brief Accesseur: definit la position du champ
			/// \param l Emplacement gauche (left)
			/// \param t Emplacement haut (top)
			void SetPosition(const double l, const double t);
			/// \brief Accesseur: memorisation du statut des touches
			/// \param in Input sfml
			void SetInput(const sf::Input* in);
			/// \brief Accesseur: active la capture des touches
			void Active();
			/// \brief Accesseur: desactive la capture des touches
			void Desactive();
			/// \brief Accesseur: affiche le champ
			void Affiche();
			///  \brief Accesseur: cache le champ
			void Cache();
			/// \brief Accesseur: definit le texte d'invite
			/// \param txt Nouveau texte
			void SetTexte(const std::string txt);
			/// \brief Accesseur: definit le texte du champ
			/// \param txt Nouveau texte
			void SetContenu(const std::string txt);
			
			/// \brief Accesseur: definit la fonction recevant la chaine
			/// \param  fct Fonction cible
			void SetFonction(void (*fct)(std::string));
			/// \brief Accesseur: definit l'objet recevant la chaine
			/// \param objet Objet cible
			void SetObjet(ReceveurChaine* objet);
			
			/// \brief Accesseur: renvoie le texte d'invite
			std::string GetTexte() const;
			/// \brief Accesseur: renvoie le texte du champ
			std::string GetContenu() const;
			
		private:
			/// Le champs est-il affiche
			bool mAfficher;
			/// La capture des touches est-elle active
			bool mActif;
			
			/// Abscisse du champ
			double mLeft;
			/// Ordonnee du champ
			double mTop;
			
			/// Texte d'invite
			sf::String* mTexteInvite;
			/// Texte actuel du champ
			sf::String* mTexteUtilisateur;
			
			/// L'insertion est-elle active
			bool mInsere;
			/// Emplacement du chariot
			unsigned int mCaractereAcuel;
			/// Forme du chariot
			sf::String* mChariot;
			/// Texte precedent le chariot (calcul de largeur)
			sf::String* mTextePrecedent;
			
			/// Pour les touches speciales
			char mNumChars[10];
			/// Memoire du statut des touches
			const sf::Input* mInput;
			/// Fonction de retour
			void (*mFonctionRetour)(std::string);
			/// Objet de retour
			ReceveurChaine* mCible;
	};
}

#endif
