/**
* \file data.hpp
* \brief Donnees
* \author Yoha
* \version alpha-0.0.07
* \date Aot 2009
*
* Ce fichier regroupe le systeme de donnees
*
*/

#ifndef DATA_HPP
#define DATA_HPP

/// Mode dynamique
#define SFML_DYNAMIC

#include <SFML/Graphics.hpp>

namespace Interface
{
	class Interface;
}

/// Ce namespace regroupe des donnees utilisables par les autres modules
namespace Prgm
{
	/// Regroupe les informations concernant la fenetre d'affichage
	namespace Window
	{
		/// Largeur de la fenetre
		extern unsigned int Width;
		/// Hauteur de la fenetre
		extern unsigned int Height;
		
		/// Moitie horizontale de la fenetre, evite certains calculs
		extern unsigned int MilieuX;
		/// Moitie verticale de la fenetre, evite certains calculs
		extern unsigned int MilieuY;
		
		/// Pointeur sur la fenetre utilisee (pour les rapports d'erreur notamment)
		extern sf::RenderTarget* App;
	}
	
	namespace ElementsInterface
	{
		/// Avecement du chargement de l'interface (entre 0 et 1)
		extern double AvancementChargement;
		
		/// Pointeur vers le gestionnaire des entrees
		extern const sf::Input* Input;
		/// Pointeur sur l'unique instance de la classe interface
		extern Interface::Interface* interface;
	}
	
	/// Regroupe les donnees de definition du monde
	namespace Monde
	{
		/// Largeur du terrain en nombre de terrains elementaires
		extern unsigned int Largeur;
		/// Hauteur du terrain en nombre de terrains elementaires
		extern unsigned int Hauteur;
		/// Intensite de la pesanteur
		extern double Pesanteur;
	}
	
	/// Regroupe les informations concernant les terrains
	namespace Terrains
	{
		/// Largeur d'un terrain
		extern unsigned int Largeur;
		/// Hauteur d'un terrain
		extern unsigned int Hauteur;
		
		/// Cote d'un BoutDeMonde en nombre de terrains
		extern unsigned int CoteCarte;
		/// Largeur en pixels d'un BoutDeMonde
		extern unsigned int LargeurBoutDeMonde;
		/// Hauteur en pixels d'un BoutDeMonde
		extern unsigned int HauteurBoutDeMonde;
	}
	
	/// Regroupe les informations concernant les ressources
	namespace Ressources
	{
		/// Cote des icones des ressources
		extern unsigned int CoteIcone;
	}
	
	/// Regroupe les informations concernant les ressources de terrain
	namespace TerRessources
	{
		/// Largeur d'une ressource de terrain
		extern unsigned int Largeur;
		/// Hauteur d'une ressource de terrain
		extern unsigned int Hauteur;
	}
	
	/// Regroupe les informations concernant les objets
	namespace Objets
	{
		/// Cote des icones des objets
		extern unsigned int CoteIcone;
	}
	
	/// Regroupe les informations concernant le personnage
	namespace Personnage
	{
		/// Largeur de l'image
		extern unsigned int Largeur;
		/// Hauteur de l'image
		extern unsigned int Hauteur;
		/// Largeur d'un sprite
		extern unsigned int LargeurSprite;
		/// Hauteur d'un sprite
		extern unsigned int HauteurSprite;
		/// Nombre de pas effectues par round
		extern double PasParSeconde;
		/// Image contenant les sprites du personnage
		extern sf::Image* ImageDefaut;
	}
	
	/// Regroupe les informations concernant les batiments
	namespace Batiments
	{
		/// Sprite de construction possible
		extern sf::Sprite* SpriteConstructionPossible;
		/// Sprite de construction impossible
		extern sf::Sprite* SpriteConstructionImpossible;
		/// Cote d'un sprite de bouton de batiment
		extern unsigned int CoteSpriteBouton;
		/// Cote d'un bouton de batiment
		extern unsigned int CoteBouton;
		/// Nombre de lignes du panneau retractable
		extern unsigned int NombreLignesPanneau;
		/// Sprite de debut de construction
		extern sf::Sprite* DebutConstruction;
	}
	
	/// Regroupe les informations concernant les sous-fenetres
	namespace SFenetres
	{
		/// Marges
		extern double Marges;
		
		/// Taille du texte du titre
		extern double TailleTexteTitre;
		/// Ordonnee du titre
		extern double OrdonneeTitre;
		
		/// Taille du texte
		extern double TailleTexte;
		/// Sprite des sous-fenetres
		extern sf::Sprite* Sprite;
		
		/// Regroupe les informations concernant la sous-fenetre de batiment
		namespace Batiment
		{
			/// Taile du texte du nom du batiment charge
			extern double TailleTexteNom;
			/// Ordonnee du nom du batiment charge
			extern double OrdonneeNom;
			
			/// Abscisse de la liste des objets constructibles
			extern double AbscsisseListes;
			/// Ordonnee de la liste des objets constructibles
			extern double OrdonneeListes;
			
			/// Largeur de la colone des noms des objets constructibles
			extern double LargeurColoneNoms;
			/// Espace interligne
			extern double EspaceInterligne;
			
			/// Bouton transformation ressources
			extern sf::Sprite* TransformationRessources;
			/// Bouton fabrication objets
			extern sf::Sprite* FabricationObjets;
			/// Largeur du bouton de permutation entre transformation des ressources et fabrication des objets
			extern unsigned int CoteBoutonTransformationFabrication;
			/// Abscisse du bouton de permutation entre transformation des ressources et fabrication des objets
			extern double AbscisseBoutonTransformationFabrication;
			/// Ordonnee du bouton de permutation entre transformation des ressources et fabrication des objets
			extern double OrdonneeBoutonTransformationFabrication;
			
			/// Abscisse du texte de la quantite de ressource a transformer
			extern double AbscisseQuantiteRessourceATransformer;
			/// Ordonnee du texte de la quantite de ressource a transformer
			extern double OrdonneeQuantiteRessourceATransformer;
		}
		
		/// Regroupe les informations concernant la sous-fenetre d'inventaire
		namespace Inventaire
		{
			/// Sprite de la grille d'objets
			extern sf::Sprite* SpriteGrilleObjets;
			/// Cote d'une celle de la grille
			extern double CoteCellule;
			/// Abscisse de la grille d'objets (sprite)
			extern double AbscisseGrilleObjets;
			/// Ordonnee de la grille d'objets (sprite)
			extern double OrdonneeGrilleObjets;
			/// Marge superieure de la grille d'objets
			extern double MargeSuperieureGrilleObjets;
			/// Nombre de lignes de la grille d'objets
			extern unsigned int NombreLignes;
			/// Nombre de colones de la grille d'objets
			extern unsigned int NombreColones;
		}
	}
	
	/// Regroupe les informations concernant l'affichage des textes
	namespace Textes
	{
		/// Sprite dessinant le fond du texte
		extern sf::Sprite* Fond;
	}
	
	/// Regroupe les informations concernant les curseurs
	namespace Curseurs
	{
		/// Largeur et hauteur d'un sprite contenant un curseur
		extern unsigned int CoteSprite;
		/// Largeur et hauteur d'un curseur sur l'ecran
		extern unsigned int Cote;
		/// Image contenant les sprites des curseurs
		extern sf::Image* Image;
	}
	
	/// Regroupe les informations concernant les fleches
	namespace Fleches
	{
		/// Sprite de fleche gauche
		extern sf::Sprite* FlecheGauche;
		/// Sprite de fleche droite
		extern sf::Sprite* FlecheDroite;
		/// Cote d'un sprite de fleche
		extern double Cote;
	}
	
	/// Regroupe les informations concernant les barres de chargement ou de statut
	namespace BarresChargement
	{
		/// Largeur des barres de chargement
		extern double Largeur;
		/// Hauteur des barres de chargement
		extern double Hauteur;
		/// Largeur du contour des barres de chargement
		extern double EpaisseurContours;
		/// Ecart vertical eventuel entre les differentes barres de chargement
		extern double Ecart;
		/// Marge verticale par rapport au bord de l'ecran
		extern double MargeVerticale;
		/// Marge horizontale par rapport au bord de l'ecran
		extern double MargeHorizontale;
		/// Taille du texte
		extern double TailleTexte;
	}
}

#endif
