/**
* \file etre.hpp
* \brief GS::Etre
* \author Yoha
* \version alpha-0.0.07
* \date Aot 2009
*
* Dfinit la classe d'hritage Etre
*
*/

#ifndef ETRE_HPP
#define ETRE_HPP

#include "tinyxml/tinyxml.h"
#include "lunar.hpp"

#include "gamesystem.hpp"

/// Game System
namespace GS
{
	/// Dfinit un etre du systme de jeu
	class Etre
	{
		public:
			/// \brief Constructeur
			Etre(GS::Etre* parent, TiXmlElement* element = NULL);
			/// \brief Constructeur Lua
			Etre(lua_State*);
			/// \brief Destructeur
			virtual ~Etre();
			
			/// \brief Charge une sauvegarde
			/// \param element Element contenant la sauvegarde
			virtual void Charge();
			/// \brief Sauvegarde
			virtual TiXmlElement* Sauvegarde();
			
			/// \brief Capture des clics de la souris
			/// \param x Abscisse du clic
			/// \param y Ordonnee du clic
			virtual bool Catch(const double x, const double y);
			
			/// \brief Determine la position de l'etre
			virtual void SetPosition(const double x, const double y);
			
			/// \brief Retourn l'identifiant de l'etre
			virtual unsigned int GetId() const;
			/// \brief Retourne l'abscisse de l'etre
			virtual double GetX() const;
			/// \brief Retourne l'ordonnee de l'etre
			virtual double GetY() const;
			/// \brief Retourne la largeur de l'etre
			virtual double GetLargeur() const;
			/// \brief Retourne la hauteur de l'etre
			virtual double GetHauteur() const;
			/// \brief Retourne le type d'etre dont il s'agit
			virtual Types::Names GetType() const;
		
		protected:
			/// Sauvegarde de l'etre a recharger
			TiXmlElement* mXml;
			/// Increment permettant de determiner un nouvel identifiant
			static unsigned int mProchainIdentifiant;
			/// Identifiant
			unsigned int mIdentifiant;
			/// Abscisse
			double mX;
			/// Ordonnee
			double mY;
			/// Largeur
			double mLargeur;
			/// Hauteur
			double mHauteur;
			
			/// Etre pere
			Etre* mPere;
			
			/// Type
			Types::Names mType;
	};
}

#endif
