#ifndef FONCTIONS_CPP
#define FONCTIONS_CPP

#include <cmath>
#include <fstream>

#include "data.hpp"
#include "personnage.hpp"
#include "batiment.hpp"

double CalculeDistance(const double x1, const double y1, const double x2, const double y2)
{
	return sqrt( pow(x1 - x2, 2) + pow(y1 - y2, 2) );
}

double NormaliseRessource(const double quantite)
{
	return static_cast<double>( floor(quantite * 10) / 10 );
}

void ChargeLua(lua_State** pointeur)
{
	*pointeur = lua_open();
	
	Lunar<GS::Personnage>::Register(*pointeur);
	Lunar<GS::Batiment>::Register(*pointeur);
	
	luaL_openlibs(*pointeur);
}

void
CastErreur(const std::string msg, const bool fatal)
{
	static std::ofstream errorsLog("errors.txt", std::ios::trunc);
	
	std::cerr << msg << "\n";
	errorsLog << msg;
	
	if (fatal) // affichage de la console, fin du programme
	{
		if (Prgm::Window::App)
			//Prgm::Window::App->Show(false);
		
		getchar();
		exit(EXIT_FAILURE);
	}
}

bool
ChargeXmlParametre(TiXmlElement* const parent, const char* parametre, unsigned int& retour)
{
	int valeur;
	if (parent->QueryIntAttribute(parametre, &valeur) == TIXML_SUCCESS && valeur >= 0)
	{
		retour = static_cast<unsigned int>(valeur);
		return true;
	}
	else
		return false;
}

bool
ChargeXmlValeur(TiXmlNode* const parent, const char* variable, bool& retour)
{
	int valeur;
	bool charge = false;
	TiXmlElement* selement = parent->FirstChildElement(variable);
	while (selement)
	{
		if (selement->QueryIntAttribute("valeur", &valeur) == TIXML_SUCCESS)
			charge = true;
		
		selement = selement->NextSiblingElement(variable);
	}
	if (charge)
	{
		retour = (valeur != 0);
		return true;
	}
	else
		return false;
}

bool
ChargeXmlValeur(TiXmlNode* const parent, const char* variable, int& retour)
{
	int valeur;
	bool charge = false;
	TiXmlElement* selement = parent->FirstChildElement(variable);
	while (selement)
	{
		if (selement->QueryIntAttribute("valeur", &valeur) == TIXML_SUCCESS)
			charge = true;
		
		selement = selement->NextSiblingElement(variable);
	}
	if (charge)
	{
		retour = valeur;
		return true;
	}
	else
		return false;
}

bool
ChargeXmlValeur(TiXmlNode* const parent, const char* variable, unsigned int& retour)
{
	int valeur;
	
	if (ChargeXmlValeur(parent, variable, valeur) && valeur >= 0)
	{
		retour = static_cast<unsigned int>( floor(valeur) );
		return true;
	}
	else
		return false;
}

bool
ChargeXmlValeur(TiXmlNode* const parent, const char* variable, double& retour)
{
	double valeur;
	bool charge = false;
	TiXmlElement* selement = parent->FirstChildElement(variable);
	while (selement)
	{
		if (selement->QueryDoubleAttribute("valeur", &valeur) == TIXML_SUCCESS)
			charge = true;
		
		selement = selement->NextSiblingElement(variable);
	}
	if (charge)
	{
		retour = valeur;
		return true;
	}
	else
		return false;
}

bool
ChargeXmlValeur(TiXmlNode* const parent, const char* variable, std::string& retour)
{
	std::string valeur;
	bool charge = false;
	TiXmlElement* selement = parent->FirstChildElement(variable);
	while (selement)
	{
		if (selement->Attribute("valeur"))
		{
			valeur = selement->Attribute("valeur");
			charge = true;
		}
		
		selement = selement->NextSiblingElement(variable);
	}
	if (charge)
	{
		retour = valeur;
		return true;
	}
	else
		return false;
}

#endif
