/**
* \file gestionnaire.hpp
* \brief Manager
* \author Yoha
* \version alpha-0.0.07
* \date Aot 2009
*
* Definition du gestionnaire general de donnees
*
*/

#ifndef GESTIONNAIRE_HPP
#define GESTIONNAIRE_HPP

/// Mode dynamique
#define SFML_DYNAMIC

#include <SFML/Graphics.hpp>
#include <vector>
#include "tinyxml/tinyxml.h"

#include "typeterrain.hpp"
#include "typeterressource.hpp"
#include "statut.hpp"
#include "typebatiment.hpp"
#include "typeobjet.hpp"
#include "race.hpp"

/// Classe de gestion generale des donnees
class Manager
{
	public:
		/// \brief Constructeur
		Manager();
		/// \brief Destructeur
		~Manager();
		
		/// \brief Charge les donnees de configuration
		void ChargeXml();
		
		/// \brief Retourne un pointeur sur l'image demandee
		/// \param num ID de l'imagee
		sf::Image* GetImage(const unsigned int num);
		
		/// \brief Retourne le nombre de types de terrains
		unsigned int GetNombreTypesTerrains() const;
		/// \brief Retourne un type de terrain
		/// \param num ID du type de terrain
		GS::TypeTerrain* GetTypeTerrain(const unsigned int num);
		
		/// \brief Retourne le nombre de types de ressources
		unsigned int GetNombreTypesRessources() const;
		/// \brief Retourne un type de ressource
		/// \param num ID du type de ressource
		GS::TypeRessource* GetTypeRessource(const unsigned int num);
		
		/// \brief Retourne le nombre de types de ressources de terrain
		unsigned int GetNombreTypesTerRessources() const;
		/// \brief Retourne un type de ressource de terrain
		/// \param num ID du type de ressource de terrain
		GS::TypeTerRessource* GetTypeTerRessource(const unsigned int num);
		
		/// \brief Retourne le nombre de statuts
		unsigned int GetNombreStatuts() const;
		/// \brief Retourne un statut
		/// \param num ID du statut
		GS::Statut* GetStatut(const unsigned int num);
		
		/// \brief Retourne le nombre de categories d'objets
		unsigned int GetNombreCategoriesObjets() const;
		/// \brief Retourne une categorie d'objet
		/// \param num ID de la categorie d'objet
		GS::CategorieObjet* GetCategorieObjet(const unsigned int num);
		
		/// \brief Retourne le nombre de types d'objets
		unsigned int GetNombreTypesObjets() const;
		/// \brief Retourne un type d'objet
		/// \param num ID du type d'objet
		GS::TypeObjet* GetTypeObjet(const unsigned int num);
		
		/// \brief Retourne le nombre de types de batiments
		unsigned int GetNombreTypesBatiments() const;
		/// \brief Retourne un type du batiment
		/// \param num ID du type du batiment
		GS::TypeBatiment* GetTypeBatiment(const unsigned int num);
		
	private:
		/// Tableau contenant les images
		std::vector <sf::Image*> mImages;
		/// Tableau contenant le statut de chargement des images
		std::vector <bool> mChargees;
		/// Tableau contenant les adresses des images
		std::vector <std::string*> mAdresses;
		/// Nombre d'images dans le tableau
		unsigned int mNombreImages;
		
		/// Tableau contenant les sprites
		std::vector <sf::Sprite*> mSprites;
		/// Nombre de sprites dans le tableau
		unsigned int mNombreSprites;
		
		/// Tableau contenant les types de terrains
		std::vector <GS::TypeTerrain*> mTypesTerrains;
		/// Nombre de types de terrains dans le tableau
		unsigned int mNombreTypesTerrains;
		
		/// Tableau contenant les types de ressources
		std::vector <GS::TypeRessource*> mTypesRessources;
		/// Nombre de types de ressources dans le tableau
		unsigned int mNombreTypesRessources;
		
		/// Tableau contenant les types de ressources de terrain
		std::vector <GS::TypeTerRessource*> mTypesTerRessources;
		/// Nombre de types de ressources de terrains dans le tableau
		unsigned int mNombreTypesTerRessources;
		
		/// Tableau contenant les types de batiments
		std::vector <GS::Statut*> mStatuts;
		/// Nombre de types de batiments dans le tableau
		unsigned int mNombreStatuts;
		
		/// Tableau contenant les types d'objets
		std::vector <GS::CategorieObjet*> mCategoriesObjets;
		/// Nombre de types d'objets dans le tableau
		unsigned int mNombreCategoriesObjets;
		
		/// Tableau contenant les types d'objets
		std::vector <GS::TypeObjet*> mTypesObjets;
		/// Nombre de types d'objets dans le tableau
		unsigned int mNombreTypesObjets;
		
		/// Tableau contenant les types de batiments
		std::vector <GS::TypeBatiment*> mTypesBatiments;
		/// Nombre de types de batiments dans le tableau
		unsigned int mNombreTypesBatiments;
		
		/// Tableau contenant les races
		std::vector <GS::Race*> mRaces;
		/// Nombre de races dans le tableau
		unsigned int mNombreRaces;
};

extern Manager* GestionnaireDonnees;

#endif
