#ifndef INTERFACE_CPP
#define INTERFACE_CPP

#include "interface.hpp"

#include <cstring>

#include "gestionnaire.hpp"
#include "fonctions.hpp"
#include "data.hpp"

namespace Interface
{
	Interface::Interface(const sf::Input* input, TiXmlElement* element)
	{
		mXml = element;
		
		mInput = input;
		
		mCurseurBaseAff = new sf::Sprite(*Prgm::Curseurs::Image);
		
		mCurseurBaseAff->SetSubRect( sf::IntRect(0 * Prgm::Curseurs::CoteSprite, 0, (0 + 1) * Prgm::Curseurs::CoteSprite, Prgm::Curseurs::CoteSprite) );
		mCurseurBaseAff->Resize(Prgm::Curseurs::Cote, Prgm::Curseurs::Cote);
		
		mCurseurConstructionPossible = new sf::Sprite(*Prgm::Batiments::SpriteConstructionPossible);
		mCurseurConstructionImpossible = new sf::Sprite(*Prgm::Batiments::SpriteConstructionImpossible);
		
		mSourisX = Prgm::Window::MilieuX;
		mSourisY = Prgm::Window::MilieuY;
		mDecalageCurseurSpriteX = 0;
		mDecalageCurseurSpriteY = 0;
		mBatimentAConstruire = NULL;
		mPeutConstruire = true;
		
		double abscisse = Prgm::BarresChargement::MargeVerticale;
		GS::Statut* statut;
		unsigned int nombreStatuts = GestionnaireDonnees->GetNombreStatuts();
		unsigned int i = 0;
		while (i < nombreStatuts)
		{
			statut = GestionnaireDonnees->GetStatut(i);
			statut->SetPosition(abscisse, Prgm::Window::Height - Prgm::BarresChargement::Hauteur - Prgm::BarresChargement::MargeHorizontale - 2 * Prgm::BarresChargement::EpaisseurContours);
			abscisse += Prgm::BarresChargement::Ecart + Prgm::BarresChargement::Largeur + 2 * Prgm::BarresChargement::EpaisseurContours;
			
			i++;
		}
		
		mInfobulle = new Texte();
		
		if (mXml)
			mMonde = new GS::Monde(NULL, mXml->FirstChildElement("Monde"));
		else
			mMonde = new GS::Monde(NULL);
		mMonde->SetPersonnageJoueur(&mPersonnage);
		
		mChamp = new Champ("> ");
		mChamp->SetInput(mInput);
		mChamp->SetObjet(mPersonnage);
		
		unsigned int nombreTypesBatiments = GestionnaireDonnees->GetNombreTypesBatiments();
		double nombreColones = nombreTypesBatiments / Prgm::Batiments::NombreLignesPanneau;
		
		mPanneauBatimentsXMin = 2 - nombreColones * Prgm::Batiments::CoteBouton;
		mPanneauBatimentsX = mPanneauBatimentsXMin;
		
		unsigned int colone = 0;
		unsigned int ligne = 0;
		GS::TypeBatiment* batiment = NULL;;
		i = 0;
		while (i < nombreTypesBatiments)
		{
			if ( (batiment = GestionnaireDonnees->GetTypeBatiment(i)) )
			{
				batiment->SetPositionBouton(Prgm::Batiments::CoteBouton * colone + mPanneauBatimentsX, 50 + Prgm::Batiments::CoteBouton * ligne);
				
				colone++;
				if (colone >= nombreColones)
				{
					colone = 0;
					ligne++;
				}
			}
			
			i++;
		}
		
		if (mXml)
		{
			mSousFenetreBatiment = new SFBatiment(mXml->FirstChildElement("SFBatiment"));
			mSousFenetreInventaire = new SFInventaire(mXml->FirstChildElement("SFInventaire"));
		}
		else
		{
			mSousFenetreBatiment = new SFBatiment();
			mSousFenetreInventaire = new SFInventaire();
		}
	}
	
	Interface::~Interface()
	{
		delete mCurseurBaseAff;
		
		delete mCurseurConstructionPossible;
		delete mCurseurConstructionImpossible;
		
		delete mMonde;
		
		delete mInfobulle;
		delete mChamp;
		
		delete mSousFenetreBatiment;
		delete mSousFenetreInventaire;
	}
	
	bool
	Interface::Charge()
	{
		mMonde->Charge();
		
		mSousFenetreBatiment->SetPersonnage(mPersonnage);
		mSousFenetreInventaire->SetObjets(mPersonnage->GetObjetsTransportes());
		if (mXml)
		{
			mSousFenetreBatiment->Charge();
			mSousFenetreInventaire->Charge();
		}
		else
		{
			mSousFenetreBatiment->SetPosition(0, 0);
			mSousFenetreInventaire->SetPosition(0, 0);
		}
		
		return true;
	}
	
	TiXmlElement*
	Interface::Sauvegarde()
	{
		TiXmlElement* retour = new TiXmlElement("Sauvegarde");
		retour->LinkEndChild( mSousFenetreBatiment->Sauvegarde() );
		retour->LinkEndChild( mSousFenetreInventaire->Sauvegarde() );
		retour->LinkEndChild( mMonde->Sauvegarde() );
		
		return retour;
	}
	
	void
	Interface::Dessine(sf::RenderTarget* const App) const
	{
		mMonde->Dessine(App);
		mChamp->Dessine(App);
		
		GS::TypeBatiment* typeBatiment;
		unsigned int nombreTypesBatiments = GestionnaireDonnees->GetNombreTypesBatiments();
		unsigned int i = 0;
		while (i < nombreTypesBatiments)
		{
			if ( (typeBatiment = GestionnaireDonnees->GetTypeBatiment(i)) )
				typeBatiment->DessineBouton(App);
			
			i++;
		}
		
		GS::Statut* statut;
		unsigned int nombreStatuts = GestionnaireDonnees->GetNombreStatuts();
		i = 0;
		while (i < nombreStatuts)
		{
			statut = GestionnaireDonnees->GetStatut(i);
			statut->Dessine(App);
			i++;
		}
		
		mSousFenetreInventaire->Dessine(App);
		mSousFenetreBatiment->Dessine(App);
		
		if (mBatimentAConstruire)
		{
			if (mPeutConstruire)
				App->Draw(*mCurseurConstructionPossible);
			else
				App->Draw(*mCurseurConstructionImpossible);
		}
		else
			App->Draw(*mCurseurBaseAff);
		
		mInfobulle->Dessine(App);
	}
	
	void
	Interface::CatchEvent(sf::Event& Event)
	{
		bool traite;
		
		switch (Event.Type)
		{
			case sf::Event::KeyPressed:
				if (!( traite = mChamp->Ecrit(Event.Key.Code) )) // diverses captures de touches
				{
					if (Event.Key.Code == sf::Key::Return)
					{
						mChamp->ReInitialise();
						mChamp->Affiche();
						mChamp->Active();
					}
					else if (Event.Key.Code == sf::Key::B)
						mSousFenetreBatiment->ChangeVisibilite();
					else if (Event.Key.Code == sf::Key::I)
						mSousFenetreInventaire->ChangeVisibilite();
				}
				break;
			case sf::Event::MouseMoved:
				mSourisX = Event.MouseMove.X; // enregistrement des positions de la souris
				mSourisY = Event.MouseMove.Y;
				
				mInfobulle->SetPosition(mSourisX + Prgm::Curseurs::Cote, mSourisY + Prgm::Curseurs::Cote);
				
				mSousFenetreBatiment->SetMousePosition(mSourisX, mSourisY);
				mSousFenetreInventaire->SetMousePosition(mSourisX, mSourisY);
				break;
			case sf::Event::MouseButtonPressed:
				if (!mSousFenetreBatiment->Catch(mSourisX, mSourisY, Event))
					mSousFenetreInventaire->Catch(mSourisX, mSourisY, Event);
				break;
			case sf::Event::MouseButtonReleased:
				if (!mSousFenetreBatiment->Catch(mSourisX, mSourisY, Event))
				{
					if (!mSousFenetreInventaire->Catch(mSourisX, mSourisY, Event))
					{
						switch (Event.MouseButton.Button)
						{
							case sf::Mouse::Left:
								{
									bool capture = false;
									GS::TypeBatiment* typeBatiment;
									unsigned int nombreTypesBatiments = GestionnaireDonnees->GetNombreTypesBatiments();
									unsigned int i = 0;
									while (i < nombreTypesBatiments && !capture)
									{
										if ( (typeBatiment = GestionnaireDonnees->GetTypeBatiment(i)) )
											capture = typeBatiment && typeBatiment->Catch(mSourisX, mSourisY);
										
										if (!capture)
											i++;
									}
									
									if (capture)
									{
										this->SetBatimentAConstruire(GestionnaireDonnees->GetTypeBatiment(i));
										mCurseurConstructionPossible->SetPosition(mSourisX + mDecalageCurseurSpriteX, mSourisY + mDecalageCurseurSpriteY);
										mCurseurConstructionImpossible->SetPosition(mSourisX + mDecalageCurseurSpriteX, mSourisY + mDecalageCurseurSpriteY);
									}
									else
									{
										if (mBatimentAConstruire)
										{
											if (mMonde->ConstruitBatiment(mSourisX + mDecalageCurseurSpriteX, mSourisY + mDecalageCurseurSpriteY, mBatimentAConstruire))
											{
												mBatimentAConstruire = NULL;
												
												mDecalageCurseurSpriteX = 0;
												mDecalageCurseurSpriteY = 0;
												
												mCurseurBaseAff->SetPosition(mSourisX + mDecalageCurseurSpriteX, mSourisY + mDecalageCurseurSpriteY);
											}
										}
										else
										{
											if (GS::Etre* cible = mMonde->GetEtrePoint(mSourisX, mSourisY))
												mPersonnage->Go(cible);
											else
												mPersonnage->Go(mSourisX - mMonde->GetImgX(), mSourisY - mMonde->GetImgY());
										}
									}
								}
								break;
							case sf::Mouse::Right:
								if (mBatimentAConstruire)
									this->SetBatimentAConstruire(NULL);
								break;
							default:
								break;
						}
						break;
					}
				}
				default:
					break;
			case sf::Event::MouseWheelMoved:
				mSousFenetreBatiment->Catch(mSourisX, mSourisY, Event);
				break;
		}
	}
	
	void
	Interface::DerouleRound(float tempsFrame)
	{
		mMonde->DerouleRound(tempsFrame);
		mMonde->SetVue(mPersonnage->GetX(), mPersonnage->GetY());
		
		mSousFenetreBatiment->ChargeBatiment(mPersonnage->GetBatimentActuel());
		
		GS::TypeBatiment* typeBatiment;
		bool capture = false;
		unsigned int nombreTypesBatiments = GestionnaireDonnees->GetNombreTypesBatiments();
		unsigned int i = 0;
		while (i < nombreTypesBatiments && !capture)
		{
			if ( (typeBatiment = GestionnaireDonnees->GetTypeBatiment(i)) )
				capture = typeBatiment && typeBatiment->Catch(mSourisX, mSourisY);
			
			if (!capture)
				i++;
		}
		if (capture)
		{
			mInfobulle->SetText(mPersonnage->GetTexteTypeBatiment(GestionnaireDonnees->GetTypeBatiment(i)));
			
			if (mPanneauBatimentsX < 0)
			{
				mPanneauBatimentsX += 5;
				
				if (mPanneauBatimentsX >= 0)
					mPanneauBatimentsX = 0;
				
				double nombreColones = nombreTypesBatiments / Prgm::Batiments::NombreLignesPanneau;
				unsigned int colone = 0;
				unsigned int ligne = 0;
				typeBatiment = NULL;;
				i = 0;
				while (i < nombreTypesBatiments)
				{
					if ( (typeBatiment = GestionnaireDonnees->GetTypeBatiment(i)) )
					{
						typeBatiment->SetPositionBouton(Prgm::Batiments::CoteBouton * colone + mPanneauBatimentsX, 50 + Prgm::Batiments::CoteBouton * ligne);
						
						colone++;
						if (colone >= nombreColones)
						{
							colone = 0;
							ligne++;
						}
					}
					
					i++;
				}
			}
		}
		else
		{
			std::string texte = mSousFenetreBatiment->GetText(mSourisX, mSourisY, mPersonnage);
			
			if (texte != "")
				mInfobulle->SetText(texte);
			else
			{
				GS::Etre* etre = mMonde->GetEtrePoint(mSourisX, mSourisY);
				if (etre)
					mInfobulle->SetText(mPersonnage->GetTexteEtre(etre));
				else
					mInfobulle->SetText("");
				
				if (mPanneauBatimentsX > mPanneauBatimentsXMin)
				{
					mPanneauBatimentsX -= 5;
					
					if (mPanneauBatimentsX < mPanneauBatimentsXMin)
						mPanneauBatimentsX = mPanneauBatimentsXMin;
					
					double nombreColones = nombreTypesBatiments / Prgm::Batiments::NombreLignesPanneau;
					unsigned int colone = 0;
					unsigned int ligne = 0;
					GS::TypeBatiment* typeBatiment = NULL;;
					i = 0;
					while (i < nombreTypesBatiments)
					{
						if ( (typeBatiment = GestionnaireDonnees->GetTypeBatiment(i)) )
						{
							typeBatiment->SetPositionBouton(Prgm::Batiments::CoteBouton * colone + mPanneauBatimentsX, 50 + Prgm::Batiments::CoteBouton * ligne);
							
							colone++;
							if (colone >= nombreColones)
							{
								colone = 0;
								ligne++;
							}
						}
						
						i++;
					}
				}
			}
		}
		
		if (mBatimentAConstruire)
		{
			mPeutConstruire = mMonde->GetConstructible(mSourisX + mDecalageCurseurSpriteX, mSourisY + mDecalageCurseurSpriteY, mBatimentAConstruire->GetLargeurBatiment(), mBatimentAConstruire->GetHauteurBatiment());
			
			if (mPeutConstruire)
				mCurseurConstructionPossible->SetPosition(mSourisX + mDecalageCurseurSpriteX, mSourisY + mDecalageCurseurSpriteY);
			else
				mCurseurConstructionImpossible->SetPosition(mSourisX + mDecalageCurseurSpriteX, mSourisY + mDecalageCurseurSpriteY);
		}
		else
			mCurseurBaseAff->SetPosition(mSourisX + mDecalageCurseurSpriteX, mSourisY + mDecalageCurseurSpriteY);
	}
	
	void
	Interface::SetBatimentAConstruire(GS::TypeBatiment* batiment)
	{
		mBatimentAConstruire = batiment;
		
		if (mBatimentAConstruire)
		{
			mDecalageCurseurSpriteX = -static_cast<int>( mBatimentAConstruire->GetLargeurBatiment() / 2 );
			mDecalageCurseurSpriteY = -static_cast<int>( mBatimentAConstruire->GetHauteurBatiment() / 2 );
			
			mPeutConstruire = mMonde->GetConstructible(static_cast<int>( mSourisX + mDecalageCurseurSpriteX ), static_cast<int>( mSourisY + mDecalageCurseurSpriteY ), mBatimentAConstruire->GetLargeurBatiment(), mBatimentAConstruire->GetHauteurBatiment());
			
			mCurseurConstructionPossible->Resize(mBatimentAConstruire->GetLargeurBatiment(), mBatimentAConstruire->GetHauteurBatiment());
			mCurseurConstructionPossible->SetPosition(mSourisX + mDecalageCurseurSpriteX, mSourisY + mDecalageCurseurSpriteY);
			
			mCurseurConstructionImpossible->Resize(mBatimentAConstruire->GetLargeurBatiment(), mBatimentAConstruire->GetHauteurBatiment());
			mCurseurConstructionImpossible->SetPosition(mSourisX + mDecalageCurseurSpriteX, mSourisY + mDecalageCurseurSpriteY);
		}
		else
		{
			mDecalageCurseurSpriteX = 0;
			mDecalageCurseurSpriteY = 0;
			
			mCurseurBaseAff->SetPosition(mSourisX + mDecalageCurseurSpriteX, mSourisY + mDecalageCurseurSpriteY);
		}
	}
}

#endif
