/**
* \file interface.hpp
* \brief Interface::Interface
* \author Yoha
* \version alpha-0.0.07
* \date Aot 2009
*
* Definition de l'interface
*
*/

#ifndef INTERFACE_HPP
#define INTERFACE_HPP

/// Mode dynamique
#define SFML_DYNAMIC

#include <vector>
#include <cstring>
#include <SFML/Graphics.hpp>
#include <SFML/Audio.hpp>
#include "tinyxml/tinyxml.h"

#include "personnage.hpp"
#include "monde.hpp"
#include "champ.hpp"
#include "typebatiment.hpp"
#include "texte.hpp"
#include "gamesystem.hpp"
#include "sfbatiment.hpp"
#include "sfinventaire.hpp"

/// Groupement de tous les elements concernant l'interface
namespace Interface
{
	/// Gestion de l'interface (affichage, capture des evenements)
	class Interface
	{
		public:
			/// \brief Constructeur
			Interface(const sf::Input* input, TiXmlElement* element = NULL);
			/// \brief Destructeur
			~Interface();
			
			/// \brief Charge une sauvegarde
			/// \param input Gestionnaire des touches clavier
			/// \param element Element contenant la sauvegarde
			bool Charge();
			/// \brief Sauvegarde
			TiXmlElement* Sauvegarde();
			
			/// \brief Methode de dessin  de l'interface
			/// \param App Pointeur sur la fenetre de rendu
			void Dessine(sf::RenderTarget* const App) const;
			/// \brief Capture un evenement SFML
			/// \param Event Evenement SFML
			void CatchEvent(sf::Event& Event);
			/// \brief Deroulement du round
			/// \param tempsFrame Duree de la frame
			void DerouleRound(float tempsFrame);
			
			/// \brief Definit le batiment que le joueur souhaite constuire
			/// \param batiment Type de batiment a constuire
			void SetBatimentAConstruire(GS::TypeBatiment* batiment);
			
		private:
			/// Sauvegarde de l'interface
			TiXmlElement* mXml;
			
			/// Sprite d'affichage du curseur de base
			sf::Sprite* mCurseurBaseAff;
			/// Sprite d'affichage du curseur de construction possible
			sf::Sprite* mCurseurConstructionPossible;
			/// Sprite d'affichage du curseur de construction possible
			sf::Sprite* mCurseurConstructionImpossible;
			/// Abscisse de la souris
			int mSourisX;
			/// Ordonnee de la souris
			int mSourisY;
			/// Decalage horizontal du curseur
			int mDecalageCurseurSpriteX;
			/// Decalage vertical du curseur
			int mDecalageCurseurSpriteY;
			/// Eventuel type de batiment a construire
			GS::TypeBatiment* mBatimentAConstruire;
			/// Memorise la possibilit de construction
			bool mPeutConstruire;
			
			/// Infobulle
			Texte* mInfobulle;
			
			/// Pointeur sur le gestionnaire d'evenement SFML
			const sf::Input* mInput;
			/// Pointeur sur le monde
			GS::Monde* mMonde;
			/// Personnage joueur
			GS::Personnage* mPersonnage;
			/// Pointeur sur le champ de parole
			Champ* mChamp;
			
			/// Abscisse du panneau lateral
			double mPanneauBatimentsX;
			/// Abscisse minimale du panneau lateral
			double mPanneauBatimentsXMin;
			
			/// Sous-fenetre de batiment
			SFBatiment* mSousFenetreBatiment;
			/// Sous-fenetre d'inventaire
			SFInventaire* mSousFenetreInventaire;
	};
}

#endif
