#ifndef MONDE_CPP
#define MONDE_CPP

#include "monde.hpp"

#include <cmath>
#include <cstring>
#include <sstream>

#include "projet.hpp"
#include "fonctions.hpp"
#include "gestionnaire.hpp"
#include "data.hpp"
#include "typeterrain.hpp"

#define NOMBRE_PERSONNAGES 5

namespace GS
{
	Monde::Monde(GS::Etre* parent, TiXmlElement* element) : Etre(parent, element)
	{
		mXvue = 0;
		mYvue = 0;
		mXSprite = 0;
		mYSprite = 0;
		mLargeur = 0;
		mHauteur = 0;
		mType = GS::Types::Monde;
		
		mNombreCasesTerrains = 0;
		mGrilleTerrains = NULL;
		mImage = NULL;
		mSpriteImage = NULL;
		
		mNombreEtres = 0;
		mNombreTerRessources = 0;
		mNombreBatiments = 0;
		mNombrePersonnages = 0;
		mNombreObjets = 0;
	}
	
	Monde::~Monde()
	{
		unsigned int i = 0;
		while (i < mNombreEtres)
		{
			delete mEtres[i];
			
			i++;
		}
	}
	
	void
	Monde::Charge()
	{
		if (mXml)
		{
			ChargeXmlValeur(mXml, "LargeurCarte", mLargeur);
			ChargeXmlValeur(mXml, "HauteurCarte", mHauteur);
		}
		else
		{
			mLargeur = Prgm::Terrains::CoteCarte;
			mHauteur = Prgm::Terrains::CoteCarte;
		}
		mNombreCasesTerrains = mLargeur * mHauteur;
		
		// creation du terrain
		{
			std::string stringTerrains;
			const char* chaineTerrains = NULL;
			char caractere = 0;
			unsigned int rang = 0;
			unsigned int longueur = 0;
			std::ostringstream identifiant;
			if (mXml)
			{
				ChargeXmlValeur(mXml, "GrilleTerrains", stringTerrains);
				chaineTerrains = stringTerrains.c_str();
				
				longueur = strlen(chaineTerrains);
			}
			
			mGrilleTerrains = new unsigned int[mNombreCasesTerrains];
			mImage = new sf::Image(Prgm::Terrains::LargeurBoutDeMonde, Prgm::Terrains::HauteurBoutDeMonde);
			mSpriteImage = new sf::Sprite(*mImage);
			
			bool continuer;
			unsigned int nbTypesTerrains = GestionnaireDonnees->GetNombreTypesTerrains();
			TypeTerrain* typeTer = NULL;
			sf::Sprite* spriteTer;
			sf::Color color;
			
			unsigned int i = 0;
			unsigned int j;
			unsigned int k;
			unsigned int l;
			while (i < Prgm::Terrains::CoteCarte)
			{
				j = 0;
				while (j < Prgm::Terrains::CoteCarte)
				{
					k = 0;
					
					if (mXml)
					{
						identifiant.str("");
						while (rang < longueur && caractere != '/')
						{
							caractere = chaineTerrains[rang];
							
							if (caractere != '/')
								identifiant << caractere;
							rang++;
						}
						caractere = 0;
						k = atoi(identifiant.str().c_str());
						typeTer = GestionnaireDonnees->GetTypeTerrain(k);
					}
					else
					{
						continuer = true;
						while (k < nbTypesTerrains && continuer)
						{
							typeTer = GestionnaireDonnees->GetTypeTerrain(k);
							if (typeTer->GetProbabilite() >= sf::Randomizer::Random(0.f, 1.f) )
								continuer = false;
							else
								k++;
						}
					}
					
					if (k >= nbTypesTerrains)
						k = 0;
					
					mGrilleTerrains[i * Prgm::Terrains::CoteCarte + j] = k;
					
					if (typeTer)
					{
						if (typeTer->TestSprite())
						{
							spriteTer = typeTer->GetSprite();
							spriteTer->Resize(Prgm::Terrains::Largeur, Prgm::Terrains::Hauteur);
							k = 0;
							while (k < Prgm::Terrains::Largeur)
							{
								l = 0;
								while (l < Prgm::Terrains::Hauteur)
								{
									color = spriteTer->GetPixel(k, l);
									mImage->SetPixel(i * Prgm::Terrains::Largeur + k, j * Prgm::Terrains::Hauteur + l, color); // remplissage du terrain
									
									l++;
								}
								k++;
							}
						}
					}
					
					j++;
				}
				
				i++;
			}
		}
		
		// creation des etres
		{
			double abscisse;
			double ordonnee;
			if (mXml)
			{
				TiXmlElement* xmlEtres = mXml->FirstChildElement("Etres");
				
				TiXmlElement* xmlTerRessouces = NULL;
				if (xmlEtres)
					xmlTerRessouces = xmlEtres->FirstChildElement("TerRessource");
				
				while (xmlTerRessouces)
				{
					mTerRessources.resize(mNombreTerRessources + 1);
					
					mTerRessources[mNombreTerRessources] = new GS::TerRessource(this, xmlTerRessouces);
					this->AjouteEtre(mTerRessources[mNombreTerRessources]);
					
					mNombreTerRessources++;
					
					xmlTerRessouces = xmlTerRessouces->NextSiblingElement("TerRessource");
				}
				
				
				TiXmlElement* xmlPersonnages = NULL;
				if (xmlEtres)
					xmlPersonnages = xmlEtres->FirstChildElement("Personnage");
				
				while (xmlPersonnages)
				{
					mPersonnages.resize(mNombrePersonnages + 1);
					
					mPersonnages[mNombrePersonnages] = new GS::Personnage(this, xmlPersonnages, "script.lua");
					this->AjouteEtre(mPersonnages[mNombrePersonnages]);
					
					mNombrePersonnages++;
					
					if (xmlPersonnages)
						xmlPersonnages = xmlPersonnages->NextSiblingElement("Personnage");
				}
				
				
				TiXmlElement* xmlBatiments = NULL;
				if (xmlEtres)
					xmlBatiments = xmlEtres->FirstChildElement("Batiment");
				
				while (xmlBatiments)
				{
					mBatiments.resize(mNombreBatiments + 1);
					
					mBatiments[mNombreBatiments] = new GS::Batiment(this, xmlBatiments);
					this->AjouteEtre(mBatiments[mNombreBatiments]);
					
					mNombreBatiments++;
					
					if (xmlBatiments)
						xmlBatiments = xmlBatiments->NextSiblingElement("Batiment");
				}
				
				
				TiXmlElement* xmlObjets = NULL;
				if (xmlEtres)
					xmlObjets = xmlEtres->FirstChildElement("Objet");
				
				while (xmlObjets)
				{
					mObjets.resize(mNombreBatiments + 1);
					
					mObjets[mNombreObjets] = new GS::Objet(this, xmlObjets);
					this->AjouteEtre(mObjets[mNombreObjets]);
					
					mNombreObjets++;
					
					if (xmlObjets)
						xmlObjets = xmlObjets->NextSiblingElement("Objet");
				}
			}
			else
			{
				mTerRessources.resize(mNombreTerRessources);
				
				unsigned int nbTypesTerRessources = GestionnaireDonnees->GetNombreTypesTerRessources();
				
				while (mNombreTerRessources < 16)
				{
					abscisse = sf::Randomizer::Random(0.0, static_cast<float>(mLargeur * Prgm::Terrains::Largeur) );
					ordonnee = sf::Randomizer::Random(0.0, static_cast<float>(mHauteur * Prgm::Terrains::Hauteur) );
					
					if (this->GetPeutPlacerRessource(abscisse, ordonnee, Prgm::TerRessources::Largeur, Prgm::TerRessources::Hauteur))
					{
						mTerRessources.resize(mNombreTerRessources + 1);
						
						mTerRessources[mNombreTerRessources] = new GS::TerRessource(this);
						this->AjouteEtre(mTerRessources[mNombreTerRessources]);
						mTerRessources[mNombreTerRessources]->SetType( GestionnaireDonnees->GetTypeTerRessource( sf::Randomizer::Random(0, nbTypesTerRessources - 1) ) );
						mTerRessources[mNombreTerRessources]->SetPosition(abscisse, ordonnee);
						
						mNombreTerRessources++;
					}
				}
				
				while (mNombrePersonnages < NOMBRE_PERSONNAGES)
				{
					mPersonnages.resize(mNombrePersonnages + 1);
					
					mPersonnages[mNombrePersonnages] = new GS::Personnage(this, NULL, "script.lua");
					this->AjouteEtre(mPersonnages[mNombrePersonnages]);
					
					mNombrePersonnages++;
				}
				
				mBatiments.resize(0);
				mObjets.resize(0);
			}
			
			unsigned int i = 0;
			while (i < mNombreEtres)
			{
				if (mEtres[i])
					mEtres[i]->Charge();
				i++;
			}
		}
		
		*mPersonnageJoueur = mPersonnages[0];
	}
	
	TiXmlElement*
	Monde::Sauvegarde()
	{
		TiXmlElement* retour = new TiXmlElement("Monde");
		TiXmlElement* selement;
		TiXmlElement* sselement;
		
		selement = new TiXmlElement("LargeurCarte");
		retour->LinkEndChild(selement);
		selement->SetAttribute("valeur", mLargeur);
		
		selement = new TiXmlElement("HauteurCarte");
		retour->LinkEndChild(selement);
		selement->SetAttribute("valeur", mHauteur);
		
		std::ostringstream grilleTerrains;
		unsigned int i = 0;
		while (i < mNombreCasesTerrains)
		{
			if (mGrilleTerrains[i])
				grilleTerrains << mGrilleTerrains[i] << "/";
			else
				grilleTerrains << "/";
			
			i++;
		}
		selement = new TiXmlElement("GrilleTerrains");
		selement->SetAttribute("valeur", grilleTerrains.str().c_str());
		retour->LinkEndChild(selement);
		
		selement = new TiXmlElement("Etres");
		i = 0;
		while (i < mNombreEtres)
		{
			if (mEtres[i])
			{
				sselement = mEtres[i]->Sauvegarde();
				if (sselement)
					selement->LinkEndChild(sselement);
			}
			
			i++;
		}
		retour->LinkEndChild(selement);
		
		return retour;
	}
	
	void
	Monde::SetPersonnageJoueur(GS::Personnage** personnage)
	{
		mPersonnageJoueur = personnage;
	}
	
	void
	Monde::Dessine(sf::RenderTarget* const App)
	{
		mXSprite = Prgm::Window::MilieuX - mXvue;
		mYSprite = Prgm::Window::MilieuY - mYvue;
		
		mSpriteImage->SetPosition(mXSprite, mYSprite);
		
		App->Draw(*mSpriteImage);
		
		unsigned int i = 0;
		while (i < mNombreTerRessources)
		{
			mTerRessources[i]->Dessine(App, mXSprite, mYSprite);
			i++;
		}
		
		i = 0;
		while (i < mNombreBatiments)
		{
			mBatiments[i]->Dessine(App, mXSprite, mYSprite);
			i++;
		}
		
		i = 0;
		while (i < mNombrePersonnages)
		{
			mPersonnages[i]->Dessine(App, mXSprite, mYSprite);
			i++;
		}
	}

	void
	Monde::SetVue(const double x, const double y)
	{
		mXvue = x;
		mYvue = y;
	}
	
	void
	Monde::DerouleRound(float tempsFrame)
	{
		unsigned int i = 0;
		while (i < mNombrePersonnages)
		{
			mPersonnages[i]->DerouleRound(tempsFrame);
			i++;
		}
	}
	
	double
	Monde::GetImgX() const
	{
		return Prgm::Window::MilieuX - mXvue;
	}
	
	double
	Monde::GetImgY() const
	{
		return Prgm::Window::MilieuY - mYvue;
	}
	
	GS::Etre*
	Monde::GetEtrePoint(const double x, const double y)
	{
		double x_rel = x - mXSprite;
		double y_rel = y - mYSprite;
		
		if (0 <= x_rel && x_rel < Prgm::Terrains::LargeurBoutDeMonde && 0 <= y_rel && y_rel < Prgm::Terrains::HauteurBoutDeMonde)
		{
			bool capture = false;
			unsigned int i = 0;
			
			while (i < mNombreEtres && !capture)
			{
				if (mEtres[i])
					capture = mEtres[i]->Catch(x_rel, y_rel);
				
				if (!capture)
					i++;
			}
			
			if (capture)
				return mEtres[i];
			else
				return NULL;
		}
		else
			return NULL;
	}
	
	unsigned int
	Monde::GetIdTerrain(const double x, const double y) const
	{
		double x_rel = x - mXSprite;
		double y_rel = y - mYSprite;
		
		int maxX = Prgm::Terrains::LargeurBoutDeMonde;
		int maxY = Prgm::Terrains::HauteurBoutDeMonde;
		
		if (0 <= x_rel && x_rel < maxX && 0 <= y_rel && y_rel < maxY)
		{
			int TabX = static_cast<int>(x_rel / Prgm::Terrains::Largeur);
			int TabY = static_cast<int>(y_rel / Prgm::Terrains::Hauteur);
			
			return mGrilleTerrains[TabX * Prgm::Terrains::CoteCarte + TabY];
		}
		else
			return 0;
	}
	
	void
	Monde::AjouteEtre(GS::Etre* etre)
	{
		unsigned int id = etre->GetId();
		
		if (id >= mNombreEtres)
		{
			mNombreEtres = id + 1;
			mEtres.resize(mNombreEtres);
		}
		
		mEtres[id] = etre;
	}

	GS::Etre*
	Monde::GetEtre(const unsigned int num)
	{
		if (num < mNombreEtres)
			return mEtres[num];
		else
			return NULL;
	}
	
	GS::TerRessource*
	Monde::ChercheTerRessource(const double x, const double y, GS::TypeTerRessource* type) const
	{
		TerRessource* ressourceTemp = NULL;
		TerRessource* ressourcePlusProche = NULL;
		
		unsigned int i = 0;
		while (i < mNombreTerRessources)
		{
			ressourceTemp = mTerRessources[i];
			if (ressourceTemp->GetTypeTerRessource() == type && ( !ressourcePlusProche || CalculeDistance(x, y, ressourceTemp->GetX(), ressourceTemp->GetY()) < CalculeDistance(x, y, ressourcePlusProche->GetX(), ressourcePlusProche->GetY()) ))
				ressourcePlusProche = ressourceTemp;
			
			i++;
		}
		
		return ressourcePlusProche;
	}
	
	bool
	Monde::GetPeutPlacerRessource(const double x, const double y, const double largeur, const double hauteur) const
	{
		double x_rel = x;
		double y_rel = y;
		
		if (0 <= x_rel && x_rel < Prgm::Terrains::LargeurBoutDeMonde && 0 <= y_rel && y_rel < Prgm::Terrains::HauteurBoutDeMonde)
		{
			bool portable = true;
			
			unsigned int TabX = static_cast<unsigned int>(x_rel / Prgm::Terrains::Largeur);
			unsigned int TabY = static_cast<unsigned int>(y_rel / Prgm::Terrains::Hauteur);
			unsigned int TabYSauve = TabY;
			unsigned int MaxX = TabX + static_cast<unsigned int>(largeur / Prgm::Terrains::Largeur) - 1;
			unsigned int MaxY = TabY + static_cast<unsigned int>(hauteur / Prgm::Terrains::Hauteur) - 1 ;
			
			while (TabX < MaxX && portable)
			{
				TabY = TabYSauve;
				while (TabY < MaxY && portable)
				{
					portable = GestionnaireDonnees->GetTypeTerrain(mGrilleTerrains[TabX * Prgm::Terrains::CoteCarte + TabY])->GetPorteRessources();
					
					TabY++;
				}
				TabX++;
			}
			
			unsigned int i = 0;
			while (i < mNombreTerRessources && portable)
			{
				portable = !mTerRessources[i]->Catch(x_rel, y_rel);
				
				i++;
			}
			
			return portable;
		}
		else
			return false;
	}
	
	bool
	Monde::GetConstructible(const double x, const double y, const unsigned int largeur, const unsigned int hauteur) const
	{
		double x_rel = x - mXSprite;
		double y_rel = y - mYSprite;
		
		if (0 <= x_rel && x_rel + largeur <= Prgm::Terrains::LargeurBoutDeMonde && 0 <= y_rel && y_rel + hauteur <= Prgm::Terrains::HauteurBoutDeMonde)
		{
			bool constructible = true;
			
			unsigned int TabX = static_cast<unsigned int>(x_rel / Prgm::Terrains::Largeur);
			unsigned int TabY = static_cast<unsigned int>(y_rel / Prgm::Terrains::Hauteur);
			int TabYSauve = TabY;
			
			unsigned int MaxX = TabX + largeur / Prgm::Terrains::Largeur;
			if (MaxX > Prgm::Terrains::CoteCarte)
				MaxX = Prgm::Terrains::CoteCarte;
			
			unsigned int MaxY = TabY + hauteur / Prgm::Terrains::Hauteur;
			if (MaxY > Prgm::Terrains::CoteCarte)
				MaxY = Prgm::Terrains::CoteCarte;
			
			while (TabX <= MaxX && constructible)
			{
				TabY = TabYSauve;
				while (TabY <= MaxY && constructible)
				{
					constructible = GestionnaireDonnees->GetTypeTerrain(mGrilleTerrains[TabX * Prgm::Terrains::CoteCarte + TabY])->GetConstructible();
					
					TabY++;
				}
				
				TabX++;
			}
			
			unsigned int i = 0;
			while (i < mNombreTerRessources && constructible)
			{
				constructible = !mTerRessources[i]->GetConstructible(x_rel, y_rel, largeur, hauteur);
				
				i++;
			}
			
			i = 0;
			while (i < mNombreBatiments && constructible)
			{
				constructible = !mBatiments[i]->GetConstructible(x_rel, y_rel, largeur, hauteur);
				
				i++;
			}
			
			return constructible;
		}
		else
			return false;
	}
	
	GS::Batiment*
	Monde::ConstruitBatiment(const double x, const double y, GS::TypeBatiment* batimentAConstruire)
	{
		if (this->GetConstructible(x, y, batimentAConstruire->GetLargeurBatiment(), batimentAConstruire->GetHauteurBatiment()))
		{
			mBatiments.resize(mNombreBatiments + 1);
			
			mBatiments[mNombreBatiments] = new GS::Batiment(this);
			mBatiments[mNombreBatiments]->Charge();
			mBatiments[mNombreBatiments]->SetType(batimentAConstruire);
			mBatiments[mNombreBatiments]->SetPosition(x - mXSprite, y - mYSprite);
			this->AjouteEtre(mBatiments[mNombreBatiments]);
			
			return mBatiments[mNombreBatiments++];
		}
		else
			return NULL;
	}
	
	GS::Objet*
	Monde::CreerObjet()
	{
		mObjets.resize(mNombreObjets + 1);
		mObjets[mNombreObjets] = new GS::Objet(this);
		mObjets[mNombreObjets]->Charge();
		this->AjouteEtre(mObjets[mNombreObjets]);
		
		return mObjets[mNombreObjets++];
	}
}

#endif
