#ifndef PERSONNAGE_CPP
#define PERSONNAGE_CPP

#include "personnage.hpp"

#include <iostream>
#include <cmath>
#include <sstream>

#include "projet.hpp"
#include "gestionnaire.hpp"
#include "data.hpp"
#include "categorieobjet.hpp"
#include "fonctions.hpp"
#include "monde.hpp"

#define method(class, name) {#name, &class::name}

namespace GS
{
	const char Personnage::className[] = "Personnage";
	Lunar<Personnage>::RegType Personnage::methods[] =
		{
			method(Personnage, GetMonde),
			method(Personnage, Dire),
			method(Personnage, Aller),
			method(Personnage, GoRessource),
			{0, 0}
		};
	
	Personnage::Personnage(GS::Etre* parent, TiXmlElement* element, const char* scriptFile) : Etre(parent, element)
	{
		mType = Types::Personnage;
		
		ChargeLua(&mIA);
		if (strcmp(scriptFile,"") && luaL_dofile(mIA, scriptFile) != 0)
			CastErreur(lua_tostring(mIA, -1));
		
		mLargeur = Prgm::Personnage::Largeur;
		mHauteur = Prgm::Personnage::Hauteur;
	}
	
	Personnage::Personnage(lua_State* lua) : Etre(lua)
	{
	}
	
	Personnage::~Personnage()
	{
		delete mImagePerso;
		delete mAffichePerso;
		
		delete[] mQuantitesRessources;
		
		delete mPhrase;
	}
	
	void
	Personnage::Charge()
	{
		if (mXml)
		{
			ChargeXmlValeur(mXml, "Nom", mNom);
			ChargeXmlValeur(mXml, "Abscisse", mX);
			ChargeXmlValeur(mXml, "Ordonnee", mY);
			ChargeXmlValeur(mXml, "Direction", mDirection);
			ChargeXmlValeur(mXml, "Vitesse", mVitesse);
			ChargeXmlValeur(mXml, "Pas", mPas);
			ChargeXmlValeur(mXml, "Visible", mVisible);
			ChargeXmlValeur(mXml, "AbscisseCible", mCibleX);
			ChargeXmlValeur(mXml, "OrdonneeCible", mCibleY);
			
			if (!ChargeXmlValeur(mXml, "Cible", mCibleACharger))
				mCibleACharger = 0;
			
			if (!ChargeXmlValeur(mXml, "BatimentActuel", mBatimentACharger))
				mBatimentACharger = 0;
		}
		else
		{
			mNom = "Yoha";
			mDirection = 2;
			mX = 0;
			mY = 0;
			mVitesse = 100;
			mPas = 1;
			mVisible = true;
			mCibleX = 0;
			mCibleY = 0;
			mCibleACharger = 0;
			mBatimentACharger = 0;
		}
		
		mCible = NULL;
		mBatimentActuel = NULL;
		
		mMasseTransportee = 0;
		mMasseTransportable = 20;
		
		std::string stringRessources;
		const char* listeRessources = NULL;
		unsigned int longueur = 0;
		char caractere = 0;
		unsigned int rang = 0;
		if (mXml)
		{
			ChargeXmlValeur(mXml, "Ressources", stringRessources);
			listeRessources = stringRessources.c_str();
			longueur = strlen(listeRessources);
		}
		
		unsigned int l = GestionnaireDonnees->GetNombreTypesRessources();
		mQuantitesRessources = new double[l];
		unsigned int i = 0;
		while (i < l)
		{
			if (mXml)
			{
				std::ostringstream identifiant;
				
				identifiant.str("");
				while (rang < longueur && caractere != '/')
				{
					caractere = listeRessources[rang];
					if (caractere != '/')
						identifiant << caractere;
					rang++;
				}
				
				caractere = 0;
				if (strcmp(identifiant.str().c_str(), ""))
				{
					double quantite = atof(identifiant.str().c_str());
					if (quantite > 0)
						mQuantitesRessources[i] = quantite;
					else
						mQuantitesRessources[i] = 0;
				}
				else
					mQuantitesRessources[i] = 0;
			}
			else
				mQuantitesRessources[i] = 0;
			
			i++;
		}
		
		mImagePerso = new sf::Image(*Prgm::Personnage::ImageDefaut);
		
		mAffichePerso = new sf::Sprite();
		mAffichePerso->SetImage(*mImagePerso);
		mAffichePerso->SetSubRect( sf::IntRect( static_cast<int>(mPas) * Prgm::Personnage::LargeurSprite, mDirection * Prgm::Personnage::HauteurSprite, static_cast<int>(mPas + 1) * Prgm::Personnage::LargeurSprite, (mDirection + 1) * Prgm::Personnage::HauteurSprite ) );
		mAffichePerso->Resize(mLargeur, mHauteur);
		
		mPhrase = new Texte();
		
		std::string stringListeObjets;
		const char* listeObjets = NULL;
		if (mXml)
		{
			ChargeXmlValeur(mXml, "Objets", stringListeObjets);
			listeObjets = stringListeObjets.c_str();
			longueur = strlen(listeObjets);
		}
		else
		{
			longueur = 0;
			rang = 0;
			caractere = 0;
		}
		
		unsigned int nombreCategories = GestionnaireDonnees->GetNombreCategoriesObjets() + 1;
		unsigned int nombrePages = 1;
		
		unsigned int categorie = 0;
		unsigned int page;
		unsigned int ligne;
		unsigned int colone;
		
		mObjetsTransportes.resize(nombreCategories); // par categorie (+ tous les objets)
		while (categorie < nombreCategories)
		{
			mObjetsTransportes[categorie].resize(nombrePages); // par page
			
			page = 0;
			while (page < nombrePages)
			{
				mObjetsTransportes[categorie][page].resize(Prgm::SFenetres::Inventaire::NombreLignes); // par ligne
				
				ligne = 0;
				while (ligne < Prgm::SFenetres::Inventaire::NombreLignes)
				{
					mObjetsTransportes[categorie][page][ligne].resize(Prgm::SFenetres::Inventaire::NombreColones); // par colone
					
					colone = 0;
					while (colone < Prgm::SFenetres::Inventaire::NombreColones)
					{
						if (mXml)
						{
							std::ostringstream identifiant;
							
							identifiant.str("");
							while (rang < longueur && caractere != '/')
							{
								caractere = listeObjets[rang];
								if (caractere != '/')
									identifiant << caractere;
								rang++;
							}
							
							caractere = 0;
							if (strcmp(identifiant.str().c_str(), ""))
							{
								unsigned int id = atoi(identifiant.str().c_str());
								GS::Etre* etre = static_cast<GS::Monde*>(mPere)->GetEtre(id);
								
								if (etre && etre->GetType() == GS::Types::Objet)
									mObjetsTransportes[categorie][page][ligne][colone] = static_cast<GS::Objet*>(etre);
							}
						}
						
						colone++;
					}
					
					ligne++;
				}
				
				if (rang < longueur)
				{
					nombrePages++;
					mObjetsTransportes[categorie].resize(nombrePages);
				}
				
				page++;
			}
			
			categorie++;
		}
		
		if (!mXml)
		{
			Lunar<Personnage>::push(mIA, this);
			Lunar<Personnage>::call(mIA, "onApparait", 0, 0);
		}
	}
	
	TiXmlElement*
	Personnage::Sauvegarde()
	{
		TiXmlElement* retour = new TiXmlElement("Personnage");
		TiXmlElement* selement;
		std::ostringstream chaineRetour;
		
		retour->SetAttribute("id", mIdentifiant);
		
		selement = new TiXmlElement("Nom");
		retour->LinkEndChild(selement);
		selement->SetAttribute("valeur", mNom.c_str());
		
		selement = new TiXmlElement("Direction");
		retour->LinkEndChild(selement);
		selement->SetAttribute("valeur", mDirection);
		
		selement = new TiXmlElement("Abscisse");
		retour->LinkEndChild(selement);
		selement->SetDoubleAttribute("valeur", mX);
		
		selement = new TiXmlElement("Ordonnee");
		retour->LinkEndChild(selement);
		selement->SetDoubleAttribute("valeur", mY);
		
		selement = new TiXmlElement("Vitesse");
		retour->LinkEndChild(selement);
		selement->SetDoubleAttribute("valeur", mVitesse);
		
		selement = new TiXmlElement("Pas");
		retour->LinkEndChild(selement);
		selement->SetDoubleAttribute("valeur", mPas);
		
		selement = new TiXmlElement("Visible");
		retour->LinkEndChild(selement);
		selement->SetAttribute("valeur", mVisible);
		
		selement = new TiXmlElement("AbscisseCible");
		retour->LinkEndChild(selement);
		selement->SetDoubleAttribute("valeur", mCibleX);
		
		selement = new TiXmlElement("OrdonneeCible");
		retour->LinkEndChild(selement);
		selement->SetDoubleAttribute("valeur", mCibleY);
		
		if (mCible)
		{
			selement = new TiXmlElement("Cible");
			retour->LinkEndChild(selement);
			selement->SetAttribute("valeur", mCible->GetId());
		}
		
		if (mBatimentActuel)
		{
			selement = new TiXmlElement("BatimentActuel");
			retour->LinkEndChild(selement);
			selement->SetAttribute("valeur", mBatimentActuel->GetId());
		}
		
		chaineRetour.str("");
		unsigned int l = GestionnaireDonnees->GetNombreTypesRessources();
		unsigned int i = 0;
		while (i < l)
		{
			if (mQuantitesRessources[i] >= 0.01)
				chaineRetour << mQuantitesRessources[i] << "/";
			else
				chaineRetour << "/";
			i++;
		}
		selement = new TiXmlElement("Ressources");
		retour->LinkEndChild(selement);
		selement->SetAttribute("valeur", chaineRetour.str().c_str());
		
		chaineRetour.str("");
		unsigned int nombrePages = mObjetsTransportes[0].size();
		unsigned int nombreLignes;
		unsigned int nombreColones;
		unsigned int page = 0;
		unsigned int ligne;
		unsigned int colone;
		while (page < nombrePages)
		{
			nombreLignes = mObjetsTransportes[0][page].size();
			ligne = 0;
			while (ligne < nombreLignes)
			{
				colone = 0;
				nombreColones = mObjetsTransportes[0][page][ligne].size();
				while (colone < nombreColones)
				{
					if (mObjetsTransportes[0][page][ligne][colone])
						chaineRetour << mObjetsTransportes[0][page][ligne][colone]->GetId() << "/";
					else
						chaineRetour << "/";
					
					colone++;
				}
				ligne++;
			}
			page++;
		}
		selement = new TiXmlElement("Objets");
		retour->LinkEndChild(selement);
		selement->SetAttribute("valeur", chaineRetour.str().c_str());
		
		return retour;
	}
	
	bool
	Personnage::Catch(const double x, const double y)
	{
		return mX + Prgm::Terrains::Largeur / 2 - mLargeur / 2 <= x && x < mX + Prgm::Terrains::Largeur / 2 + mLargeur / 2 && mY + Prgm::Terrains::Hauteur - mHauteur <= y && y <= mY + Prgm::Terrains::Hauteur;
	}
	
	void
	Personnage::RecoitChaine(const std::string txt)
	{
		mPhrase->SetText(txt);
	}
	
	void
	Personnage::DerouleRound(const float tempsFrame)
	{
		if (mCibleACharger)
		{
			mCible = static_cast<GS::Monde*>(mPere)->GetEtre(mCibleACharger);
			mCibleACharger = 0;
		}
		
		if (mBatimentACharger)
		{
			GS::Etre* etre;
			etre = static_cast<GS::Monde*>(mPere)->GetEtre(mBatimentACharger);
			if (etre->GetType() == GS::Types::Batiment)
				mBatimentActuel = static_cast<GS::Batiment*>(etre);
			
			mBatimentACharger = 0;
		}
		
		double deplacementFinal = GestionnaireDonnees->GetTypeTerrain( static_cast<GS::Monde*>(mPere)->GetIdTerrain(mX + mLargeur / 2, mY + Prgm::Terrains::Hauteur) )->GetFacteurVitesse() * mVitesse * tempsFrame;
		
		if (mCibleX || mCibleY) // si le personnage est a une distance non nulle de la cible
		{
			if (mBatimentActuel)
			{
				// Le personnage sort d'un batiment
				mBatimentActuel = NULL;
				mVisible = true;
				
				lua_getglobal(mIA, "onSortBatiment");
				if (lua_isfunction(mIA, -1))	
				{
					Lunar<Personnage>::push(mIA, this);
					lua_call(mIA, 1, 0);
				}
			}
			
			// Scripts de mouvements
			lua_getglobal(mIA, "onBouge");
			if (lua_isfunction(mIA, -1))	
			{
				Lunar<Personnage>::push(mIA, this);
				lua_call(mIA, 1, 0);
			}
			
			double angleDirection;
			
			if (mCibleX)
				angleDirection = atan( mCibleY / mCibleX );
			else
				angleDirection = DPI;
			if (mCibleX < 0)
				angleDirection += PI;
			else if (mCibleY < 0)
			{
				if (mCibleX)
					angleDirection += PI2;
				else
					angleDirection += PI;
			}
			
			double depX = deplacementFinal * cos(angleDirection);
			if ( (depX < mCibleX && mCibleX < 0) || (0 <= mCibleX && mCibleX < depX) )
				depX = mCibleX;
			mCibleX -= depX;
			mX += depX;
			
			double depY = deplacementFinal * sin(angleDirection);
			if ( (depY < mCibleY && mCibleY < 0) || (0 <= mCibleY && mCibleY < depY) )
				depY = mCibleY;
			mCibleY -= depY;
			mY += depY;
			
			mPas += Prgm::Personnage::PasParSeconde * tempsFrame;
			if (mPas >= 4)
				mPas = 0;
			
			if (mCibleX == 0 && mCibleY == 0) // si le personnage atteint la cible
			{
				lua_getglobal(mIA, "onStoppe");
				if (lua_isfunction(mIA, -1))	
				{
					Lunar<Personnage>::push(mIA, this);
					lua_call(mIA, 1, 0);
				}
				
				if (mCible)
				{
					angleDirection = DPI;
					mPas = 1;
				}
				else // si le personnage s'arrete
				{
					mPas = 1;
					
					lua_getglobal(mIA, "onArrete");
					if (lua_isfunction(mIA, -1))	
					{
						Lunar<Personnage>::push(mIA, this);
						lua_call(mIA, 1, 0);
					}
				}
			}
			
			mDirection = static_cast<int>((angleDirection + QPI) / DPI) + 1;
			if (mDirection >= 4)
				mDirection = 0;
			
			if (static_cast<int>(mPas) == 3)
				mAffichePerso->SetSubRect(sf::IntRect(Prgm::Personnage::LargeurSprite , mDirection * Prgm::Personnage::HauteurSprite, 2 * Prgm::Personnage::LargeurSprite, (mDirection + 1) * Prgm::Personnage::HauteurSprite));
			else
				mAffichePerso->SetSubRect( sf::IntRect( static_cast<int>( floor(mPas) * Prgm::Personnage::LargeurSprite ), mDirection * Prgm::Personnage::HauteurSprite, static_cast<int>( floor(mPas + 1) * Prgm::Personnage::LargeurSprite ), (mDirection + 1) * Prgm::Personnage::HauteurSprite ) );
		}
		else if (mCible) // si le personnage se dirige vers un objet precis
		{
			if (mCible->GetType() == Types::Batiment)
			{
				Batiment* batiment = static_cast<Batiment*>(mCible);
				
				if (batiment->Construit(this, tempsFrame))
				{
					// Le personnage entre dans un batiment
					mVisible = false;
					mBatimentActuel = batiment;
					
					// transmission aux scripts
					lua_getglobal(mIA, "onEntreBatiment");
					if (lua_isfunction(mIA, -1))
					{
						Lunar<Personnage>::push(mIA, this);
						Lunar<Batiment>::push(mIA, mBatimentActuel);
						lua_call(mIA, 2, 0);
					}
					
					mCible = NULL;
				}
			}
			else if (mCible->GetType() == Types::Ressource)
			{
				TerRessource* cibleRessource = static_cast<TerRessource*>(mCible);
				cibleRessource->CollecteRessources(this, tempsFrame);
			}
		}
		else if (mBatimentActuel)
		{
			mBatimentActuel->Travaille(this, tempsFrame);
		}
	}
	
	void
	Personnage::Dessine(sf::RenderTarget* const App, const double x, const double y) const
	{
		if (mVisible)
		{
			mAffichePerso->SetPosition(mX + x + Prgm::Terrains::Largeur / 2 - mLargeur / 2,
				mY + y + Prgm::Terrains::Hauteur - mHauteur);
			App->Draw(*mAffichePerso);
		}
		
		mPhrase->SetPosition(mX + x + mLargeur / 2 - mPhrase->GetRect().GetWidth() / 2,
			mY + y + Prgm::Terrains::Hauteur - mHauteur - mPhrase->GetRect().GetHeight());
		mPhrase->Dessine(App);
	}
	
	void
	Personnage::Go(Etre* cible)
	{
		if (cible)
		{
			mCible = cible;
			if (mCible->GetType() == Types::Ressource || mCible->GetType() == Types::Batiment)
			{
				mCibleX = mCible->GetX() - mX + mCible->GetLargeur() / 2;
				mCibleY = mCible->GetY() - mY + mCible->GetHauteur() / 2;
			}
		}
	}
	
	void
	Personnage::Go(const double x, const double y)
	{
		mCibleX = x - mX - mLargeur / 2;
		mCibleY = y - mY - mHauteur / 2;
		mCible = NULL;
	}
	
	Batiment*
	Personnage::ConstruitBatiment(const double x, const double y, TypeBatiment* batimentAConstruire)
	{
		return static_cast<GS::Monde*>(mPere)->ConstruitBatiment(x, y, batimentAConstruire);
	}
	
	std::string
	Personnage::GetNom() const
	{
		return mNom;
	}
	
	GS::Batiment*
	Personnage::GetBatimentActuel() const
	{
		return mBatimentActuel;
	}
	
	std::string
	Personnage::GetTexteTypeBatiment(const TypeBatiment* typeBatiment) const
	{
		std::ostringstream chaineRetour;
		
		chaineRetour << typeBatiment->GetNom();
		chaineRetour << "\n" << typeBatiment->GetDescription();
		
		unsigned int longueur = typeBatiment->GetNombreTypesRessourcesNecessaires();
		unsigned int i = 0;
		unsigned int numRessource;
		while (i < longueur)
		{
			numRessource = typeBatiment->GetTypeRessourceNecessaire(i);
			chaineRetour << "\n" << NormaliseRessource(mQuantitesRessources[numRessource]) << "/" << typeBatiment->GetQuantiteRessourceNecessaire(i) << " " << GestionnaireDonnees->GetTypeRessource(numRessource)->GetNom();
			i++;
		}
		
		return chaineRetour.str();
	}
	
	std::string
	Personnage::GetTexteEtre(GS::Etre* etre) const
	{
		if (etre->GetType() == GS::Types::Personnage)
		{
			return static_cast<GS::Personnage*>(etre)->GetNom();
		}
		else if (etre->GetType() == GS::Types::Batiment)
		{
			GS::Batiment* batiment = static_cast<GS::Batiment*>(etre);
			GS::TypeBatiment* typeBatiment = batiment->GetTypeBatiment();
			
			std::ostringstream chaineRetour;
			
			chaineRetour << typeBatiment->GetNom();
			
			if (!batiment->GetConstruit())
			{
				chaineRetour << "\nRessources necessaires:";
				
				unsigned int longueur = typeBatiment->GetNombreTypesRessourcesNecessaires();
				unsigned int i = 0;
				double quantiteNecessaire;
				while (i < longueur)
				{
					quantiteNecessaire = typeBatiment->GetQuantiteRessourceNecessaire(i) - batiment->GetQuantitesRessourceConstruction(i);
					
					if (quantiteNecessaire)
						chaineRetour << "\n" << NormaliseRessource(quantiteNecessaire) << " " << GestionnaireDonnees->GetTypeRessource(typeBatiment->GetTypeRessourceNecessaire(i))->GetNom();
					
					i++;
				}
			}
			
			return chaineRetour.str();
		}
		else if (etre->GetType() == GS::Types::Ressource)
		{
			GS::TerRessource* terRessource = static_cast<GS::TerRessource*>(etre);
			GS::TypeRessource* typeRessource = terRessource->GetTypeTerRessource()->GetTypeRessource();
			
			std::ostringstream chaineRetour;
			chaineRetour << typeRessource->GetNom() << "(" << NormaliseRessource( mQuantitesRessources[typeRessource->GetId()] ) << ")";
			return chaineRetour.str();
		}
		else if (etre->GetType() == GS::Types::Objet)
		{
			return static_cast<GS::Objet*>(etre)->GetTypeObjet()->GetNom();
		}
		else
			return "";
	}
	
	double Personnage::GetQuantiteRessource(const unsigned int idRessource) const
	{
		return mQuantitesRessources[idRessource];
	}
	
	double
	Personnage::AjouteRessource(const unsigned int idRessource, const double quantite)
	{
		double max = 20 - mQuantitesRessources[idRessource];
			
		/*
		double masseUnitaire = GestionnaireDonnees->GetTypeRessource(idRessource)->GetMasseUnitaire();
		double poidsAjoutable = mMasseTransportable - mMasseTransportee;
		double aAjouter = masseUnitaire * quantite;
		*/
		
		double quantiteRecue = quantite < max ? quantite : max;
		mQuantitesRessources[idRessource] += quantiteRecue;
		return quantite - quantiteRecue;
		
	}
	
	double
	Personnage::EnleveRessource(const unsigned int idRessource, const double quantite)
	{
		double quantiteEnlevee = mQuantitesRessources[idRessource] >= quantite ? quantite : mQuantitesRessources[idRessource];
		mQuantitesRessources[idRessource] -= quantiteEnlevee;
		mMasseTransportee -= GestionnaireDonnees->GetTypeRessource(idRessource)->GetMasseUnitaire() * quantiteEnlevee;
		
		return quantiteEnlevee;
	}
	
	std::vector < std::vector < std::vector < std::vector <GS::Objet*> > >	>*
	Personnage::GetObjetsTransportes()
	{
		return &mObjetsTransportes;
	}
	
	bool
	Personnage::AjouteObjet(Objet* objet)
	{
		unsigned int idCategorie = objet->GetTypeObjet()->GetCategorieObjet()->GetId() + 1;
		
		unsigned int x = 0;
		unsigned int y = 0;
		unsigned int page = 0;
		bool emplacementGlobalTrouve = false;
		bool emplacementCategorieTrouve = false;
		while (!emplacementGlobalTrouve || !emplacementCategorieTrouve)
		{
			if (y != Prgm::SFenetres::Inventaire::NombreLignes - 1 || (x != 0 && x != Prgm::SFenetres::Inventaire::NombreColones - 1))
			{
				if (!emplacementGlobalTrouve && mObjetsTransportes[0][page][y][x] == NULL)
				{
					mObjetsTransportes[0][page][y][x] = objet;
					emplacementGlobalTrouve = true;
				}
				
				if (!emplacementCategorieTrouve && mObjetsTransportes[idCategorie][page][y][x] == NULL)
				{
					mObjetsTransportes[idCategorie][page][y][x] = objet;
					emplacementCategorieTrouve = true;
				}
			}
			
			x++;
			if (x >= Prgm::SFenetres::Inventaire::NombreColones)
			{
				x = 0;
				y++;
				
				if (y >= Prgm::SFenetres::Inventaire::NombreLignes)
				{
					y = 0;
					page++;
					
					if (!emplacementGlobalTrouve && page >= mObjetsTransportes[0].size())
					{
						mObjetsTransportes[0].resize(page + 1);
						
						mObjetsTransportes[0][page].resize(Prgm::SFenetres::Inventaire::NombreLignes);
						
						y = 0;
						while (y < Prgm::SFenetres::Inventaire::NombreLignes)
						{
							mObjetsTransportes[0][page][y].resize(Prgm::SFenetres::Inventaire::NombreColones);
							
							x = 0;
							while (x < Prgm::SFenetres::Inventaire::NombreColones)
							{
								mObjetsTransportes[0][page][y][x] = NULL;
								x++;
							}
							
							y++;
						}
					}
					
					if (!emplacementCategorieTrouve && page >= mObjetsTransportes[idCategorie].size())
					{
						mObjetsTransportes[idCategorie].resize(page + 1);
						
						mObjetsTransportes[idCategorie][page].resize(Prgm::SFenetres::Inventaire::NombreLignes);
						
						y = 0;
						while (y < Prgm::SFenetres::Inventaire::NombreLignes)
						{
							mObjetsTransportes[idCategorie][page][y].resize(Prgm::SFenetres::Inventaire::NombreColones);
							
							x = 0;
							while (x < Prgm::SFenetres::Inventaire::NombreColones)
							{
								mObjetsTransportes[idCategorie][page][y][x] = NULL;
								x++;
							}
							
							y++;
						}
					}
					
					x = 0;
					y = 0;
				}
			}
		}
		
		return true;
	}
	
	int
	Personnage::GetMonde(lua_State* L)
	{
		if (lua_gettop(L) == 0)
		{
			Lunar<Personnage>::push(L, this, false);
			return 1;
		}
		else
		{
			std::cout << "Appel de la fonction Personnage::Monde avec " << lua_gettop(L) << " argument(s) (0 attendu)";
			return 0;
		}
	}
	
	int
	Personnage::Dire(lua_State* L)
	{
		if (lua_gettop(L) == 1)
			mPhrase->SetText(lua_tostring(L, 1));
		else
			std::cout << "Appel de la fonction Personnage::Parler avec " << lua_gettop(L) << " argument(s) (1 attendu)";
		
		return 0;
	}
	
	int
	Personnage::Aller(lua_State* L)
	{
		if (lua_gettop(L) == 2)
		{
			mCibleX = lua_tonumber(L, 1);
			mCibleY = lua_tonumber(L, 2);
		}
		else
			std::cout << "Appel de la fonction Personnage::Go avec " << lua_gettop(L) << " argument(s) (2 attendus)";
		
		return 0;
	}
	
	int
	Personnage::GoRessource(lua_State* L)
	{
		if (lua_gettop(L) == 1)
			this->Go(static_cast<GS::Monde*>(mPere)->ChercheTerRessource(mX, mY, GestionnaireDonnees->GetTypeTerRessource(static_cast<unsigned int>(lua_tonumber(L, 1) - 1))));
		else
			std::cout << "Appel de la fonction Personnage::GoRessource avec " << lua_gettop(L) << " argument(s) (1 attendu)";
		
		return 0;
	}
}

#endif
