#ifndef PROJET_CPP
#define PROJET_CPP

#include <sstream>
#include <sys/types.h>
#include <dirent.h>
#include <SFML/System.hpp>

#include "projet.hpp"
#include "gestionnaire.hpp"
#include "data.hpp"
#include "monde.hpp"
#include "personnage.hpp"
#include "champ.hpp"
#include "interface.hpp"

bool interfaceChargee = false;

void ChargeInterface(void* voidSeed)
{
	if (voidSeed)
	{
		unsigned int* seed = static_cast<unsigned int*>(voidSeed);
		sf::Randomizer::SetSeed(*seed);
	}
	
	GestionnaireDonnees->ChargeXml();
	
	TiXmlDocument* sauvegarde = new TiXmlDocument();
	if (sauvegarde->LoadFile("./Donnees/Sauvegardes/test.xml"))
		Prgm::ElementsInterface::interface = new Interface::Interface(Prgm::ElementsInterface::Input, sauvegarde->FirstChildElement("Sauvegarde"));
	else
		Prgm::ElementsInterface::interface = new Interface::Interface(Prgm::ElementsInterface::Input);
	
	interfaceChargee = Prgm::ElementsInterface::interface->Charge();
}

int main()
{
	std::cout << "Chargement en cours...\n";
	
	// initialisation de la fenetre
		sf::VideoMode configEcran = sf::VideoMode::GetDesktopMode();
		sf::RenderWindow* App = new sf::RenderWindow(configEcran, "Vendetta version alpha 0.0.09.0", sf::Style::None);
		App->SetFramerateLimit(0);
		App->ShowMouseCursor(false);
		App->UseVerticalSync(false);
	
	// Icne
		sf::Image* icone = new sf::Image();
		if (icone->LoadFromFile("./Donnees/Images/icone.png"))
			App->SetIcon(16, 16, icone->GetPixelsPtr());
	
	// mise en memoire des informations pour accelerer l'acces aux donnees
		Prgm::Window::App = static_cast<sf::RenderTarget*>(App);
		Prgm::Window::Width = configEcran.Width;
		Prgm::Window::Height = configEcran.Height;
		Prgm::Window::MilieuX = Prgm::Window::Width / 2;
		Prgm::Window::MilieuY = Prgm::Window::Height / 2;
	
	// chargement des images d'attente du chargement de l'interface
		struct dirent* lecture = NULL;
		DIR* rep = opendir("./Donnees/Images/Chargement/");
		unsigned int l = 0;
		while (readdir(rep)) l++;
		l = sf::Randomizer::Random(2, l - 2);
		seekdir(rep, l);
		lecture = readdir(rep);
		
		std::ostringstream fichierChargement;
		fichierChargement << "./Donnees/Images/Chargement/" << lecture->d_name;
		closedir(rep);
		
		sf::Image* ImageFondChargement = new sf::Image(1, 1, sf::Color(255, 0, 0));
		sf::Sprite* SpriteFondChargement = new sf::Sprite(*ImageFondChargement);
		
		sf::Image* ImageChargement = new sf::Image();
		ImageChargement->LoadFromFile(fichierChargement.str());
		sf::Sprite* SpriteChargement = new sf::Sprite(*ImageChargement);
		SpriteChargement->Resize(Prgm::Window::Width, Prgm::Window::Height);
	
	// chargement de l'interface
		sf::Event Event;
		const sf::Input& Input = App->GetInput();
		Prgm::ElementsInterface::Input = &Input;
		unsigned int seed = sf::Randomizer::Random(0, 100);
		sf::Thread threadChargeInterface(&ChargeInterface, &seed);
		threadChargeInterface.Launch();
	
	// boucle principale
		sf::Clock* chronoFrame = new sf::Clock();
		bool boucler = true;
		while (boucler)
		{
			chronoFrame->Reset();
			App->Clear();
			
			if (interfaceChargee)
				Prgm::ElementsInterface::interface->Dessine(App);
			else
			{
				if (Prgm::ElementsInterface::AvancementChargement)
				{
					ImageFondChargement->SetPixel(0, 0, sf::Color((sf::Uint8)(127 - 127 * Prgm::ElementsInterface::AvancementChargement), (sf::Uint8)(255 * Prgm::ElementsInterface::AvancementChargement), 0));
					SpriteFondChargement->Resize(Prgm::Window::Width * Prgm::ElementsInterface::AvancementChargement, Prgm::Window::Height);
					App->Draw(*SpriteFondChargement);
				}
				App->Draw(*SpriteChargement);
			}
			
			App->Display();
			
			while (App->GetEvent(Event))
			{
				if (interfaceChargee)
					Prgm::ElementsInterface::interface->CatchEvent(Event);
				
				if ( (Event.Type == sf::Event::KeyPressed && Event.Key.Code == sf::Key::Escape) || Event.Type == sf::Event::Closed)
				{
					if (interfaceChargee)
					{
						TiXmlDocument* sauvegarde = new TiXmlDocument();
						
						TiXmlDeclaration* declaration = new TiXmlDeclaration("1.0", "iso-8859-1", "");
						
						TiXmlElement* root = Prgm::ElementsInterface::interface->Sauvegarde();
						
						sauvegarde->LinkEndChild(declaration);
						sauvegarde->LinkEndChild(root);
						sauvegarde->SaveFile("./Donnees/Sauvegardes/test.xml");
					}
					boucler = false;
				}
			}
			
			if (interfaceChargee)
				Prgm::ElementsInterface::interface->DerouleRound(chronoFrame->GetElapsedTime());
		}
	
	// fin des threads et deletion des objets
		threadChargeInterface.Terminate();
		
		delete chronoFrame;
		
		delete ImageFondChargement;
		delete SpriteFondChargement;
		delete SpriteChargement;
		delete ImageChargement;
		
		delete Prgm::ElementsInterface::interface;
		delete GestionnaireDonnees;
		delete App;
	
	return EXIT_SUCCESS;
}

#endif
