#ifndef SFBATIMENT_CPP
#define SFBATIMENT_CPP

#include "sfbatiment.hpp"

#include <iostream>
#include <sstream>
#include <cmath>

#include "gestionnaire.hpp"
#include "data.hpp"
#include "typeobjet.hpp"
#include "typebatiment.hpp"
#include "fonctions.hpp"

namespace Interface
{
	SFBatiment::SFBatiment(TiXmlElement* element) : SFenetre(element)
	{
		mBatimentCharge = false;
		mBatiment = NULL;
		mTypeBatiment = NULL;
		
		mFabriqueObjets = false;
		
		if (Prgm::SFenetres::Batiment::TransformationRessources)
		{
			mTransformationRessources = new sf::Sprite(*Prgm::SFenetres::Batiment::TransformationRessources);
			mTransformationRessources->Resize(Prgm::SFenetres::Batiment::CoteBoutonTransformationFabrication, Prgm::SFenetres::Batiment::CoteBoutonTransformationFabrication);
		}
		else
			mTransformationRessources = NULL;
		
		if (Prgm::SFenetres::Batiment::FabricationObjets)
		{
			mFabricationObjets = new sf::Sprite(*Prgm::SFenetres::Batiment::FabricationObjets);
			mFabricationObjets->Resize(Prgm::SFenetres::Batiment::CoteBoutonTransformationFabrication, Prgm::SFenetres::Batiment::CoteBoutonTransformationFabrication);
		}
		else
			mTransformationRessources = NULL;
		
		mNumPage = 0;
		mNombreRessourcesParPage = static_cast<unsigned int>( (this->GetHauteur() - Prgm::SFenetres::Batiment::AbscsisseListes - Prgm::Fleches::Cote) / (Prgm::Ressources::CoteIcone + Prgm::SFenetres::Batiment::EspaceInterligne) );
		mNombreObjetsParPage = static_cast<unsigned int>( (this->GetHauteur() - Prgm::SFenetres::Batiment::AbscsisseListes - Prgm::Fleches::Cote) / (Prgm::Objets::CoteIcone + Prgm::SFenetres::Batiment::EspaceInterligne) );
		mNombrePagesRessources = 0;
		mNombrePagesObjets = 0;
		
		mQuantiteATransformer = 0;
		mChaineQuantiteATransformer = new Texte(false);
		mChaineQuantiteATransformer->SetText("Quantit: 0");
		
		if (Prgm::Fleches::FlecheGauche)
		{
			mFlecheGauche = new sf::Sprite(*Prgm::Fleches::FlecheGauche);
			mFlecheGauche->Resize(Prgm::Fleches::Cote, Prgm::Fleches::Cote);
		}
		else
			mFlecheGauche = NULL;
		
		if (Prgm::Fleches::FlecheDroite)
		{
			mFlecheDroite = new sf::Sprite(*Prgm::Fleches::FlecheDroite);
			mFlecheDroite->Resize(Prgm::Fleches::Cote, Prgm::Fleches::Cote);
		}
		else
			mFlecheDroite = NULL;
		
		mPersonnage = NULL;
		
		this->ChangeNumPage();
	}
	
	SFBatiment::~SFBatiment()
	{
		delete mTransformationRessources;
		delete mFabricationObjets;
		
		delete mChaineQuantiteATransformer;
		
		delete mFlecheGauche;
		delete mFlecheDroite;
	}
	
	void
	SFBatiment::Charge()
	{
		ChargeXmlValeur(mXml, "Visible", mVisible);
		ChargeXmlValeur(mXml, "Abscisse", mX);
		ChargeXmlValeur(mXml, "Ordonnee", mY);
		ChargeXmlValeur(mXml, "Largeur", mLargeur);
		ChargeXmlValeur(mXml, "Hauteur", mHauteur);
		ChargeXmlValeur(mXml, "FabriqueObjets", mFabriqueObjets);
		ChargeXmlValeur(mXml, "QuantiteATransformer", mQuantiteATransformer);
		ChargeXmlValeur(mXml, "NumPage", mNumPage);
		
		this->SetPosition(mX, mY);
	}
	
	TiXmlElement*
	SFBatiment::Sauvegarde()
	{
		TiXmlElement* retour = new TiXmlElement("SFBatiment");
		TiXmlElement* selement;
		
		selement = new TiXmlElement("Visible");
		retour->LinkEndChild(selement);
		selement->SetAttribute("valeur", mVisible);
		
		selement = new TiXmlElement("Abscisse");
		retour->LinkEndChild(selement);
		selement->SetAttribute("valeur", mX);
		
		selement = new TiXmlElement("Ordonnee");
		retour->LinkEndChild(selement);
		selement->SetAttribute("valeur", mY);
		
		selement = new TiXmlElement("Largeur");
		retour->LinkEndChild(selement);
		selement->SetAttribute("valeur", mLargeur);
		
		selement = new TiXmlElement("Hauteur");
		retour->LinkEndChild(selement);
		selement->SetAttribute("valeur", mHauteur);
		
		selement = new TiXmlElement("FabriqueObjets");
		retour->LinkEndChild(selement);
		selement->SetAttribute("valeur", mFabriqueObjets);
		
		selement = new TiXmlElement("QuantiteATransformer");
		retour->LinkEndChild(selement);
		selement->SetAttribute("valeur", mQuantiteATransformer);
		
		selement = new TiXmlElement("NumPage");
		retour->LinkEndChild(selement);
		selement->SetAttribute("valeur", mNumPage);
		
		return retour;
	}
	
	void
	SFBatiment::Dessine(sf::RenderTarget* const App)
	{
		SFenetre::Dessine(App);
		
		if (this->GetVisible() && mBatimentCharge)
		{
			sf::String* chaine = new sf::String();
			chaine->SetText(mTypeBatiment->GetNom());
			chaine->SetSize(Prgm::SFenetres::Batiment::TailleTexteNom);
			chaine->SetPosition(this->GetX() + (this->GetLargeur() / 2) - chaine->GetRect().GetWidth() / 2, this->GetY() + Prgm::SFenetres::Batiment::OrdonneeNom); // centre le texte
			App->Draw(*chaine);
			
			unsigned int i = 0;
			std::ostringstream chaineRetour;
			double avancementFabrication;
			if (mFabriqueObjets)
			{
				GS::TypeObjet* typeObjet;
				unsigned int nombreObjetsFabricables = mTypeBatiment->GetNombreObjetsFabricables();
				while (i < mNombreObjetsParPage && i + mNumPage * mNombreObjetsParPage < nombreObjetsFabricables)
				{
					typeObjet = GestionnaireDonnees->GetTypeObjet(mTypeBatiment->GetNumObjetFabricable(i + mNumPage * mNombreObjetsParPage));
					
					chaine->SetText(typeObjet->GetNom());
					chaine->SetSize(Prgm::SFenetres::TailleTexte);
					if (chaine->GetRect().GetWidth() > Prgm::SFenetres::Batiment::LargeurColoneNoms)
						chaine->SetScaleX(Prgm::SFenetres::Batiment::LargeurColoneNoms / chaine->GetRect().GetWidth());
					chaine->SetPosition(this->GetX() + Prgm::SFenetres::Batiment::AbscsisseListes, this->GetY() + (Prgm::Objets::CoteIcone + Prgm::SFenetres::Batiment::EspaceInterligne) * i + Prgm::SFenetres::Batiment::OrdonneeListes);
					App->Draw(*chaine);
					
					typeObjet->Dessine(App, this->GetX() + Prgm::SFenetres::Batiment::AbscsisseListes + Prgm::SFenetres::Batiment::LargeurColoneNoms, this->GetY() + (Prgm::Objets::CoteIcone + Prgm::SFenetres::Batiment::EspaceInterligne) * i + Prgm::SFenetres::Batiment::OrdonneeListes); // dessine l'icne de l'objet
					
					chaineRetour.str("");
					chaineRetour << "x" << mBatiment->GetNombreObjetsConstruits(i + mNumPage * mNombreObjetsParPage);
					
					if ( (avancementFabrication = mBatiment->GetAvancementFabricationObjet(i + mNumPage * mNombreObjetsParPage)) )
						chaineRetour << " (" << floor(avancementFabrication * 100) << "%)";
					
					chaine->SetText(chaineRetour.str());
					chaine->SetSize(Prgm::SFenetres::TailleTexte);
					chaine->SetPosition(this->GetX() + Prgm::SFenetres::Batiment::AbscsisseListes + Prgm::SFenetres::Batiment::LargeurColoneNoms + Prgm::Objets::CoteIcone, this->GetY() + (Prgm::Objets::CoteIcone + Prgm::SFenetres::Batiment::EspaceInterligne) * i + Prgm::SFenetres::Batiment::OrdonneeListes);
					App->Draw(*chaine);
					
					i++;
				}
			}
			else
			{
				mChaineQuantiteATransformer->Dessine(App);
				
				GS::TypeRessource* typeRessource;
				unsigned int nombreRessourcesTransformables = mTypeBatiment->GetNombreRessourcesTransformables();
				
				while (i < mNombreRessourcesParPage && i + mNumPage * mNombreRessourcesParPage < nombreRessourcesTransformables)
				{
					typeRessource = GestionnaireDonnees->GetTypeRessource(mTypeBatiment->GetNumRessourceTransformable(i + mNumPage * mNombreRessourcesParPage));
					
					chaine->SetText(typeRessource->GetNom());
					chaine->SetSize(Prgm::SFenetres::TailleTexte);
					
					if (chaine->GetRect().GetWidth() > Prgm::SFenetres::Batiment::LargeurColoneNoms)
						chaine->SetScaleX(Prgm::SFenetres::Batiment::LargeurColoneNoms / chaine->GetRect().GetWidth());
					chaine->SetPosition(this->GetX() + Prgm::SFenetres::Batiment::AbscsisseListes, this->GetY() + (Prgm::Ressources::CoteIcone + Prgm::SFenetres::Batiment::EspaceInterligne) * i + Prgm::SFenetres::Batiment::OrdonneeListes);
					App->Draw(*chaine);
					
					typeRessource->Dessine(App, this->GetX() + Prgm::SFenetres::Batiment::AbscsisseListes + Prgm::SFenetres::Batiment::LargeurColoneNoms, this->GetY() + (Prgm::Ressources::CoteIcone + Prgm::SFenetres::Batiment::EspaceInterligne) * i + Prgm::SFenetres::Batiment::OrdonneeListes); // dessine l'icne de la ressource
					
					i++;
				}
			}
			
			GS::ListeElements* elementACreer = mBatiment->GetListeElements();
			double maxObjets = (this->GetLargeur() - 2 * (Prgm::SFenetres::Marges + Prgm::Fleches::Cote)) / Prgm::Objets::CoteIcone - 1;
			
			i = 0;
			while (elementACreer && i < maxObjets)
			{
				if (elementACreer->numRessource >= 0)
					GestionnaireDonnees->GetTypeRessource(mTypeBatiment->GetNumRessourceTransformable(elementACreer->numRessource))->Dessine(App, this->GetX() + Prgm::SFenetres::Marges + Prgm::Fleches::Cote + i * Prgm::Ressources::CoteIcone, this->GetY() + this->GetHauteur() - Prgm::Ressources::CoteIcone - Prgm::SFenetres::Marges);
				
				if (elementACreer->numObjet >= 0)
					GestionnaireDonnees->GetTypeObjet(mTypeBatiment->GetNumObjetFabricable(elementACreer->numObjet))->Dessine(App, this->GetX() + Prgm::SFenetres::Marges + Prgm::Fleches::Cote + i * Prgm::Objets::CoteIcone, this->GetY() + this->GetHauteur() - Prgm::Objets::CoteIcone - Prgm::SFenetres::Marges);
				
				elementACreer = elementACreer->elementSuivant;
				i++;
			}
			
			if (mFabriqueObjets)
			{
				if (mTransformationRessources)
					App->Draw(*mTransformationRessources);
			}
			else
			{
				if (mFabricationObjets)
					App->Draw(*mFabricationObjets);
			}
			
			if (mFlecheGauche && mNumPage > 0)
				App->Draw(*mFlecheGauche);
			
			if (mFlecheDroite && mNumPage < (mFabriqueObjets ? mNombrePagesObjets : mNombrePagesRessources) - 1)
				App->Draw(*mFlecheDroite);
			
			delete chaine;
		}
	}
	
	void
	SFBatiment::SetPosition(const int x, const int y)
	{
		SFenetre::SetPosition(x, y);
		
		if (mTransformationRessources)
			mTransformationRessources->SetPosition(this->GetX() + Prgm::SFenetres::Batiment::AbscisseBoutonTransformationFabrication, this->GetY() + Prgm::SFenetres::Batiment::OrdonneeBoutonTransformationFabrication);
		
		if (mFabricationObjets)
			mFabricationObjets->SetPosition(this->GetX() + Prgm::SFenetres::Batiment::AbscisseBoutonTransformationFabrication, this->GetY() + Prgm::SFenetres::Batiment::OrdonneeBoutonTransformationFabrication);
		
		
		if (mChaineQuantiteATransformer)
			mChaineQuantiteATransformer->SetPosition(this->GetX() + Prgm::SFenetres::Batiment::AbscisseQuantiteRessourceATransformer, this->GetY() + Prgm::SFenetres::Batiment::OrdonneeQuantiteRessourceATransformer);
		
		
		if (mFlecheGauche)
			mFlecheGauche->SetPosition(this->GetX() + Prgm::SFenetres::Marges, this->GetY() + this->GetHauteur() - Prgm::Fleches::Cote - Prgm::SFenetres::Marges);
		
		if (mFlecheDroite)
			mFlecheDroite->SetPosition(this->GetX() + this->GetLargeur() - Prgm::Fleches::Cote - Prgm::SFenetres::Marges, this->GetY() + this->GetHauteur() - Prgm::Fleches::Cote - Prgm::SFenetres::Marges);
	}
	
	bool
	SFBatiment::Catch(const int x, const int y, const sf::Event Event)
	{
		if (this->GetVisible())
		{
			if (this->GetX() <= x && x <= this->GetX() + this->GetLargeur() &&
				this->GetY() <= y && y <= this->GetY() + this->GetHauteur() &&
				Event.Type == sf::Event::MouseWheelMoved)
			{
				mQuantiteATransformer += Event.MouseWheel.Delta;
				
				if (mQuantiteATransformer < 0)
					mQuantiteATransformer = 0;
				
				std::ostringstream chaineRetour;
				chaineRetour.str("");
				chaineRetour << "Quantit: " << mQuantiteATransformer;
				
				mChaineQuantiteATransformer->SetText(chaineRetour.str());
			}
			else if (this->GetX() + Prgm::SFenetres::Batiment::AbscisseBoutonTransformationFabrication <= x && x <= this->GetX() + Prgm::SFenetres::Batiment::AbscisseBoutonTransformationFabrication + Prgm::SFenetres::Batiment::CoteBoutonTransformationFabrication &&
				this->GetY() + Prgm::SFenetres::Batiment::OrdonneeBoutonTransformationFabrication <= y && y <= this->GetY() + Prgm::SFenetres::Batiment::OrdonneeBoutonTransformationFabrication + Prgm::SFenetres::Batiment::CoteBoutonTransformationFabrication)
			{
				if (Event.Type == sf::Event::MouseButtonReleased)
					mFabriqueObjets = !mFabriqueObjets;
				
				return true;
			}
			else if (this->GetX() + Prgm::SFenetres::Marges <= x && x <= this->GetX() + Prgm::SFenetres::Marges + Prgm::Fleches::Cote &&
				this->GetY() + this->GetHauteur() - Prgm::Fleches::Cote - Prgm::SFenetres::Marges <= y && y <= this->GetY() + this->GetHauteur() - Prgm::SFenetres::Marges
				&& mNumPage > 0)
			{
				if (Event.Type == sf::Event::MouseButtonReleased && Event.MouseButton.Button == sf::Mouse::Left)
				{
					mNumPage--;
					this->ChangeNumPage();
				}
				
				return true;
			}
			else if (this->GetX() + this->GetLargeur() - Prgm::Fleches::Cote - Prgm::SFenetres::Marges <= x && x <= this->GetX() + this->GetLargeur() - Prgm::SFenetres::Marges &&
					this->GetY() + this->GetHauteur() - Prgm::Fleches::Cote - Prgm::SFenetres::Marges <= y && y <= this->GetY() + this->GetHauteur() - Prgm::SFenetres::Marges
					&& mNumPage < (mFabriqueObjets ? mNombrePagesObjets : mNombrePagesRessources) - 1 )
			{
				if (Event.Type == sf::Event::MouseButtonReleased && Event.MouseButton.Button == sf::Mouse::Left)
				{
					mNumPage++;
					this->ChangeNumPage();
				}
				
				return true;
			}
			else if (mBatimentCharge)
			{
				bool capture = false;
				
				unsigned int i = 0;
				
				if (mFabriqueObjets)
				{
					unsigned int nombreObjetsFabricables = mTypeBatiment->GetNombreObjetsFabricables();
					while (!capture && i < mNombreObjetsParPage && i + mNumPage * mNombreObjetsParPage < nombreObjetsFabricables)
					{
						capture = GestionnaireDonnees->GetTypeObjet(mTypeBatiment->GetNumObjetFabricable(i + mNumPage * mNombreObjetsParPage))->Catch(x, y, this->GetX() + Prgm::SFenetres::Batiment::AbscsisseListes + Prgm::SFenetres::Batiment::LargeurColoneNoms, this->GetY() + (Prgm::Objets::CoteIcone + Prgm::SFenetres::Batiment::EspaceInterligne) * i + Prgm::SFenetres::Batiment::OrdonneeListes);
						
						if (!capture)
							i++;
					}
				}
				else
				{
					unsigned int nombreRessourcesTransformables = mTypeBatiment->GetNombreRessourcesTransformables();
					while (!capture && i < mNombreRessourcesParPage && i + mNumPage * mNombreRessourcesParPage < nombreRessourcesTransformables)
					{
						capture = GestionnaireDonnees->GetTypeRessource(mTypeBatiment->GetNumRessourceTransformable(i + mNumPage * mNombreObjetsParPage))->Catch(x, y, this->GetX() + Prgm::SFenetres::Batiment::AbscsisseListes + Prgm::SFenetres::Batiment::LargeurColoneNoms, this->GetY() + (Prgm::Objets::CoteIcone + Prgm::SFenetres::Batiment::EspaceInterligne) * i + Prgm::SFenetres::Batiment::OrdonneeListes);
						
						if (!capture)
							i++;
					}
				}
				
				if (capture)
				{
					if (mFabriqueObjets)
					{
						if (Event.Type == sf::Event::MouseButtonReleased && Event.MouseButton.Button == sf::Mouse::Left)
						{
							mBatiment->AjouteObjetAFabriquer(i + mNumPage * mNombreObjetsParPage);
						}
						else if (Event.Type == sf::Event::MouseButtonReleased && Event.MouseButton.Button == sf::Mouse::Right)
						{
							GS::Objet* objetAPrendre = mBatiment->PrendObjet(i + mNumPage * mNombreObjetsParPage);
							
							if (objetAPrendre)
								mPersonnage->AjouteObjet(objetAPrendre);
						}
					}
					else
					{
						if (Event.Type == sf::Event::MouseButtonReleased && Event.MouseButton.Button == sf::Mouse::Left && mQuantiteATransformer)
						{
							mBatiment->AjouteRessourceAFabriquer(i + mNumPage * mNombreRessourcesParPage, mQuantiteATransformer);
							mQuantiteATransformer = 0;
							mChaineQuantiteATransformer->SetText("Quantit: 0");
						}
					}
					
					return true;
				}
				else
				{
					{
						GS::ListeElements* elementACreer = mBatiment->GetListeElements();
						double maxObjets = (this->GetLargeur() - 2 * (Prgm::SFenetres::Marges + Prgm::Fleches::Cote)) / Prgm::Objets::CoteIcone - 1;
						unsigned int i = 0;
						while (!capture && elementACreer && i < maxObjets)
						{
							if (this->GetX() + Prgm::SFenetres::Marges + Prgm::Fleches::Cote + i * Prgm::Objets::CoteIcone <= x && x <= this->GetX() + Prgm::SFenetres::Marges + Prgm::Fleches::Cote + i * Prgm::Objets::CoteIcone + Prgm::Objets::CoteIcone
								&& this->GetY() + this->GetHauteur() - Prgm::Objets::CoteIcone - Prgm::SFenetres::Marges <= y && y <= this->GetY() + this->GetHauteur() - Prgm::Objets::CoteIcone - Prgm::SFenetres::Marges + Prgm::Objets::CoteIcone)
							{
								capture = true;
							}
							
							elementACreer = elementACreer->elementSuivant;
							if (!capture)
								i++;
						}
						
						if (capture)
						{
							if (Event.Type == sf::Event::MouseButtonReleased)
								mBatiment->EnleveElementListeTravail(i);
							
							return true;
						}
					}
				}
			}
		}
		
		return SFenetre::Catch(x, y, Event);
	}
	
	std::string
	SFBatiment::GetText(const int x, const int y, const GS::Personnage* personnage) const
	{
		if (this->GetVisible())
		{
			if (this->GetX() + Prgm::SFenetres::Marges <= x && x <= this->GetX() + Prgm::SFenetres::Marges + Prgm::Fleches::Cote &&
				this->GetY() + this->GetHauteur() - Prgm::Fleches::Cote - Prgm::SFenetres::Marges <= y && y <= this->GetY() + this->GetHauteur() - Prgm::SFenetres::Marges
				&& mNumPage > 0)
			{
				return "Aller  la page prcdente";
			}
			else if (this->GetX() + this->GetLargeur() - Prgm::Fleches::Cote - Prgm::SFenetres::Marges <= x && x <= this->GetX() + this->GetLargeur() - Prgm::SFenetres::Marges &&
					this->GetY() + this->GetHauteur() - Prgm::Fleches::Cote - Prgm::SFenetres::Marges <= y && y <= this->GetY() + this->GetHauteur() - Prgm::SFenetres::Marges
					&& mNumPage < (mFabriqueObjets ? mNombrePagesObjets : mNombrePagesRessources) - 1 )
			{
				return "Aller  la page suivante";
			}
			else if (mBatimentCharge)
			{
				bool capture = false;
				unsigned int i = 0;
				unsigned int nombreObjetsFabricables = mTypeBatiment->GetNombreObjetsFabricables();
				while (!capture && i < mNombreObjetsParPage && i + mNumPage * mNombreObjetsParPage < nombreObjetsFabricables)
				{
					capture = GestionnaireDonnees->GetTypeObjet(mTypeBatiment->GetNumObjetFabricable(i + mNumPage * mNombreObjetsParPage))->Catch(x, y, this->GetX() + Prgm::SFenetres::Batiment::AbscsisseListes + Prgm::SFenetres::Batiment::LargeurColoneNoms, this->GetY() + (Prgm::Objets::CoteIcone + Prgm::SFenetres::Batiment::EspaceInterligne) * i + Prgm::SFenetres::Batiment::OrdonneeListes); // dessine l'icne de l'objet
					
					if (!capture)
						i++;
				}
				
				if (capture)
				{
					std::ostringstream chaineRetour;
					GS::TypeObjet* typeObjet = mBatiment->GetTypeObjetFabricable(i + mNumPage * mNombreObjetsParPage);
					
					chaineRetour << typeObjet->GetNom();
					chaineRetour << "\nRessources necessaires:";
					
					unsigned int i = 0;
					unsigned int nombre = typeObjet->GetNombreTypesRessourcesNecessaires();
					while (i < nombre)
					{
						chaineRetour << "\n" << NormaliseRessource( personnage->GetQuantiteRessource(typeObjet->GetTypeRessourceNecessaire(i)) ) << "/" << typeObjet->GetQuantiteRessourceNecessaire(i);
						i++;
					}
					
					return chaineRetour.str();
				}
				else
				{
					GS::ListeElements* elementACreer = mBatiment->GetListeElements();
					double maxObjets = (this->GetLargeur() - 2 * (Prgm::SFenetres::Marges + Prgm::Fleches::Cote)) / Prgm::Objets::CoteIcone - 1;
					unsigned int i = 0;
					while (!capture && elementACreer && i < maxObjets)
					{
						if (this->GetX() + Prgm::SFenetres::Marges + Prgm::Fleches::Cote + i * Prgm::Objets::CoteIcone <= x && x <= this->GetX() + Prgm::SFenetres::Marges + Prgm::Fleches::Cote + i * Prgm::Objets::CoteIcone + Prgm::Objets::CoteIcone
							&& this->GetY() + this->GetHauteur() - Prgm::Objets::CoteIcone - Prgm::SFenetres::Marges <= y && y <= this->GetY() + this->GetHauteur() - Prgm::Objets::CoteIcone - Prgm::SFenetres::Marges + Prgm::Objets::CoteIcone)
						{
							capture = true;
						}
						
						if (!capture)
							elementACreer = elementACreer->elementSuivant;
						
						i++;
					}
					
					if (capture)	
					{
						std::ostringstream chaineRetour;
						
						if (elementACreer->numRessource >= 0)
						{
							GS::TypeRessource* typeRessource = GestionnaireDonnees->GetTypeRessource(mTypeBatiment->GetNumRessourceTransformable(elementACreer->numRessource));
							
							chaineRetour << typeRessource->GetNom();
							chaineRetour << "\nRessources necessaires:";
							
							unsigned int i = 0;
							unsigned int nombre = typeRessource->GetNombreTypesRessourcesNecessaires();
							double quantiteRessource;
							while (i < nombre)
							{
								quantiteRessource = elementACreer->quantite * typeRessource->GetQuantiteRessourceNecessaire(i);
								
								if (quantiteRessource)
									chaineRetour << "\n" << NormaliseRessource(quantiteRessource) << " " << GestionnaireDonnees->GetTypeRessource(typeRessource->GetTypeRessourceNecessaire(i))->GetNom();
								
								i++;
							}
						}
						else
						{
							GS::TypeObjet* typeObjet = GestionnaireDonnees->GetTypeObjet(mTypeBatiment->GetNumObjetFabricable(elementACreer->numObjet));
							
							chaineRetour << typeObjet->GetNom();
							chaineRetour << "\nRessources necessaires:";
							
							unsigned int i = 0;
							unsigned int nombre = typeObjet->GetNombreTypesRessourcesNecessaires();
							double quantiteRessource;
							while (i < nombre)
							{
								quantiteRessource = (1 - mBatiment->GetAvancementFabricationObjet(elementACreer->numObjet)) * typeObjet->GetQuantiteRessourceNecessaire(i);
								
								if (quantiteRessource)
									chaineRetour << "\n" << NormaliseRessource(quantiteRessource) << "/" << typeObjet->GetQuantiteRessourceNecessaire(i) << " " << GestionnaireDonnees->GetTypeRessource(typeObjet->GetTypeRessourceNecessaire(i))->GetNom();
								
								i++;
							}
						}
						
						
						
						return chaineRetour.str();
					}
				}
			}
		}
		else
			return SFenetre::GetText(x, y);
		
		return "";
	}
	
	void
	SFBatiment::ChangeNumPage()
	{
		std::ostringstream chaine;
		chaine << "Fentre de batiment";
		
		if ((mFabriqueObjets ? mNombrePagesObjets : mNombrePagesRessources) > 1)
		{
			chaine << "(";
			chaine << mNumPage + 1;
			chaine << "/";
			chaine << (mFabriqueObjets ? mNombrePagesObjets : mNombrePagesRessources);
			chaine << ")";
		}
		
		this->SetTitre(chaine.str());
	}
	
	void
	SFBatiment::ChargeBatiment(GS::Batiment* batiment)
	{
		if (batiment)
		{
			mBatimentCharge = true;
			mBatiment = batiment;
			mTypeBatiment = mBatiment->GetTypeBatiment();
			mNumPage = 0;
			
			mNombrePagesRessources = static_cast<unsigned int>( ceil( static_cast<double>(mTypeBatiment->GetNombreRessourcesTransformables() / mNombreRessourcesParPage) ) );
			if (mNombrePagesRessources == 0)
				mNombrePagesRessources = 1;
			
			mNombrePagesObjets = static_cast<unsigned int>( ceil( static_cast<double>(mTypeBatiment->GetNombreObjetsFabricables() / mNombreObjetsParPage) ) );
			if (mNombrePagesObjets == 0)
				mNombrePagesObjets = 1;
			
			this->ChangeNumPage();
		}
		else
		{
			mBatimentCharge = false;
			mNumPage = 0;
			mNombrePagesObjets = 0;
			mNombrePagesRessources = 0;
			
			this->ChangeNumPage();
		}
	}
	
	void
	SFBatiment::SetPersonnage(GS::Personnage* personnage)
	{
		mPersonnage = personnage;
	}
}

#endif
