/**
* \file sfbatiment.hpp
* \brief Interface::SFBatiment
* \author Yoha
* \version alpha-0.0.07
* \date Aot 2009
*
* Definition de la sous-fenetre de batiment
*
*/

#ifndef SFBATIMENT_HPP
#define SFBATIMENT_HPP

/// Mode dynamique
#define SFML_DYNAMIC

#include <SFML/Graphics.hpp>

#include "sfenetre.hpp"
#include "batiment.hpp"
#include "personnage.hpp"
#include "texte.hpp"

/// Groupement de tous les elements concernant l'interface
namespace Interface
{
	/// Definit la sous-fenetre de batiment
	class SFBatiment : public SFenetre
	{
		public:
			/// \brief Constructeur
			SFBatiment(TiXmlElement* element = NULL);
			/// \brief Constructeur
			virtual ~SFBatiment();
			
			/// \brief Charge une sauvegarde
			/// \param element Element contenant la sauvegarde
			void Charge();
			/// \brief Sauvegarde
			TiXmlElement* Sauvegarde();
			
			/// \brief SFenetre: Methode de dessine de la sous-fenetre de batiment
			/// \param App Pointeur sur la fenetre de rendu
			void Dessine(sf::RenderTarget* const App);
			/// \brief Positionne la sous-fenetre de batiment
			/// \param x Nouvelle abscisse
			/// \param y Nouvelle ordonnee
			void SetPosition(const int x, const int y);
			/// \brief Capture des clics de la souris
			/// \param x Abscisse du clic
			/// \param y Ordonnee du clic
			/// \param Event Evenement SFML
			bool Catch(const int x, const int y, const sf::Event Event);
			/// \brief Affichage des infobulles au survol du curseur
			/// \param x Abscisse du curseur
			/// \param y Ordonnee du curseur
			/// \param personnage Personnage pour lequel afficher le texte
			std::string GetText(const int x, const int y, const GS::Personnage* personnage) const;
			
			/// \brief Change le titre de la sous-fenetre pour prendre en compte le numero et le nombre de pages
			void ChangeNumPage();
			/// \brief Charge la sous-fenetre de batiment avec les donnees d'un type de batiment
			/// \param batiment Id du type de batiment a charger
			void ChargeBatiment(GS::Batiment* batiment);
			
			/// \brief Definit le personnage qui interagit par cette sous-fenetre de batiment
			/// \param personnage Pointeur sur le personnage concerne
			void SetPersonnage(GS::Personnage* personnage);
		
		private:
			/// Un batiment est-il charge
			bool mBatimentCharge;
			/// Pointeur sur le type de batiment
			GS::Batiment* mBatiment;
			/// Type de ce batiment
			GS::TypeBatiment* mTypeBatiment;
			
			/// Mode fabrication (false) ou construction (true)
			bool mFabriqueObjets;
			/// Bouton transformation ressources
			sf::Sprite* mTransformationRessources;
			/// Bouton fabrication objets
			sf::Sprite* mFabricationObjets;
			
			/// Numero de la page
			unsigned int mNumPage;
			/// Nombre de ressources par page
			unsigned int mNombreRessourcesParPage;
			/// Nombre d'objets par page
			unsigned int mNombreObjetsParPage;
			/// Nombre de pages necessaires pour les ressources
			unsigned int mNombrePagesRessources;
			/// Nombre de pages necessaires pour les objets
			unsigned int mNombrePagesObjets;
			
			/// Quantite de ressource a transformer
			int mQuantiteATransformer;
			/// Texte d'affichage de la quantite de ressource a transformer
			Texte* mChaineQuantiteATransformer;
			
			/// Fleche gauche
			sf::Sprite* mFlecheGauche;
			/// Fleche droite
			sf::Sprite* mFlecheDroite;
			
			/// Pointeur sur le personnage
			GS::Personnage* mPersonnage;
	};
}

#endif
