#ifndef SFENETRE_CPP
#define SFENETRE_CPP

#include "sfenetre.hpp"

#include "fonctions.hpp"
#include "data.hpp"

namespace Interface
{
	SFenetre::SFenetre(TiXmlElement* element)
	{
		mXml = element;
		if (Prgm::SFenetres::Sprite)
		{
			mVisible = false;
			mDrag = false;
			mDragX = 0;
			mDragY = 0;
			mX = 0;
			mY = 0;
			
			mTitreSFenetre = new sf::String();
			mTitreSFenetre->SetSize(Prgm::SFenetres::TailleTexteTitre);
			mTitreSFenetre->SetColor(sf::Color(0, 0, 0, 255));
			this->SetTitre("SFenetre");
			
			mAbscisseTitre = 0;
			
			mSpriteSFenetre = new sf::Sprite(*Prgm::SFenetres::Sprite);
			
			mLargeur = static_cast<int>( mSpriteSFenetre->GetSize().x );
			mHauteur = static_cast<int>( mSpriteSFenetre->GetSize().y );
		}
		else
		{
			mSpriteSFenetre = NULL;
			CastErreur("Le sprite de la sous-fenetre n'a pas put etre charge.");
		}
	}
	
	SFenetre::~SFenetre()
	{
		delete mSpriteSFenetre;
		delete mTitreSFenetre;
	}
	
	void
	SFenetre::Charge()
	{
	}
	
	TiXmlElement*
	SFenetre::Sauvegarde()
	{
		TiXmlElement* retour = new TiXmlElement("SFenetre");
		TiXmlElement* selement;
		
		selement = new TiXmlElement("Visible");
		retour->LinkEndChild(selement);
		selement->SetAttribute("valeur", mVisible);
		
		selement = new TiXmlElement("Abscisse");
		retour->LinkEndChild(selement);
		selement->SetAttribute("valeur", mX);
		
		selement = new TiXmlElement("Ordonnee");
		retour->LinkEndChild(selement);
		selement->SetAttribute("valeur", mY);
		
		selement = new TiXmlElement("Largeur");
		retour->LinkEndChild(selement);
		selement->SetAttribute("valeur", mLargeur);
		
		selement = new TiXmlElement("Hauteur");
		retour->LinkEndChild(selement);
		selement->SetAttribute("valeur", mHauteur);
		
		return retour;
	}
	
	void
	SFenetre::Dessine(sf::RenderTarget* const App)
	{
		if (mVisible)
		{
			App->Draw(*mSpriteSFenetre);
			App->Draw(*mTitreSFenetre);
		}
	}
	
	void
	SFenetre::SetPosition(const int x, const int y)
	{
		mX = x;
		mY = y;
		
		mSpriteSFenetre->SetPosition(mX, mY);
		mTitreSFenetre->SetPosition(mX + mAbscisseTitre, mY + Prgm::SFenetres::OrdonneeTitre);
	}
	
	void
	SFenetre::SetMousePosition(const unsigned int x, const unsigned int y)
	{
		if (mDrag)
		{
			this->SetPosition(mX + x - mDragX, mY + y - mDragY);
			
			mDragX = x;
			mDragY = y;
		}
	}
	
	bool
	SFenetre::Catch(const int x, const int y, const sf::Event Event)
	{
		if (mVisible)
		{
			if (Event.Type == sf::Event::MouseButtonPressed)
			{
				if (mX <= x && x <= mX + mLargeur && mY <= y && y <= mY + mHauteur)
				{
					mDrag = true;
					mDragX = x;
					mDragY = y;
					return true;
				}
				
			}
			else if (Event.Type == sf::Event::MouseButtonReleased && mDrag)
			{
				mDrag = false;
				return true;
			}
			else
				return false;
		}
		return false;
	}
	
	std::string
	SFenetre::GetText(const int, const int) const
	{
		return "";
	}
	
	void
	SFenetre::Affiche()
	{
		mVisible = true;
	}
	
	void
	SFenetre::Cache()
	{
		mVisible = false;
	}
	
	void
	SFenetre::SetVisible(const bool visible)
	{
		mVisible = visible;
	}
	
	void
	SFenetre::ChangeVisibilite()
	{
		mVisible = !mVisible;
	}
	
	void
	SFenetre::SetTitre(std::string titre)
	{
		mTitreSFenetre->SetText(titre);
		mAbscisseTitre = (mLargeur / 2) - mTitreSFenetre->GetRect().GetWidth() / 2;
	}
	
	bool
	SFenetre::GetVisible() const
	{
		return mVisible;
	}
	
	int
	SFenetre::GetX() const
	{
		return mX;
	}
	
	int
	SFenetre::GetY() const
	{
		return mY;
	}
	
	int
	SFenetre::GetLargeur() const
	{
		return mLargeur;
	}
	
	int
	SFenetre::GetHauteur() const
	{
		return mHauteur;
	}
}

#endif
