/**
* \file sfenetre.hpp
* \brief Interface::SFenetre
* \author Yoha
* \version alpha-0.0.07
* \date Aot 2009
*
* Definition des sous-fenetres
*
*/

#ifndef SFENETRE_HPP
#define SFENETRE_HPP

/// Mode dynamique
#define SFML_DYNAMIC

#include <iostream>
#include <SFML/Graphics.hpp>
#include "tinyxml/tinyxml.h"

/// Groupement de tous les elements concernant l'interface
namespace Interface
{
	/// Definit les sous-fenetres
	class SFenetre
	{
		public:
			SFenetre(TiXmlElement* element = NULL);
			virtual ~SFenetre();
			
			/// \brief Charge une sauvegarde
			/// \param element Element contenant la sauvegarde
			virtual void Charge();
			/// \brief Sauvegarde
			virtual TiXmlElement* Sauvegarde();
			
			/// \brief Methode de dessine de la sous-fenetre
			/// \param App Pointeur sur la fenetre de rendu
			virtual void Dessine(sf::RenderTarget* const App);
			/// \brief Positionne la sous-fenetre
			/// \param x Nouvelle abscisse
			/// \param y Nouvelle ordonnee
			virtual void SetPosition(const int x, const int y);
			/// \brief Lorsque la souris se deplace (gestion des draGS)
			/// \param x Nouvelle abscisse de la souris
			/// \param y Nouvelle ordonnee de la souris
			virtual void SetMousePosition(const unsigned int x, const unsigned int y);
			/// \brief Capture des clics de la souris
			/// \param x Abscisse du clic
			/// \param y Ordonnee du clic
			/// \param Event Evenement SFML
			virtual bool Catch(const int x, const int y, const sf::Event Event);
			/// \brief Affichage des infobulles au survol du curseur
			/// \param x Abscisse du curseur
			/// \param y Ordonnee du curseur
			virtual std::string GetText(const int x, const int y) const;
			
			/// \brief Accesseur: affiche la sous-fenetre
			virtual void Affiche();
			/// \brief Accesseur: cache la sous-fenetre
			virtual void Cache();
			/// \brief Accesseur: definit la visibilite de la sous-fenetre
			virtual void SetVisible(const bool visible);
			/// \brief Accesseur: inverse la visibilite de la sous-fenetre
			virtual void ChangeVisibilite();
			/// \brief Accesseur: definit le titre de la sous-fenetre
			virtual void SetTitre(std::string titre);
			
			/// \brief Accesseur: renvoie la visibilite de la sous-fenetre
			virtual bool GetVisible() const;
			/// \brief Accesseur: renvoie l'abscisse de la sous-fenetre
			virtual int GetX() const;
			/// \brief Accesseur: renvoie l'ordonnee de la sous-fenetre
			virtual int GetY() const;
			/// \brief Accesseur: renvoie la largeur de la sous-fenetre
			virtual int GetLargeur() const;
			/// \brief Accesseur: renvoir la hauteur de la sous-fenetre
			virtual int GetHauteur() const;
		
		protected:
			/// Sauvegarde de la fenetre
			TiXmlElement* mXml;
			
			/// La sous-fenetre doit-elle etre affichee
			bool mVisible;
			/// La souris effectue-t-elle un drag
			bool mDrag;
			/// Derniere abscisse de la souris
			unsigned int mDragX;
			/// Derniere ordonnee de la souris
			unsigned int mDragY;
			/// Abscisse de la fenetre
			int mX;
			/// Ordonnee de la fenetre
			int mY;
			/// Largeur de la fenetre
			int mLargeur;
			/// Hauteur de la fenetre
			int mHauteur;
			
			/// Titre de la sous-fenetre
			sf::String* mTitreSFenetre;
			/// Abscisse relative du titre de la sous-fenetre
			float mAbscisseTitre;
			/// Sprite d'affichage de la sous-fenetre
			sf::Sprite* mSpriteSFenetre;
	};
}

#endif
