#ifndef SFINVENTAIRE_CPP
#define SFINVENTAIRE_CPP

#include "sfinventaire.hpp"

#include <iostream>
#include <sstream>
#include <cmath>

#include "fonctions.hpp"
#include "data.hpp"

namespace Interface
{
	SFInventaire::SFInventaire(TiXmlElement* element) : SFenetre(element)
	{
		if (Prgm::SFenetres::Inventaire::SpriteGrilleObjets)
		{
			mOnglet = false;
			mObjets = NULL;
			
			mDragObjet = false;
			mDragObjetX = 0;
			mDragObjetY = 0;
			mDragObjetCoordX = 0;
			mDragObjetCoordY = 0;
			mSourisX = 0;
			mSourisY = 0;
			
			mNumPage = 0;
			
			mSpriteGrilleObjets = new sf::Sprite(*Prgm::SFenetres::Inventaire::SpriteGrilleObjets);
			
			if (Prgm::Fleches::FlecheGauche)
			{
				mFlecheGauche = new sf::Sprite(*Prgm::Fleches::FlecheGauche);
				mFlecheGauche->Resize(Prgm::Fleches::Cote, Prgm::Fleches::Cote);
			}
			else
				mFlecheGauche = NULL;
			
			if (Prgm::Fleches::FlecheDroite)
			{
				mFlecheDroite = new sf::Sprite(*Prgm::Fleches::FlecheDroite);
				mFlecheDroite->Resize(Prgm::Fleches::Cote, Prgm::Fleches::Cote);
			}
			else
				mFlecheDroite = NULL;
			
			this->ChangeNumPage();
		}
		else
		{
			mSpriteGrilleObjets = NULL;
			CastErreur("Le sprite de la grille d'inventaire n'a pas put etre charge.");
		}
	}
	
	SFInventaire::~SFInventaire()
	{
		delete mSpriteGrilleObjets;
		
		delete mFlecheGauche;
		delete mFlecheDroite;
	}
	
	void
	SFInventaire::Charge()
	{
		ChargeXmlValeur(mXml, "Visible", mVisible);
		ChargeXmlValeur(mXml, "Abscisse", mX);
		ChargeXmlValeur(mXml, "Ordonnee", mY);
		ChargeXmlValeur(mXml, "Largeur", mLargeur);
		ChargeXmlValeur(mXml, "Hauteur", mHauteur);
		ChargeXmlValeur(mXml, "Onglet", mOnglet);
		ChargeXmlValeur(mXml, "NumPage", mNumPage);
		
		this->SetPosition(mX, mY);
	}
	
	TiXmlElement*
	SFInventaire::Sauvegarde()
	{
		TiXmlElement* retour = new TiXmlElement("SFInventaire");
		TiXmlElement* selement;
		
		selement = new TiXmlElement("Visible");
		retour->LinkEndChild(selement);
		selement->SetAttribute("valeur", mVisible);
		
		selement = new TiXmlElement("Abscisse");
		retour->LinkEndChild(selement);
		selement->SetAttribute("valeur", mX);
		
		selement = new TiXmlElement("Ordonnee");
		retour->LinkEndChild(selement);
		selement->SetAttribute("valeur", mY);
		
		selement = new TiXmlElement("Largeur");
		retour->LinkEndChild(selement);
		selement->SetAttribute("valeur", mLargeur);
		
		selement = new TiXmlElement("Hauteur");
		retour->LinkEndChild(selement);
		selement->SetAttribute("valeur", mHauteur);
		
		selement = new TiXmlElement("Onglet");
		retour->LinkEndChild(selement);
		selement->SetAttribute("valeur", mOnglet);
		
		selement = new TiXmlElement("NumPage");
		retour->LinkEndChild(selement);
		selement->SetAttribute("valeur", mNumPage);
		
		return retour;
	}
	
	void
	SFInventaire::Dessine(sf::RenderTarget* const App)
	{
		SFenetre::Dessine(App);
		
		if (this->GetVisible())
		{
			App->Draw(*mSpriteGrilleObjets);
			
			double centrage = Prgm::SFenetres::Inventaire::CoteCellule / 2 - Prgm::Objets::CoteIcone / 2;
			
			unsigned int y; // lignes
			unsigned int yl;
			
			unsigned int x; // colones
			unsigned int xl;
			
			y = 0;
			yl = (*mObjets)[mOnglet][mNumPage].size();
			while (y < yl)
			{
				x = 0;
				xl = (*mObjets)[mOnglet][mNumPage][y].size();
				while (x < xl)
				{
					if ((*mObjets)[mOnglet][mNumPage][y][x])
					{
						if (mDragObjet && x == mDragObjetX && y == mDragObjetY)
							(*mObjets)[mOnglet][mNumPage][y][x]->GetTypeObjet()->Dessine(App, mDragObjetCoordX, mDragObjetCoordY);
						else
							(*mObjets)[mOnglet][mNumPage][y][x]->GetTypeObjet()->Dessine(App,
								this->GetX() + Prgm::SFenetres::Inventaire::AbscisseGrilleObjets + centrage + Prgm::SFenetres::Inventaire::CoteCellule * x,
								this->GetY() + Prgm::SFenetres::Inventaire::OrdonneeGrilleObjets + Prgm::SFenetres::Inventaire::MargeSuperieureGrilleObjets + centrage + Prgm::SFenetres::Inventaire::CoteCellule * y);
					}
					x++;
				}
				y++;
			}
			
			if (mFlecheGauche && mNumPage > 0)
				App->Draw(*mFlecheGauche);
				
			if (mFlecheDroite && mNumPage < (*mObjets)[mOnglet].size() - 1)
				App->Draw(*mFlecheDroite);
		}
	}
	
	void
	SFInventaire::SetPosition(const int x, const int y)
	{
		SFenetre::SetPosition(x, y);
		
		mSpriteGrilleObjets->SetPosition(x + Prgm::SFenetres::Inventaire::AbscisseGrilleObjets, y + Prgm::SFenetres::Inventaire::OrdonneeGrilleObjets);
		
		if (mFlecheGauche)
			mFlecheGauche->SetPosition(this->GetX() + Prgm::SFenetres::Marges, this->GetY() + this->GetHauteur() - Prgm::Fleches::Cote - Prgm::SFenetres::Marges);
		
		if (mFlecheDroite)
			mFlecheDroite->SetPosition(this->GetX() + this->GetLargeur() - Prgm::Fleches::Cote - Prgm::SFenetres::Marges, this->GetY() + this->GetHauteur() - Prgm::Fleches::Cote - Prgm::SFenetres::Marges);
	}
	
	void
	SFInventaire::SetMousePosition(const unsigned int x, const unsigned int y)
	{
		SFenetre::SetMousePosition(x, y);
		
		if (mDragObjet)
		{
			mDragObjetCoordX = mDragObjetCoordX + x - mSourisX;
			mDragObjetCoordY = mDragObjetCoordY + y - mSourisY;
			
			mSourisX = x;
			mSourisY = y;
		}
	}
	
	bool
	SFInventaire::Catch(const int x, const int y, const sf::Event Event)
	{
		if (this->GetVisible())
		{
			if (this->GetX() + Prgm::SFenetres::Marges <= x && x <= this->GetX() + Prgm::SFenetres::Marges + Prgm::Fleches::Cote &&
				this->GetY() + this->GetHauteur() - Prgm::Fleches::Cote - Prgm::SFenetres::Marges <= y && y <= this->GetY() + this->GetHauteur() - Prgm::SFenetres::Marges)
			{
				if (Event.Type == sf::Event::MouseButtonReleased && Event.MouseButton.Button == sf::Mouse::Left && mNumPage > 0)
				{
					mNumPage--;
					this->ChangeNumPage();
				}
				
				return true;
			}
			else if (this->GetX() + this->GetLargeur() - Prgm::Fleches::Cote - Prgm::SFenetres::Marges <= x && x <= this->GetX() + this->GetLargeur() - Prgm::SFenetres::Marges &&
					this->GetY() + this->GetHauteur() - Prgm::Fleches::Cote - Prgm::SFenetres::Marges <= y && y <= this->GetY() + this->GetHauteur() - Prgm::SFenetres::Marges)
			{
				if (Event.Type == sf::Event::MouseButtonReleased && Event.MouseButton.Button == sf::Mouse::Left && mNumPage < (*mObjets)[mOnglet].size() - 1)
				{
					mNumPage++;
					this->ChangeNumPage();
				}
				
				return true;
			}
			else
			{
				if (Event.Type == sf::Event::MouseButtonPressed)
				{
					bool capture = false;
					double centrage = Prgm::SFenetres::Inventaire::CoteCellule / 2 - Prgm::Objets::CoteIcone / 2;
					
					unsigned int grilleY = 0; // lignes
					unsigned int yl = (*mObjets)[mOnglet][mNumPage].size();
					
					unsigned int grilleX = 0; // colones
					unsigned int xl = 0;
					
					while (!capture && grilleY < yl)
					{
						grilleX = 0;
						xl = (*mObjets)[mOnglet][mNumPage][grilleY].size();
						
						while (!capture && grilleX < xl)
						{
							if ((*mObjets)[mOnglet][mNumPage][grilleY][grilleX])
							{
								capture = (*mObjets)[mOnglet][mNumPage][grilleY][grilleX]->GetTypeObjet()->Catch(x, y, 
									this->GetX() + Prgm::SFenetres::Inventaire::AbscisseGrilleObjets + centrage + Prgm::SFenetres::Inventaire::CoteCellule * grilleX,
									this->GetY() + Prgm::SFenetres::Inventaire::OrdonneeGrilleObjets + Prgm::SFenetres::Inventaire::MargeSuperieureGrilleObjets + centrage + Prgm::SFenetres::Inventaire::CoteCellule * grilleY);
							}
							
							if (!capture)
								grilleX++;
						}
						
						if (!capture)
							grilleY++;
					}
					
					if (capture)
					{
						if (Event.MouseButton.Button == sf::Mouse::Left)
						{
							mDragObjet = true;
							mDragObjetX = grilleX;
							mDragObjetY = grilleY;
							mDragObjetY = grilleY;
							mDragObjetCoordX = static_cast<unsigned int>( this->GetX() + Prgm::SFenetres::Inventaire::AbscisseGrilleObjets + centrage + Prgm::SFenetres::Inventaire::CoteCellule * grilleX );
							mDragObjetCoordY = static_cast<unsigned int>( this->GetY() + Prgm::SFenetres::Inventaire::OrdonneeGrilleObjets + Prgm::SFenetres::Inventaire::MargeSuperieureGrilleObjets + centrage + Prgm::SFenetres::Inventaire::CoteCellule * grilleY );
							mSourisX = x;
							mSourisY = y;
						}
					}
					
					if (capture)
						return true;
				}
				else if (mDragObjet)
				{
					int caseX = static_cast<int>( floor( (x - this->GetX() - Prgm::SFenetres::Inventaire::AbscisseGrilleObjets) / Prgm::SFenetres::Inventaire::CoteCellule ) );
					int caseY = static_cast<int>( floor( (y - this->GetY() - Prgm::SFenetres::Inventaire::OrdonneeGrilleObjets - Prgm::SFenetres::Inventaire::MargeSuperieureGrilleObjets) / Prgm::SFenetres::Inventaire::CoteCellule ) );
					
					if (0 <= caseX && caseX < static_cast<int>(Prgm::SFenetres::Inventaire::NombreColones)
						&& 0 <= caseY && caseY < static_cast<int>(Prgm::SFenetres::Inventaire::NombreLignes)
						&& (caseY != static_cast<int>(Prgm::SFenetres::Inventaire::NombreLignes) - 1 || (caseX != 0 && caseX != static_cast<int>(Prgm::SFenetres::Inventaire::NombreColones) - 1)) )
					{
						mDragObjet = false;
						
						GS::Objet* temp = (*mObjets)[mOnglet][mNumPage][caseY][caseX];
						(*mObjets)[mOnglet][mNumPage][caseY][caseX] = (*mObjets)[mOnglet][mNumPage][mDragObjetY][mDragObjetX];
						(*mObjets)[mOnglet][mNumPage][mDragObjetY][mDragObjetX] = temp;
					}
					else
					{
						mDragObjet = false;
						mDragObjetX = 0;
						mDragObjetY = 0;
						mDragObjetCoordX = 0;
						mDragObjetCoordY = 0;
					}
					
					return true;
				}
			}
		}
		
		return SFenetre::Catch(x, y, Event);
	}
	
	void
	SFInventaire::ChangeNumPage()
	{
		std::ostringstream chaine;
		chaine << "Inventaire";
		
		if (mObjets && (*mObjets)[mOnglet].size() > 1)
		{
			chaine << "(";
			chaine << mNumPage + 1;
			chaine << "/";
			chaine << (*mObjets)[mOnglet].size();
			chaine << ")";
		}
		
		this->SetTitre(chaine.str());
	}
	
	void
	SFInventaire::SetObjets(std::vector < std::vector < std::vector < std::vector <GS::Objet*> > > >* vector)
	{
		mObjets = vector;
	}
	
	void
	SFInventaire::SetOnglet(const unsigned int num)
	{
		mOnglet = num;
		mNumPage = 0;
	}
}

#endif
