/**
* \file sfinventaire.hpp
* \brief Interface::SFInventaire
* \author Yoha
* \version alpha-0.0.07
* \date Aot 2009
*
* Definition de la sous-fenetre d'inventaire
*
*/

#ifndef SFINVENTAIRE_HPP
#define SFINVENTAIRE_HPP

/// Mode dynamique
#define SFML_DYNAMIC

#include <SFML/Graphics.hpp>

#include "sfenetre.hpp"
#include "personnage.hpp"

/// Groupement de tous les elements concernant l'interface
namespace Interface
{
	/// Definit la sous-fenetre d'inventaire
	class SFInventaire : public SFenetre
	{
		public:
			/// \brief Constructeur
			SFInventaire(TiXmlElement* element = NULL);
			/// \brief Destructeur
			~SFInventaire();
			
			/// \brief Charge une sauvegarde
			/// \param element Element contenant la sauvegarde
			void Charge();
			/// \brief Sauvegarde
			TiXmlElement* Sauvegarde();
			
			/// \brief SFenetre: Methode de dessine de la sous-fenetre d'inventaire
			/// \param App Pointeur sur la fenetre de rendu
			void Dessine(sf::RenderTarget* const App);
			/// \brief Positionne la sous-fenetre d'inventaire
			/// \param x Nouvelle abscisse
			/// \param y Nouvelle ordonnee
			void SetPosition(const int x, const int y);
			/// \brief Lorsque la souris se deplace (gestion des draGS)
			/// \param x Nouvelle abscisse de la souris
			/// \param y Nouvelle ordonnee de la souris
			void SetMousePosition(const unsigned int x, const unsigned int y);
			/// \brief Capture des clics de la souris
			/// \param x Abscisse du clic
			/// \param y Ordonnee du clic
			/// \param Event Evenement SFML
			bool Catch(const int x, const int y, const sf::Event Event);
			
			/// \brief Change le titre de la sous-fenetre pour prendre en compte le numero et le nombre de pages
			void ChangeNumPage();
			/// \brief Definit le vector de listes chainees d'objets a afficher
			/// \param vector Pointeur sur le vector de listes chainees
			void SetObjets(std::vector < std::vector < std::vector < std::vector <GS::Objet*> > > >* vector);
			/// \brief Definit l'onglet a afficher
			/// \param num Numero de l'onglet
			void SetOnglet(const unsigned int num);
		
		private:
			/// Numero de la categorie a afficher + 1 ou 0 pour tous
			bool mOnglet;
			/// Vector des lites chainees d'objets a afficher
			std::vector < std::vector < std::vector < std::vector <GS::Objet*> > > >* mObjets;
			
			/// La souris effectue-t-elle un drag d'objet
			bool mDragObjet;
			/// Abscisse de l'objet concerne dans la grille
			unsigned int mDragObjetX;
			/// Ordonnee de l'objet concerne dans la grille
			unsigned int mDragObjetY;
			/// Abscisse actuelle du sprite de l'objet
			unsigned int mDragObjetCoordX;
			/// Ordonnee actuelle du sprite de l'objet
			unsigned int mDragObjetCoordY;
			/// Absisse de la souris
			unsigned int mSourisX;
			/// Ordonee de la souris
			unsigned int mSourisY;
			
			/// Numero de la page
			unsigned int mNumPage;
			
			/// Sprite de la grille des objets a afficher
			sf::Sprite* mSpriteGrilleObjets;
			/// Fleche gauche
			sf::Sprite* mFlecheGauche;
			/// Fleche droite
			sf::Sprite* mFlecheDroite;
	};
}

#endif
