/**
* \file statut.hpp
* \brief GS::Statut
* \author Yoha
* \version alpha-0.0.07
* \date Aot 2009
*
* Appartient au systeme de jeu
*
*/

#ifndef STATUT_HPP
#define STATUT_HPP

/// Mode dynamique
#define SFML_DYNAMIC

#include <iostream>
#include <SFML/Graphics.hpp>

/// Game System
namespace GS
{
	/// Barres et statuts du personnage
	class Statut
	{
		public:
			/// \brief Constructeur
			Statut();
			/// \brief Destructeur
			~Statut();
			
			/// \brief Methode de dessine du bouton
			/// \param App Pointeur sur la fenetre de rendu
			void Dessine(sf::RenderTarget* const App) const;
			
			/// \brief Accesseur: definit le nom du statut
			/// \param nom Nom a charger
			void SetNom(const std::string nom);
			/// \brief Accesseur: definit la valeur par defaut
			/// \param valeur Valeur a charger
			void SetValeurDefaut(const double valeur);
			/// \brief Accesseur: definit la valeur actuelle (personnage actuel)
			/// \param valeur Valeur a charger
			void SetValeur(const double valeur);
			/// \brief Accesseur: definit la position
			/// \param x Abscisse
			/// \param y Ordonnee
			void SetPosition(const double x, const double y);
			
			/// \brief Accesseur: renvoie le nom du statut
			std::string GetNom() const;
			/// \brief Accesseur: renvoie la valeur par defaut
			double GetValeurDefaut() const;
			
		private:
			/// Nom du statut
			std::string mNom;
			/// Valeur par defaut du statut
			double mValeurDefaut;
			/// Valeur actuelle (personnage actuel)
			double mValeur;
			
			/// Image du contour de la valeur du statut
			sf::Image* mImageContourValeur;
			/// Afficheur du contour de la valeur du statut
			sf::Sprite* mSpriteContourValeur;
			
			/// Image du fond de la valeur du statut
			sf::Image* mImageFondValeur;
			/// Afficheur du fond de la valeur du statut
			sf::Sprite* mSpriteFondValeur;
			
			/// Image de la valeur du statut
			sf::Image* mImageValeur;
			/// Afficheur de la valeur du statut
			sf::Sprite* mSpriteValeur;
			
			/// Texte affichant le nom du statut
			sf::String* mTexte;
	};
}

#endif
