#ifndef TERRESSOURCE_CPP
#define TERRESSOURCE_CPP

#include "terressource.hpp"

#include "data.hpp"
#include "fonctions.hpp"
#include "gestionnaire.hpp"
#include "personnage.hpp"

namespace GS
{
	TerRessource::TerRessource(GS::Etre* parent, TiXmlElement* element) : Etre(parent, element), mTypeTerRessource(NULL)
	{
		mType = Types::Ressource;
		mLargeur = Prgm::TerRessources::Largeur;
		mHauteur = Prgm::TerRessources::Hauteur;
	}
	
	TerRessource::~TerRessource()
	{
	}
	
	void
	TerRessource::Charge()
	{
		if (mXml)
		{
			mXml->QueryDoubleAttribute("Abscisse", &mX);
			mXml->QueryDoubleAttribute("Ordonnee", &mY);
			
			unsigned int type;
			ChargeXmlParametre(mXml, "Type", type);
			mTypeTerRessource = GestionnaireDonnees->GetTypeTerRessource(type);
		}
	}
	
	TiXmlElement*
	TerRessource::Sauvegarde()
	{
		TiXmlElement* retour = new TiXmlElement("TerRessource");
		retour->SetAttribute("id", mIdentifiant);
		retour->SetDoubleAttribute("Abscisse", mX);
		retour->SetDoubleAttribute("Ordonnee", mY);
		retour->SetAttribute("Type", mTypeTerRessource->GetId());
		
		return retour;
	}
	
	void
	TerRessource::CollecteRessources(Etre* collecteur, double tempsFrame)
	{
		if (collecteur->GetType() == Types::Personnage)
		{
			Personnage* perso = static_cast<Personnage*>(collecteur);
			perso->AjouteRessource(mTypeTerRessource->GetResId(), mTypeTerRessource->GetRessourceParSeconde() * tempsFrame);
		}
	}
	
	std::string
	TerRessource::GetTexte() const
	{
		return mTypeTerRessource->GetAction();
	}
	
	Types::Names
	TerRessource::GetType() const
	{
		return Types::Ressource;
	}
	
	void
	TerRessource::Dessine(sf::RenderTarget* const App, const double x, const double y) const
	{
		mTypeTerRessource->Dessine(App, mX + x, mY + y);
	}
	
	void
	TerRessource::SetType(TypeTerRessource* type)
	{
		mTypeTerRessource = type;
	}
	
	TypeTerRessource*
	TerRessource::GetTypeTerRessource() const
	{
		return mTypeTerRessource;
	}
	
	bool
	TerRessource::GetConstructible(const double x, const double y, const unsigned int largeur, const unsigned int hauteur) const
	{
		return mX <= x + largeur && mY <= y + hauteur && x <= mX + mLargeur && y <= mY + mHauteur;
	}
}

#endif
