/**
* \file terressource.hpp
* \brief GS::TerRessource
* \author Yoha
* \version alpha-0.0.07
* \date Aot 2009
*
* Definition des ressources de terrain
*
*/

#ifndef TERRESSOURCE_HPP
#define TERRESSOURCE_HPP

/// Mode dynamique
#define SFML_DYNAMIC

#include <SFML/Graphics.hpp>

#include "etre.hpp"
#include "typeterressource.hpp"

/// Game System
namespace GS
{
	/// Represente une ressource de terrain
	class TerRessource : public Etre
	{
		public:
			/// \brief Constructeur
			TerRessource(GS::Etre* parent, TiXmlElement* element = NULL);
			/// \brief Destructeur
			virtual ~TerRessource();
			
			/// \brief Charge une sauvegarde
			/// \param element Element contenant la sauvegarde
			virtual void Charge();
			/// \brief Sauvegarde
			virtual TiXmlElement* Sauvegarde();
			
			void CollecteRessources(Etre* collecteur, double tempsFrame);
			
			/// \brief Etre: Retourne le texte qui s'affiche lors du pointage au curseur
			std::string GetTexte() const;
			/// \brief Etre: retourne le type d'etre
			Types::Names GetType() const;
			
			/// \brief Methode d'affichage de la ressource de terrain
			/// \param App Pointeur sur la fenetre de rendu
			/// \param x Abscisse de la carte
			/// \param y Ordonnee de la carte
			void Dessine(sf::RenderTarget* const App, const double x, const double y) const;
			
			/// \brief Accesseur: definit le type de la ressource de terrain
			/// \param type Pointeur sur un type de ressource de terrain
			void SetType(TypeTerRessource* type);
			
			/// \brief Accesseur: retourne le type de ressource de terrain dont il s'agit
			TypeTerRessource* GetTypeTerRessource() const;
			
			/// \brief Retourne la possibilite de construire un batiment terrestre dans ce rectangle
			/// \brief x Abcisse du batiment
			/// \brief y Ordonnee du batiment
			/// \brief largeur Largeur du batiment
			/// \brief hauteur Hauteur du batiment
			bool GetConstructible(const double x, const double y, const unsigned int largeur, const unsigned int hauteur) const;
			
		private:
			/// Type de la ressource de terrain
			TypeTerRessource* mTypeTerRessource;
	};
}

#endif
