#ifndef TYPEBATIMENT_CPP
#define TYPEBATIMENT_CPP

#include "typebatiment.hpp"

#include "projet.hpp"
#include "fonctions.hpp"
#include "data.hpp"

namespace GS
{
	TypeBatiment::TypeBatiment()
	{
		mId = 0;
		mNom = "";
		mDescription = "";
		mMaxVie = 0;
		mNombreEtats = 0;
		
		mLargeurBatiment = 0;
		mHauteurBatiment = 0;
		mLargeurSprite = 0;
		mHauteurSprite = 0;
		mImageStructure = new sf::Image();
		mSpriteStructure = new sf::Sprite();
		
		mX = 0;
		mY = 0;
		mImageBouton = new sf::Image();
		mSpriteBouton = new sf::Sprite();
		
		mImageFondAvancement = new sf::Image(1, 1, sf::Color(0, 0, 0, 255));
		mSpriteFondAvancement = new sf::Sprite(*mImageFondAvancement);
		
		mImageAvancement = new sf::Image(1, 1, sf::Color(255, 255, 255, 255));
		mSpriteAvancement = new sf::Sprite(*mImageAvancement);
	}
	
	TypeBatiment::~TypeBatiment()
	{
		delete mImageStructure;
		delete mSpriteStructure;
		
		delete mImageBouton;
		delete mSpriteBouton;
		
		delete mImageFondAvancement;
		delete mSpriteFondAvancement;
		
		delete mImageAvancement;
		delete mSpriteAvancement;
	}
	
	void
	TypeBatiment::ChargeFichierStructure(const std::string adresse)
	{
		if (!mImageStructure->LoadFromFile(R_IMAGES_BATIMENTS_STRUCTURES + adresse))
		{
			mSpriteStructure->SetImage(*(new sf::Image(0, 0)));
			CastErreur("Erreur lors du chargement de l'image d'un batiment\n", false);
		}
		else
			mSpriteStructure->SetImage(*mImageStructure);
	}
	
	void
	TypeBatiment::DessineStructure(sf::RenderTarget* const App, const double x, const double y, const double vie, const bool construit)
	{
		unsigned int numEtat;
		
		if (vie >= mMaxVie || construit)
			numEtat = mNombreEtats - 1;
		else
			numEtat = static_cast<unsigned int>( (mNombreEtats - 1) * vie / mMaxVie );
		
		if (numEtat == 0)
		{
			if (Prgm::Batiments::DebutConstruction)
			{
				Prgm::Batiments::DebutConstruction->Resize(mLargeurBatiment, mHauteurBatiment);
				Prgm::Batiments::DebutConstruction->SetPosition(x, y);
				App->Draw(*Prgm::Batiments::DebutConstruction);
			}
			else
				CastErreur("L'image de debut de construction n'a pas put etre chargee");
		}
		else
		{
			numEtat--;
			
			mSpriteStructure->SetSubRect( sf::IntRect( 0, numEtat * mHauteurSprite, mLargeurSprite, (numEtat + 1) * mHauteurSprite) );
			mSpriteStructure->Resize(mLargeurBatiment, mHauteurBatiment);
			mSpriteStructure->SetPosition(x, y);
			App->Draw(*mSpriteStructure);
		}
		
		mSpriteFondAvancement->Resize(mLargeurBatiment, 5);
		mSpriteFondAvancement->SetPosition(x, y + mHauteurBatiment);
		App->Draw(*mSpriteFondAvancement);
		
		if (vie)
		{
			mImageAvancement->SetPixel(0, 0, sf::Color((sf::Uint8)((float)(127 - 127 * (vie / mMaxVie))), (sf::Uint8)((float)(255 * (vie / mMaxVie))), 0, 255) );
			mSpriteAvancement->Resize(vie / mMaxVie * mLargeurBatiment, 5);
			mSpriteAvancement->SetPosition(x, y + mHauteurBatiment);
			App->Draw(*mSpriteAvancement);
		}
	}
	
	void
	TypeBatiment::ChargeFichierBouton(const std::string adresse)
	{
		mImageBouton->LoadFromFile(R_IMAGES_BATIMENTS_BOUTONS + adresse);
		mSpriteBouton->SetImage(*mImageBouton);
	}
	
	void
	TypeBatiment::SetPositionBouton(const double x, const double y)
	{
		mX = x;
		mY = y;
		mSpriteBouton->SetPosition(mX, mY);
	}
	
	void
	TypeBatiment::DessineBouton(sf::RenderTarget* const App)
	{
		mSpriteBouton->SetSubRect( sf::IntRect(0, Prgm::Batiments::CoteSpriteBouton, Prgm::Batiments::CoteSpriteBouton, 2 * Prgm::Batiments::CoteSpriteBouton) );
		mSpriteBouton->Resize(Prgm::Batiments::CoteBouton, Prgm::Batiments::CoteBouton);
		App->Draw(*mSpriteBouton);
	}
	
	bool
	TypeBatiment::Catch(const int x, const int y) const
	{
		return (mX <= x && x <= mX + Prgm::Batiments::CoteBouton && mY <= y && y <= mY + Prgm::Batiments::CoteBouton);
	}
	
	void
	TypeBatiment::AjouteRessourceNecessaire(const unsigned int num, const double quantite)
	{
		unsigned int longueur = mTypesRessourcesNecessaires.size();
		
		mTypesRessourcesNecessaires.resize(longueur + 1);
		mTypesRessourcesNecessaires[longueur] = num;
		
		mQuantitesRessourcesNecessaires.resize(longueur + 1);
		mQuantitesRessourcesNecessaires[longueur] = quantite;
	}
	
	void
	TypeBatiment::AjouteObjetFabricable(const unsigned int num)
	{
		unsigned int longueur = mTypesObjetsFabricables.size(); 
		
		mTypesObjetsFabricables.resize(longueur + 1);
		mTypesObjetsFabricables[longueur] = num;
	}
	
	void
	TypeBatiment::AjouteRessourceTransformable(const unsigned int num)
	{
		unsigned int longueur = mTypesRessourcesTransformables.size(); 
		
		mTypesRessourcesTransformables.resize(longueur + 1);
		mTypesRessourcesTransformables[longueur] = num;
	}
	
	void
	TypeBatiment::SetId(const int num)
	{
		mId = num;
	}
	
	void
	TypeBatiment::SetNom(const std::string nom)
	{
		mNom = nom;
	}
	
	void
	TypeBatiment::SetDescription(const std::string description)
	{
		mDescription = description;
	}
	
	void
	TypeBatiment::SetMaxVie(const double vie)
	{
		mMaxVie = vie;
	}
	
	void
	TypeBatiment::SetLargeurBatiment(const unsigned int largeur)
	{
		mLargeurBatiment = largeur;
	}
	
	void
	TypeBatiment::SetHauteurBatiment(const unsigned int hauteur)
	{
		mHauteurBatiment = hauteur;
	}
	
	void
	TypeBatiment::SetLargeurSprite(const unsigned int largeur)
	{
		mLargeurSprite = largeur;
	}
	
	void
	TypeBatiment::SetHauteurSprite(const unsigned int hauteur)
	{
		mHauteurSprite = hauteur;
	}
	
	void
	TypeBatiment::SetNombreEtats(const unsigned int nombre)
	{
		mNombreEtats = nombre;
	}
	
	int
	TypeBatiment::GetId() const
	{
		return mId;
	}
	
	std::string
	TypeBatiment::GetNom() const
	{
		return mNom;
	}
	
	std::string
	TypeBatiment::GetDescription() const
	{
		return mDescription;
	}
	
	double
	TypeBatiment::GetMaxVie() const
	{
		return mMaxVie;
	}
	
	unsigned int
	TypeBatiment::GetLargeurBatiment() const
	{
		return mLargeurBatiment;
	}
	
	unsigned int
	TypeBatiment::GetHauteurBatiment() const
	{
		return mHauteurBatiment;
	}
	
	unsigned int
	TypeBatiment::GetNombreTypesRessourcesNecessaires() const
	{
		return mTypesRessourcesNecessaires.size();
	}
	
	unsigned int
	TypeBatiment::GetTypeRessourceNecessaire(const unsigned int num) const
	{
		return mTypesRessourcesNecessaires[num];
	}
	
	double
	TypeBatiment::GetQuantiteRessourceNecessaire(const unsigned int num) const
	{
		return mQuantitesRessourcesNecessaires[num];
	}
	
	unsigned int
	TypeBatiment::GetNombreObjetsFabricables() const
	{
		return mTypesObjetsFabricables.size();
	}
	
	unsigned int
	TypeBatiment::GetNumObjetFabricable(const unsigned int indice) const
	{
		return mTypesObjetsFabricables[indice];
	}
	
	unsigned int
	TypeBatiment::GetNombreRessourcesTransformables() const
	{
		return mTypesRessourcesTransformables.size();
	}
	
	unsigned int
	TypeBatiment::GetNumRessourceTransformable(const unsigned int indice) const
	{
		return mTypesRessourcesTransformables[indice];
	}
}

#endif
