/**
* \file typebatiment.hpp
* \brief GS::TypeBatiment
* \author Yoha
* \version alpha-0.0.07
* \date Aot 2009
*
* Appartient au systeme de jeu
*
*/

#ifndef TYPEBATIMENT_HPP
#define TYPEBATIMENT_HPP

/// Mode dynamique
#define SFML_DYNAMIC

#include <SFML/Graphics.hpp>
#include <iostream>
#include <vector>

/// Game System
namespace GS
{
	/// Definit un type de batiment
	class TypeBatiment
	{
		public:
			/// \brief Constructeur
			TypeBatiment();
			/// \brief Destructeur
			~TypeBatiment();
			
			/// \brief Charge le fichier de l'image de la structure du batiment
			/// \brief adresse Adresse du fichier a charger
			void ChargeFichierStructure(const std::string adresse);
			/// \brief Methode de dessin du batiment
			/// \param App Pointeur sur la fenetre de rendu
			/// \param x Abscisse de l'emplacement du batiment
			/// \param y Ordonnee de l'emplacement du batiment
			/// \param vie Vie du batiment a dessiner
			/// \param construit Le batiment est-il termine
			void DessineStructure(sf::RenderTarget* const App, const double x, const double y, const double vie, const bool construit);
			/// \brief Charge le fichier de l'image du bouton du batiment
			/// \brief adresse Adresse du fichier a charger
			void ChargeFichierBouton(const std::string adresse);
			/// \brief Determine la position du bouton sur l'ecran
			/// \param x Abscisse du bouton
			/// \param y Ordonnee du bouton
			void SetPositionBouton(const double x, const double y);
			/// \brief Methode de dessine du bouton
			/// \param App Pointeur sur la fenetre de rendu
			void DessineBouton(sf::RenderTarget* const App);
			/// \brief Capture des clics de la souris
			/// \param x Abscisse du clic
			/// \param y Ordonnee du clic
			bool Catch(const int x, const int y) const;
			
			/// \brief Ajoute une ressource necessaire a la construction
			/// \param num Numero du type de ressource
			/// \param quantite Quantite de ressource necessaire
			void AjouteRessourceNecessaire(const unsigned int num, const double quantite);
			/// \brief Ajoute un objet fabricable
			/// \param num Numero de l'objet
			void AjouteObjetFabricable(const unsigned int num);
			/// \brief Ajoute une ressource transformable
			/// \param num Numero de la ressource
			void AjouteRessourceTransformable(const unsigned int num);
			
			/// \brief Accesseur: definit l'Id du type de batiment
			/// \param num Numero a charger
			void SetId(const int num);
			/// \brief Accesseur: definit le nom du batiment
			/// \param nom Nom a charger
			void SetNom(const std::string nom);
			/// \brief Accesseur: definit la description du batiment
			/// \param description Description a charger
			void SetDescription(const std::string description);
			/// \brief Accesseur: definit la vie maximale du batiment
			/// \param vie Maximum de vie a charger
			void SetMaxVie(const double vie);
			/// \brief Accesseur: definit la largeur du batiment
			/// \param largeur Nouvelle largeur
			void SetLargeurBatiment(const unsigned int largeur);
			/// \brief Accesseur: definit la hauteur du batiment
			/// \param hauteur Nouvelle hauteur
			void SetHauteurBatiment(const unsigned int hauteur);
			/// \brief Accesseur: definit la largeur du sprite du batiment
			/// \param largeur Nouvelle largeur
			void SetLargeurSprite(const unsigned int largeur);
			/// \brief Accesseur: definit la hauteur du sprite du batiment
			/// \param hauteur Nouvelle hauteur
			void SetHauteurSprite(const unsigned int hauteur);
			/// \brief Accesseur: definit le nombre d'etat
			/// \param nombre Nouveau nombre d'etats
			void SetNombreEtats(const unsigned int nombre);
			
			/// \brief Accesseur: renvoie l'Id du type du type de batiment
			int GetId() const;
			/// \brief Accesseur: renvoie le nom du batiment
			std::string GetNom() const;
			/// \brief Accesseur: renvoie la description du batiment
			std::string GetDescription() const;
			/// \brief Accesseur: renvoie le maximum de vie du batiment
			double GetMaxVie() const;
			/// \brief Accesseur: renvoie la largeur du batiment
			unsigned int GetLargeurBatiment() const;
			/// \brief Accesseur: renvoie la hauteur du batiment
			unsigned int GetHauteurBatiment() const;
			
			/// \brief Accesseur: renvoie le nombre de types de ressources necessaires
			unsigned int GetNombreTypesRessourcesNecessaires() const;
			/// \brief Accesseur: renvoie le numero du type de ressource correspondant
			/// \param num Numero dans le tableau
			unsigned int GetTypeRessourceNecessaire(const unsigned int num) const;
			/// \brief Accesseur: renvoie la quantite de ressource necessaire
			/// \param num Numero dans le tableau
			double GetQuantiteRessourceNecessaire(const unsigned int num) const;
			
			/// \brief Accesseur: renvoie le nombre d'objets fabricables
			unsigned int GetNombreObjetsFabricables() const;
			/// \brief Accesseur: renvoie l'Id de l'objet fabricable
			/// \param indice Indice de l'objet fabricable dans le tableau
			unsigned int GetNumObjetFabricable(const unsigned int indice) const;
			/// \brief Accesseur: renvoie le nombre de ressources transformables
			unsigned int GetNombreRessourcesTransformables() const;
			/// \brief Accesseur: renvoie l'Id de la ressource transformable
			/// \param indice Indice de la ressource transformable dans le tableau
			unsigned int GetNumRessourceTransformable(const unsigned int indice) const;
			
		private:
			/// ID du type de batiment
			int mId;
			/// Nom du batiment
			std::string mNom;
			/// Courte description du batiment
			std::string mDescription;
			/// Maximum de vie du batiment
			double mMaxVie;
			/// Nombre d'etapes de construction + etat utilise
			unsigned int mNombreEtats;
			/// Types des ressources necessaires a la construction
			std::vector <unsigned int> mTypesRessourcesNecessaires;
			/// Quantites correspondantes
			std::vector <double> mQuantitesRessourcesNecessaires;
			/// Types d'objets fabricables
			std::vector <unsigned int> mTypesObjetsFabricables;
			/// Types de ressources transformatbles
			std::vector <unsigned int> mTypesRessourcesTransformables;
			
			/// Largeur du batiment
			unsigned int mLargeurBatiment;
			/// Hauteur du batiment
			unsigned int mHauteurBatiment;
			/// Largeur du sprite du batiment
			unsigned int mLargeurSprite;
			/// Hauteur du sprite du batiment
			unsigned int mHauteurSprite;
			/// Image de la structure du batiment
			sf::Image* mImageStructure;
			/// Afficheur du batiment
			sf::Sprite* mSpriteStructure;
			
			/// Abscisse du bouton
			double mX;
			/// Ordonnee du bouton
			double mY;
			/// Image du bouton du batiment
			sf::Image* mImageBouton;
			/// Afficheur du bouton
			sf::Sprite* mSpriteBouton;
			
			/// Image du fond d'avancement
			sf::Image* mImageFondAvancement;
			/// Afficheur du fond d'avancement
			sf::Sprite* mSpriteFondAvancement;
			
			/// Image de l'avancement
			sf::Image* mImageAvancement;
			/// Afficheur de l'avancement
			sf::Sprite* mSpriteAvancement;
	};
}

#endif
