#ifndef TYPEOBJET_CPP
#define TYPEOBJET_CPP

#include "typeobjet.hpp"

#include "projet.hpp"
#include "data.hpp"

namespace GS
{
	TypeObjet::TypeObjet()
	{
		mNom = "";
		mDescription = "";
		mDureeFabrication = 0;
		mCategorieObjet = NULL;
		
		mImageObjet = new sf::Image();
		mSpriteObjet = new sf::Sprite();
	}
	
	TypeObjet::~TypeObjet()
	{
		delete mImageObjet;
		delete mSpriteObjet;
	}
	
	void
	TypeObjet::Dessine(sf::RenderTarget* const App, const double x, const double y)
	{
		if (mSpriteObjet)
		{
			mSpriteObjet->SetPosition(x, y);
			App->Draw(*mSpriteObjet);
		}
	}
	
	void
	TypeObjet::ChargeImage(const std::string nom)
	{
		mImageObjet->LoadFromFile(R_IMAGES_OBJETS + nom);
		mSpriteObjet->SetImage(*mImageObjet);
		mSpriteObjet->Resize(Prgm::Objets::CoteIcone, Prgm::Objets::CoteIcone);
	}
	
	bool
	TypeObjet::Catch(const int x, const int y, const double posX, const double posY)
	{
		return mSpriteObjet
				&& posX <= x && x <= posX + mSpriteObjet->GetSize().x
				&& posY <= y && y <= posY + mSpriteObjet->GetSize().y;
	}
	
	void
	TypeObjet::AjouteRessourceNecessaire(const unsigned int num, const double quantite)
	{
		unsigned int longueur = mTypesRessourcesNecessaires.size();
		
		mTypesRessourcesNecessaires.resize(longueur + 1);
		mTypesRessourcesNecessaires[longueur] = num;
		
		mQuantitesRessourcesNecessaires.resize(longueur + 1);
		mQuantitesRessourcesNecessaires[longueur] = quantite;
	}
	
	void
	TypeObjet::SetId(const int num)
	{
		mId = num;
	}
	
	void
	TypeObjet::SetNom(const std::string nom)
	{
		mNom = nom;
	}
	
	void
	TypeObjet::SetDescription(const std::string description)
	{
		mDescription = description;
	}
	
	void
	TypeObjet::SetDureeFabrication(const double duree)
	{
		mDureeFabrication = duree;
	}
	
	void
	TypeObjet::SetCategorieObjet(CategorieObjet* categorie)
	{
		mCategorieObjet = categorie;
	}
	
	unsigned int
	TypeObjet::GetId() const
	{
		return mId;
	}
	
	std::string
	TypeObjet::GetNom() const
	{
		return mNom;
	}
	
	std::string
	TypeObjet::GetDescription() const
	{
		return mDescription;
	}
	
	double
	TypeObjet::GetDureeFabrication() const
	{
		return mDureeFabrication;
	}
	
	CategorieObjet*
	TypeObjet::GetCategorieObjet() const
	{
		return mCategorieObjet;
	}
	
	unsigned int
	TypeObjet::GetNombreTypesRessourcesNecessaires() const
	{
		return mTypesRessourcesNecessaires.size();
	}
	
	unsigned int
	TypeObjet::GetTypeRessourceNecessaire(const unsigned int num) const
	{
		return mTypesRessourcesNecessaires[num];
	}
	
	double
	TypeObjet::GetQuantiteRessourceNecessaire(const unsigned int num) const
	{
		return mQuantitesRessourcesNecessaires[num];
	}
}

#endif
