/**
* \file typeobjet.hpp
* \brief GS::TypeObjet
* \author Yoha
* \version alpha-0.0.07
* \date Aot 2009
*
* Appartient au systeme de jeu
*
*/

#ifndef TYPEOBJET_HPP
#define TYPEOBJET_HPP

/// Mode dynamique
#define SFML_DYNAMIC

#include <SFML/Graphics.hpp>
#include <iostream>

#include "categorieobjet.hpp"

/// Game System
namespace GS
{
	/// Definit un type d'objet
	class TypeObjet
	{
		public:
			/// \brief Constructeur
			TypeObjet();
			/// \brief Destructeur
			~TypeObjet();
			
			/// \brief Methode de dessin de l'objet
			/// \param App Pointeur sur la fenetre de rendu
			/// \param x Abscisse de l'emplacement de l'objet
			/// \param y Ordonnee de l'emplacement de l'objet
			void Dessine(sf::RenderTarget* const App, const double x, const double y);
			/// \brief Charge l'image de l'objet
			/// \param nom Nom du fichier a charger
			void ChargeImage(const std::string nom);
			/// \brief Capture des clics de la souris
			/// \param x Abscisse du clic
			/// \param y Ordonnee du clic
			/// \param posX Abscisse du sprite
			/// \param posY Ordonnee du sprite
			bool Catch(const int x, const int y, const double posX, const double posY);
			/// \brief Ajoute une ressource necessaire a la fabrication
			/// \param num Numero du type de ressource
			/// \param quantite Quantite de ressource necessaire
			void AjouteRessourceNecessaire(const unsigned int num, const double quantite);
			
			/// \brief Accesseur: definit l'Id du type d'objet
			/// \param num Numero a charger
			void SetId(const int num);
			/// \brief Accesseur: definit le nom de l'objet
			/// \param nom Nom a charger
			void SetNom(const std::string nom);
			/// \brief Accesseur: definit la description de l'objet
			/// \param description Description a charger
			void SetDescription(const std::string description);
			/// \brief Accesseur: definit la duree de fabrication
			/// \param duree Duree en secondes
			void SetDureeFabrication(const double duree);
			/// \brief Accesseur: definit la categorie de ce type d'objet
			/// \param categorie Categorie a charger
			void SetCategorieObjet(CategorieObjet* categorie);
			
			/// \brief Accesseur: renvoie l'Id du type d'objet
			unsigned int GetId() const;
			/// \brief Accesseur: renvoie le nom de l'objet
			std::string GetNom() const;
			/// \brief Accesseur: renvoie la description de l'objet
			std::string GetDescription() const;
			/// \brief Accesseur: renvoie la duree de fabrication de l'objet (en secondes)
			double GetDureeFabrication() const;
			/// \brief Accesseur: renvoie la categorie de l'objet
			CategorieObjet* GetCategorieObjet() const;
			
			/// \brief Accesseur: renvoie le nombre de types de ressources necessaires
			unsigned int GetNombreTypesRessourcesNecessaires() const;
			/// \brief Accesseur: renvoie le numero du type de ressource correspondant
			/// \param num Numero dans le tableau
			unsigned int GetTypeRessourceNecessaire(const unsigned int num) const;
			/// \brief Accesseur: renvoie la quantite de ressource necessaire
			/// \param num Numero dans le tableau
			double GetQuantiteRessourceNecessaire(const unsigned int num) const;
		
		private:
			/// Id du type d'objet
			unsigned int mId;
			/// Nom de l'objet
			std::string mNom;
			/// Courte description de l'objet
			std::string mDescription;
			/// Duree de fabrication de l'objet en secondes
			double mDureeFabrication;
			/// Categorie de l'objet
			CategorieObjet* mCategorieObjet;
			
			/// Types des ressources necessaires a la fabrication
			std::vector <unsigned int> mTypesRessourcesNecessaires;
			/// Quantites correspondantes
			std::vector <double> mQuantitesRessourcesNecessaires;
			
			/// Image de l'icone
			sf::Image* mImageObjet;
			/// Sprite d'affichage de l'icone
			sf::Sprite* mSpriteObjet;
	};
}

#endif
