#ifndef TYPERESSOURCE_CPP
#define TYPERESSOURCE_CPP

#include "typeressource.hpp"

#include "projet.hpp"
#include "data.hpp"

namespace GS
{
	TypeRessource::TypeRessource()
	{
		mId = 0;
		mNom = "";
		mMasseUnitaire = 0;
		mVitesseTransformation = 0;
		
		mImageRessource = new sf::Image();
		mSpriteRessource = new sf::Sprite();
	}
	
	TypeRessource::~TypeRessource()
	{
		delete mImageRessource;
		delete mSpriteRessource;
	}
	
	void
	TypeRessource::Dessine(sf::RenderTarget* const App, const double x, const double y)
	{
		if (mSpriteRessource)
		{
			mSpriteRessource->SetPosition(x, y);
			App->Draw(*mSpriteRessource);
		}
	}
	
	void
	TypeRessource::ChargeImage(const std::string nom)
	{
		mImageRessource->LoadFromFile(R_IMAGES_RESSOURCES + nom);
		mSpriteRessource->SetImage(*mImageRessource);
		mSpriteRessource->Resize(Prgm::Ressources::CoteIcone, Prgm::Ressources::CoteIcone);
	}
	
	bool
	TypeRessource::Catch(const int x, const int y, const double posX, const double posY)
	{
		return mSpriteRessource
				&& posX <= x && x <= posX + mSpriteRessource->GetSize().x
				&& posY <= y && y <= posY + mSpriteRessource->GetSize().y;
	}
	
	void
	TypeRessource::AjouteRessourceNecessaire(const unsigned int num, const double quantite)
	{
		unsigned int longueur = mTypesRessourcesNecessaires.size();
		
		mTypesRessourcesNecessaires.resize(longueur + 1);
		mTypesRessourcesNecessaires[longueur] = num;
		
		mQuantitesRessourcesNecessaires.resize(longueur + 1);
		mQuantitesRessourcesNecessaires[longueur] = quantite;
	}
	
	void
	TypeRessource::SetId(const int num)
	{
		mId = num;
	}
	
	void
	TypeRessource::SetNom(const std::string nom)
	{
		mNom = nom;
	}
	
	void
	TypeRessource::SetMasseUnitaire(const double masse)
	{
		mMasseUnitaire = masse;
	}
	
	void
	TypeRessource::SetVitesseTransformation(const double duree)
	{
		mVitesseTransformation = duree;
	}
	
	int
	TypeRessource::GetId() const
	{
		return mId;
	}
	
	std::string
	TypeRessource::GetNom() const
	{
		return mNom;
	}
	
	double
	TypeRessource::GetMasseUnitaire() const
	{
		return mMasseUnitaire;
	}
	
	double
	TypeRessource::GetVitesseTransformation() const
	{
		return mVitesseTransformation;
	}
	
	unsigned int
	TypeRessource::GetNombreTypesRessourcesNecessaires() const
	{
		return mTypesRessourcesNecessaires.size();
	}
	
	unsigned int
	TypeRessource::GetTypeRessourceNecessaire(const unsigned int num) const
	{
		return mTypesRessourcesNecessaires[num];
	}
	
	double
	TypeRessource::GetQuantiteRessourceNecessaire(const unsigned int num) const
	{
		return mQuantitesRessourcesNecessaires[num];
	}
}

#endif
