/**
* \file typeressource.hpp
* \brief GS::TypeRessource
* \author Yoha
* \version alpha-0.0.07
* \date Aot 2009
*
* Appartient au systeme de jeu
*
*/

#ifndef TYPERESSOURCE_HPP
#define TYPERESSOURCE_HPP

/// Mode dynamique
#define SFML_DYNAMIC

#include <SFML/Graphics.hpp>
#include <iostream>
#include <vector>

/// Game System
namespace GS
{
	/// Definit un type de ressource
	class TypeRessource
	{
		public:
			/// \brief Constructeur
			TypeRessource();
			/// \brief Destructeur
			~TypeRessource();
			
			/// \brief Methode de dessin de la ressource
			/// \param App Pointeur sur la fenetre de rendu
			/// \param x Abscisse de l'emplacement de la ressource
			/// \param y Ordonnee de l'emplacement de la ressource
			void Dessine(sf::RenderTarget* const App, const double x, const double y);
			/// \brief Charge l'image de la ressourcce
			/// \param nom Nom du fichier a charger
			void ChargeImage(const std::string nom);
			/// \brief Capture des clics de la souris
			/// \param x Abscisse du clic
			/// \param y Ordonnee du clic
			/// \param posX Abscisse du sprite
			/// \param posY Ordonnee du sprite
			bool Catch(const int x, const int y, const double posX, const double posY);
			/// \brief Ajoute une ressource necessaire a la transformation
			/// \param num Numero du type de ressource
			/// \param quantite Quantite de ressource necessaire
			void AjouteRessourceNecessaire(const unsigned int num, const double quantite);
			
			/// \brief Accesseur: definit l'ID de la ressource
			/// \param num Numero a charger
			void SetId(const int num);
			/// \brief Accesseur: definit le nom de la ressource
			/// \param nom Nom a charger
			void SetNom(const std::string nom);
			/// \brief Accesseur: definit la masse unitaire de la ressource
			/// \param masse Masse a chcarger
			void SetMasseUnitaire(const double masse);
			/// \brief Accesseur: definit la vitesse de transformation
			/// \param duree Vitesse en unites par seconde
			void SetVitesseTransformation(const double duree);
			
			/// \brief Acesseur: renvoie l'ID de la ressource
			int GetId() const;
			/// \brief Accesseur: renvoie le nom de la ressource
			std::string GetNom() const;
			/// \brief Acesseur: renvoie la masse unitaire de la ressource
			double GetMasseUnitaire() const;
			/// \brief Accesseur: renvoie la vitesse de transformation
			double GetVitesseTransformation() const;
			
			/// \brief Accesseur: renvoie le nombre de types de ressources necessaires
			unsigned int GetNombreTypesRessourcesNecessaires() const;
			/// \brief Accesseur: renvoie le numero du type de ressource correspondant
			/// \param num Numero dans le tableau
			unsigned int GetTypeRessourceNecessaire(const unsigned int num) const;
			/// \brief Accesseur: renvoie la quantite de ressource necessaire
			/// \param num Numero dans le tableau
			double GetQuantiteRessourceNecessaire(const unsigned int num) const;
			
		private:
			/// ID du type de ressource
			int mId;
			/// Nom de la ressource
			std::string mNom;
			/// Masse unutaire de la ressource
			double mMasseUnitaire;
			/// Duree de transformation de la ressource en secondes par unite
			double mVitesseTransformation;
			
			/// Types des ressources necessaires a la transformation
			std::vector <unsigned int> mTypesRessourcesNecessaires;
			/// Quantites correspondantes
			std::vector <double> mQuantitesRessourcesNecessaires;
			
			/// Image de l'icone
			sf::Image* mImageRessource;
			/// Sprite d'affichage de l'icone
			sf::Sprite* mSpriteRessource;
	};
}

#endif
