/**
* \file typeterrain.hpp
* \brief GS::TypeTerrain
* \author Yoha
* \version alpha-0.0.07
* \date Juin 2009
*
* Appartient au systeme de jeu
*
*/

#ifndef TYPETERRAIN_HPP
#define TYPETERRAIN_HPP

/// Mode dynamique
#define SFML_DYNAMIC

#include <SFML/Graphics.hpp>

/// Game System
namespace GS
{
	/// Definit un type de terrain
	class TypeTerrain
	{
		public:
			/// \brief Constructeur
			TypeTerrain();
			/// \brief Destructeur
			~TypeTerrain();
			
			/// \brief Teste le pointeur du sprite
			bool TestSprite();
			
			/// \brief Accesseur: definit le sprite
			/// \param sp Sprite a charger
			void SetSprite(const sf::Sprite* const sp);
			/// \brief Accesseur: definit la possibilite du terrain de porter des ressources de terrain
			/// \param porte Possibilite de porter des ressource de terrain
			void SetPorteRessources(const bool porte);
			/// \brief Accesseur: definit le facteur de vitesse sur ce terrain
			/// \param vitesse Vitesse de deplacement
			void SetFacteurVitesse(const double vitesse);
			/// \brief Accesseur: definit la possibilite de construire un batiment terrestre
			/// \param constructible Possibilite de construction
			void SetConstructible(const bool constructible);
			/// \brief Accesseur: definit la probabilite d'apparition du type de terrain
			/// \param probabilite Probabilite d'apparition
			void SetProbabilite(const double probabilite);
			
			/// \brief Accesseur: renvoie le sprite
			sf::Sprite* GetSprite() const;
			/// \brief Accesseur: renvoie la possibilite du terrain de porter des ressources de terrain
			bool GetPorteRessources() const;
			/// \brief Accesseur: renvoie la vitesse de deplacement sur ce terrain
			double GetFacteurVitesse() const;
			/// \brief Accesseur: renvoie la possibilite de construire sur ce type de terrain un batiment terrestre
			bool GetConstructible() const;
			/// \brief Accesseur: renvoie la probabilite d'apparition du type de terrain
			double GetProbabilite() const;
			
		private:
			/// Probabilite d'apparition du type de terrain
			double mProbabilite;
			/// Vitesse de deplacement
			double mFacteurVitesse;
			/// Le terrain est-il constructible
			bool mConstructible;
			/// Le terrain peut-il porter des ressources de terrain
			bool mPorteRessources;
			/// La presence d'un sprite a-t-elle deja ete teste
			bool mSpriteTeste;
			/// Sprite du terrain
			sf::Sprite* mSprite;
	};
}

#endif
