/**
* \file typeterressource.hpp
* \brief GS::TypeTerRessource
* \author Yoha
* \version alpha-0.0.07
* \date Aot 2009
*
* Appartient au systeme de jeu
*
*/

#ifndef TYPETERRESSOURCE_HPP
#define TYPETERRESSOURCE_HPP

/// Mode dynamique
#define SFML_DYNAMIC

#include <SFML/Graphics.hpp>
#include <iostream>

#include "typeressource.hpp"

/// Game System
namespace GS
{
	/// Definit un type de ressource de terrain
	class TypeTerRessource
	{
		public:
			/// \brief Constructeur
			TypeTerRessource();
			/// \brief Destructeur
			~TypeTerRessource();
			
			/// \brief Methode de dessin de la ressource de terrain
			/// \param App Pointeur sur la fenetre de rendu
			/// \param x Abscisse de l'emplacement de la ressource de terrain
			/// \param y Ordonnee de l'emplacement de la ressource de terrain
			void Dessine(sf::RenderTarget* const App, const double x, const double y);
			/// \brief Teste le pointeur du sprite
			bool TestSprite();
			
			/// \brief Accesseur: definit l'Id de la ressource de terrain
			/// \param num Numero a charger
			void SetId(const int num);
			/// \brief Acesseur: definit le sprite
			/// \param sprite Sprite a charger
			void SetSprite(const sf::Sprite* const sprite);
			/// \brief Accesseur: definit le nom
			/// \param nom Nom a charger
			void SetNom(const std::string nom);
			/// \brief Accesseur: definit le nom de l'action
			/// \param action Nom a charger
			void SetAction(const std::string action);
			/// \brief Accesseur: definit le type de ressource contenue
			/// \param tr Pointeur sur la ressource
			void SetTypeRessource(TypeRessource* tr);
			/// \brief Accesseur: definit la vitesse de recolte
			/// \param vitesse Vitesse a charger
			void SetRessourceParSeconde(const double vitesse);
			
			/// \brief Accesseur: renvoie l'Id du type de ressource de terrain
			int GetId() const;
			/// \brief Accesseur: renvoie l'ID du type de la ressource contenue
			int GetResId() const;
			/// \brief Accesseur: renvoie le type de la ressource contenu
			TypeRessource* GetTypeRessource() const;
			/// \brief Accesseur: renvoie les nom de l'action
			std::string GetAction() const;
			/// \brief Accesseur: renvoie la vitesse de recolte
			double GetRessourceParSeconde() const;
			
		private:
			/// Id du type de ressource de terrain
			int mId;
			/// Le sprite a-t-il ete teste
			bool mSpriteTeste;
			/// Vitesse de recolte
			double mRessourceParSeconde;
			/// Nom de la ressource de terrain
			std::string mNom;
			/// Nom de l'action envisageable
			std::string mAction;
			/// Sprite d'affichage
			sf::Sprite* mSprite;
			/// Type de ressource contenue
			TypeRessource* mTypeRessource;
	};
}

#endif
