/**
* \file fonctions.hpp
* \brief Fonctions diverses
* \author Yoha
* \version alpha-0.0.07
* \date Aot 2009
*
* Definition de fonctions diverses
*
*/

#ifndef FONCTIONS_HPP
#define FONCTIONS_HPP

/// Mode dynamique
#define SFML_DYNAMIC

#include "lunar.hpp"
#include "tinyxml/tinyxml.h"

/// \brief Calcul de distance entre deux points
/// \param x1 Abscisse point 1
/// \param y1 Ordonnee point 1
/// \param x2 Abscisse point 2
/// \param y2 Ordonnee point 2
double CalculeDistance(const double x1, const double y1, const double x2, const double y2);
/// \brief Permet de convertir les quantites de ressources en entiers exploitables
/// \param quantite Double a convertir
double NormaliseRessource(const double quantite);

/// \brief Prechargement du lua_State
/// \param pointeur Pointeur sur le pointeur de lua_State
void ChargeLua(lua_State** pointeur);

/// \brief Transmet une erreur pour traitement
/// \param msg Contient le message d'erreur
/// \param fatal Peut-on continuer le programme malgre l'erreur, par defaut, le programme doit etre arrete
void CastErreur(const std::string msg, const bool fatal = true);
/// \brief Permet notamment le debuggage en testant l'execution de certains points
void SetBreak();


/// \brief Retourne un entier non signe
/// \param parent Noeud contenant le parametre a recuperer
/// \param parametre Nom du parametre
/// \param retour Reference sur la variable de retour
bool ChargeXmlParametre(TiXmlElement* const parent, const char* parametre, unsigned int& retour);
/// \brief Charge une valeur XML de type entier relatif
/// \param parent Noeud contenent la valeur a chercher
/// \param variable Nom de la variable
/// \param retour Reference sur la variable de retour
bool ChargeXmlValeur(TiXmlNode* const parent, const char* variable, bool& retour);
/// \brief Charge une valeur XML de type entier relatif
/// \param parent Noeud contenent la valeur a chercher
/// \param variable Nom de la variable
/// \param retour Reference sur la variable de retour
bool ChargeXmlValeur(TiXmlNode* const parent, const char* variable, int& retour);
/// \brief Charge une valeur XML de type entier naturel
/// \param parent Noeud contenent la valeur a chercher
/// \param variable Nom de la variable
/// \param retour Reference sur la variable de retour
bool ChargeXmlValeur(TiXmlNode* const parent, const char* variable, unsigned int& retour);
/// \brief Charge une valeur XML de type double
/// \param parent Noeud contenent la valeur a chercher
/// \param variable Nom de la variable
/// \param retour Reference sur la variable de retour
bool ChargeXmlValeur(TiXmlNode* const parent, const char* variable, double& retour);
/// \brief Charge une valeur XML de type chaine
/// \param parent Noeud contenent la valeur a chercher
/// \param variable Nom de la variable
/// \param retour Reference sur la variable de retour
bool ChargeXmlValeur(TiXmlNode* const parent, const char* variable, std::string& retour);

#endif
